@@ -10,6 +10,7 @@
 
 #include "kstars_ui_tests.h"
 #include "ui_calibrationoptions.h"
+#include "indicom.h"
 #include "indi/guimanager.h"
 #include "Options.h"
 #include "skymapcomposite.h"
@@ -20,12 +21,14 @@
 #define SHUTTER_YES      1
 
 
-TestEkosCaptureWorkflow::TestEkosCaptureWorkflow(QObject *parent) : TestEkosCaptureWorkflow::TestEkosCaptureWorkflow("Internal", parent){}
+TestEkosCaptureWorkflow::TestEkosCaptureWorkflow(QObject *parent) :
+    TestEkosCaptureWorkflow::TestEkosCaptureWorkflow("Internal", parent) {}
 
 TestEkosCaptureWorkflow::TestEkosCaptureWorkflow(QString guider, QObject *parent) : QObject(parent)
 {
     m_CaptureHelper = new TestEkosCaptureHelper(guider);
-    m_CaptureHelper->m_GuiderDevice = "Guide Simulator";
+    m_CaptureHelper->m_GuiderDevice = "CCD Simulator";
+    m_CaptureHelper->m_FocuserDevice = "Focuser Simulator";
 }
 
 bool TestEkosCaptureWorkflow::executeFocusing()
@@ -59,7 +62,8 @@ void TestEkosCaptureWorkflow::testCaptureRefocus()
     m_CaptureHelper->expectedCaptureStates.append(Ekos::CAPTURE_FOCUSING);
     KTRY_CLICK(capture, startB);
     // focusing must have started 60 secs + exposure time + 10 secs delay
-    KVERIFY_EMPTY_QUEUE_WITH_TIMEOUT(m_CaptureHelper->expectedCaptureStates, 60000 + 10000 + 1000*capture->captureExposureN->value());
+    KVERIFY_EMPTY_QUEUE_WITH_TIMEOUT(m_CaptureHelper->expectedCaptureStates,
+                                     60000 + 10000 + 1000 * capture->captureExposureN->value());
 }
 
 void TestEkosCaptureWorkflow::testCaptureRefocusAbort()
@@ -78,7 +82,8 @@ void TestEkosCaptureWorkflow::testCaptureRefocusAbort()
     m_CaptureHelper->expectedFocusStates.append(Ekos::FOCUS_PROGRESS);
     KTRY_CLICK(capture, startB);
     // focusing must have started 60 secs + exposure time + 10 secs delay
-    KVERIFY_EMPTY_QUEUE_WITH_TIMEOUT(m_CaptureHelper->expectedFocusStates, 60000 + 10000 + 1000*capture->captureExposureN->value());
+    KVERIFY_EMPTY_QUEUE_WITH_TIMEOUT(m_CaptureHelper->expectedFocusStates,
+                                     60000 + 10000 + 1000 * capture->captureExposureN->value());
     // now abort capturing
     m_CaptureHelper->expectedFocusStates.append(Ekos::FOCUS_ABORTED);
     capture->abort();
@@ -170,12 +175,22 @@ void TestEkosCaptureWorkflow::testGuidingDeviationSuspendingCapture()
     KTRY_CAPTURE_ADD_LIGHT(30.0, 1, 5.0, "Green", imagepath);
     KTRY_CAPTURE_ADD_LIGHT(30.0, 1, 5.0, "Blue", imagepath);
 
-    // set Dubhe as target and slew there
-    SkyObject *target = KStars::Instance()->data()->skyComposite()->findByName("Dubhe");
+    // set a position in the west
+    SkyPoint *target = new SkyPoint();
+    target->setAz(270.0);
+    target->setAlt(KStarsData::Instance()->geo()->lat()->Degrees() / 2.0);
+    // translate to equatorial coordinates
+    const dms lst = KStarsData::Instance()->geo()->GSTtoLST(KStarsData::Instance()->clock()->utc().gst());
+    const dms *lat = KStarsData::Instance()->geo()->lat();
+    target->HorizontalToEquatorial(&lst, lat);
+
     m_CaptureHelper->slewTo(target->ra().Hours(), target->dec().Degrees(), true);
 
+    // clear calibration to ensure proper guiding
+    KTRY_CLICK(Ekos::Manager::Instance()->guideModule(), clearCalibrationB);
+
     // start guiding
-    m_CaptureHelper->startGuiding(1.0);
+    m_CaptureHelper->startGuiding(2.0);
 
     // start capture
     m_CaptureHelper->expectedCaptureStates.append(Ekos::CAPTURE_CAPTURING);
@@ -186,12 +201,12 @@ void TestEkosCaptureWorkflow::testGuidingDeviationSuspendingCapture()
     QTest::qWait(2000);
     // create a guide drift
     m_CaptureHelper->expectedCaptureStates.append(Ekos::CAPTURE_SUSPENDED);
-    Ekos::Manager::Instance()->mountModule()->doPulse(RA_INC_DIR, 2000, DEC_INC_DIR, 2000);
-    qCInfo(KSTARS_EKOS_TEST()) << "Sent 2000ms RA+DEC guiding pulses.";
+    Ekos::Manager::Instance()->mountModule()->doPulse(RA_INC_DIR, 1000, DEC_INC_DIR, 1000);
+    qCInfo(KSTARS_EKOS_TEST()) << "Sent 1000ms RA+DEC guiding pulses.";
     KTRY_SWITCH_TO_MODULE_WITH_TIMEOUT(capture, 1000);
     KVERIFY_EMPTY_QUEUE_WITH_TIMEOUT(m_CaptureHelper->expectedCaptureStates, 30000);
     // expect that capturing continues
-    QTRY_VERIFY_WITH_TIMEOUT(m_CaptureHelper->getCaptureStatus() == Ekos::CAPTURE_CAPTURING, 30000);
+    QTRY_VERIFY_WITH_TIMEOUT(m_CaptureHelper->getCaptureStatus() == Ekos::CAPTURE_CAPTURING, 60000);
     // verify that capture starts only once
     m_CaptureHelper->expectedCaptureStates.append(Ekos::CAPTURE_PROGRESS);
     QTest::qWait(20000);
@@ -219,8 +234,11 @@ void TestEkosCaptureWorkflow::testGuidingDeviationAbortCapture()
     SkyObject *target = KStars::Instance()->data()->skyComposite()->findByName("Dubhe");
     m_CaptureHelper->slewTo(target->ra().Hours(), target->dec().Degrees(), true);
 
+    // clear calibration to ensure proper guiding
+    KTRY_CLICK(Ekos::Manager::Instance()->guideModule(), clearCalibrationB);
+
     // start guiding
-    m_CaptureHelper->startGuiding(1.0);
+    m_CaptureHelper->startGuiding(2.0);
 
     // start capture
     m_CaptureHelper->expectedCaptureStates.append(Ekos::CAPTURE_CAPTURING);
@@ -244,7 +262,7 @@ void TestEkosCaptureWorkflow::testGuidingDeviationAbortCapture()
     // wait that the guiding deviation is below the limit and
     // verify that capture does not start
     m_CaptureHelper->expectedCaptureStates.append(Ekos::CAPTURE_PROGRESS);
-    QTRY_VERIFY_WITH_TIMEOUT(m_CaptureHelper->getGuideDeviation() < deviation_limit, 20000);
+    QTRY_VERIFY_WITH_TIMEOUT(m_CaptureHelper->getGuideDeviation() < deviation_limit, 60000);
 
     QTest::qWait(20000);
     QVERIFY2(m_CaptureHelper->expectedCaptureStates.size() > 0, "Capture has been restarted although aborted.");
@@ -332,7 +350,8 @@ void TestEkosCaptureWorkflow::testCaptureWaitingForTemperature()
     KVERIFY_EMPTY_QUEUE_WITH_TIMEOUT(m_CaptureHelper->expectedCaptureStates, 60000);
     // check if the temperature is at the expected level
     QTRY_VERIFY2(std::abs(capture->temperatureOUT->text().toDouble() - targetTemp) <= Options::maxTemperatureDiff(),
-                 QString("Temperature %1°C not at the expected level of %2°C").arg(capture->temperatureOUT->text()).arg(targetTemp).toLocal8Bit());
+                 QString("Temperature %1°C not at the expected level of %2°C").arg(capture->temperatureOUT->text()).arg(
+                     targetTemp).toLocal8Bit());
 
     // stop capturing
     m_CaptureHelper->expectedCaptureStates.append(Ekos::CAPTURE_ABORTED);
@@ -347,7 +366,8 @@ void TestEkosCaptureWorkflow::testCaptureWaitingForTemperature()
     KVERIFY_EMPTY_QUEUE_WITH_TIMEOUT(m_CaptureHelper->expectedCaptureStates, 20000);
     // check if the temperature is at the expected level
     QTRY_VERIFY2(std::abs(capture->temperatureOUT->text().toDouble() - targetTemp) <= Options::maxTemperatureDiff(),
-                 QString("Temperature %1°C not at the expected level of %2°C").arg(capture->temperatureOUT->text()).arg(targetTemp).toLocal8Bit());
+                 QString("Temperature %1°C not at the expected level of %2°C").arg(capture->temperatureOUT->text()).arg(
+                     targetTemp).toLocal8Bit());
 
     // stop capturing
     m_CaptureHelper->expectedCaptureStates.append(Ekos::CAPTURE_ABORTED);
@@ -366,7 +386,8 @@ void TestEkosCaptureWorkflow::testCaptureWaitingForTemperature()
     KVERIFY_EMPTY_QUEUE_WITH_TIMEOUT(m_CaptureHelper->expectedCaptureStates, 60000);
     // check if the temperature is at the expected level
     QTRY_VERIFY2(std::abs(capture->temperatureOUT->text().toDouble() - targetTemp) <= Options::maxTemperatureDiff(),
-                 QString("Temperature %1°C not at the expected level of %2°C").arg(capture->temperatureOUT->text()).arg(targetTemp).toLocal8Bit());
+                 QString("Temperature %1°C not at the expected level of %2°C").arg(capture->temperatureOUT->text()).arg(
+                     targetTemp).toLocal8Bit());
 
 }
 
@@ -385,17 +406,19 @@ void TestEkosCaptureWorkflow::testCaptureWaitingForRotator()
     // open the rotator dialog
     KTRY_CLICK(capture, rotatorB);
 
-    QWidget *rotatorDialog = nullptr;
-    QTRY_VERIFY_WITH_TIMEOUT(rotatorDialog = Ekos::Manager::Instance()->findChild<QWidget *>("RotatorDialog"), 2000);
+    // ensure that the rotator dialog appears
+    QTest::qWait(2000);
+    QWidget *rotatorDialog = Ekos::Manager::Instance()->findChild<QWidget *>("RotatorDialog");
+    QVERIFY(rotatorDialog != nullptr);
 
     // target angle rotation
     double targetAngle = 90.0;
     // enforce rotation
     KTRY_SET_CHECKBOX(rotatorDialog, enforceRotationCheck, true);
     // set the target rotation angle
-    KTRY_SET_DOUBLESPINBOX(rotatorDialog, targetPASpin, targetAngle);
+    KTRY_SET_DOUBLESPINBOX(rotatorDialog, positionAngleSpin, targetAngle);
     // Close with OK
-    KTRY_GADGET(capture, QDialogButtonBox, buttonBox);
+    KTRY_GADGET(rotatorDialog, QDialogButtonBox, buttonBox);
     QTest::mouseClick(buttonBox->button(QDialogButtonBox::Ok), Qt::LeftButton);
 
     // build a simple 1xL sequence
@@ -403,14 +426,14 @@ void TestEkosCaptureWorkflow::testCaptureWaitingForRotator()
     // expect capturing state
     m_CaptureHelper->expectedCaptureStates.append(Ekos::CAPTURE_CAPTURING);
 
-   // start capturing
+    // start capturing
     KTRY_CLICK(capture, startB);
     // check if capturing has started
     KVERIFY_EMPTY_QUEUE_WITH_TIMEOUT(m_CaptureHelper->expectedCaptureStates, 60000);
 
-    KTRY_GADGET(capture, QLineEdit, PAOut);
-    QTRY_VERIFY2(fabs(PAOut->text().toDouble() - targetAngle) * 60 <= Options::astrometryRotatorThreshold(),
-                 QString("Rotator angle %1° not at the expected value of %2°").arg(PAOut->text()).arg(targetAngle).toLocal8Bit());
+    KTRY_GADGET(rotatorDialog, QLineEdit, rawAngleOut);
+    QTRY_VERIFY2(fabs(rawAngleOut->text().toDouble() - targetAngle) * 60 <= Options::astrometryRotatorThreshold(),
+                 QString("Rotator angle %1° not at the expected value of %2°").arg(rawAngleOut->text()).arg(targetAngle).toLocal8Bit());
 }
 
 void TestEkosCaptureWorkflow::testFlatManualSource()
@@ -574,6 +597,7 @@ void TestEkosCaptureWorkflow::testDustcapSource()
 
     // switch capture type to flat so that we can set the calibration
     KTRY_CAPTURE_GADGET(QComboBox, captureTypeS);
+    QTRY_VERIFY_WITH_TIMEOUT(captureTypeS->findText("Flat", Qt::MatchExactly) >= 0, 5000);
     KTRY_CAPTURE_COMBO_SET(captureTypeS, "Flat");
 
     // select frame type and internal or external flat light
@@ -634,8 +658,8 @@ void TestEkosCaptureWorkflow::testWallSource()
     m_CaptureHelper->expectedCaptureStates.append(Ekos::CAPTURE_IDLE);
     m_CaptureHelper->expectedCaptureStates.append(Ekos::CAPTURE_IMAGE_RECEIVED);
     m_CaptureHelper->expectedCaptureStates.append(Ekos::CAPTURE_COMPLETE);
-    m_CaptureHelper->expectedMountStates.append(ISD::Telescope::MOUNT_SLEWING);
-    m_CaptureHelper->expectedMountStates.append(ISD::Telescope::MOUNT_IDLE);
+    m_CaptureHelper->expectedMountStates.append(ISD::Mount::MOUNT_SLEWING);
+    m_CaptureHelper->expectedMountStates.append(ISD::Mount::MOUNT_IDLE);
     KTRY_CLICK(capture, startB);
     // check if mount has reached the expected position
     KVERIFY_EMPTY_QUEUE_WITH_TIMEOUT(m_CaptureHelper->expectedMountStates, 60000);
@@ -653,6 +677,8 @@ void TestEkosCaptureWorkflow::testFlatPreMountAndDomePark()
     // default initialization
     QVERIFY(prepareTestCase());
 
+    QSKIP("Observatory refactoring needs to be completed until this test can be activated.");
+
     // switch to capture module
     Ekos::Capture *capture = Ekos::Manager::Instance()->captureModule();
     KTRY_SWITCH_TO_MODULE_WITH_TIMEOUT(capture, 1000);
@@ -671,7 +697,7 @@ void TestEkosCaptureWorkflow::testFlatPreMountAndDomePark()
 
     // start the sequence
     m_CaptureHelper->expectedDomeStates.append(ISD::Dome::DOME_PARKED);
-    m_CaptureHelper->expectedMountStates.append(ISD::Telescope::MOUNT_PARKED);
+    m_CaptureHelper->expectedMountStates.append(ISD::Mount::MOUNT_PARKED);
     m_CaptureHelper->expectedCaptureStates.append(Ekos::CAPTURE_CAPTURING);
     m_CaptureHelper->expectedCaptureStates.append(Ekos::CAPTURE_COMPLETE);
     KTRY_CLICK(capture, startB);
@@ -700,11 +726,11 @@ void TestEkosCaptureWorkflow::testFlatSyncFocus()
     KTRY_GADGET(Ekos::Manager::Instance()->focusModule(), QLineEdit, absTicksLabel);
     int focusPosition = absTicksLabel->text().toInt();
     // move the focuser 100 steps out
-    KTRY_SET_SPINBOX(focus, absTicksSpin, focusPosition+100);
+    KTRY_SET_SPINBOX(focus, absTicksSpin, focusPosition + 100);
     // click goto
     KTRY_CLICK(focus, startGotoB);
     // check if new position has been reached
-    QTRY_VERIFY_WITH_TIMEOUT(absTicksLabel->text().toInt() == focusPosition+100, 5000);
+    QTRY_VERIFY_WITH_TIMEOUT(absTicksLabel->text().toInt() == focusPosition + 100, 5000);
 
     // capture flats with light panel source (simplest way to test)
     Ekos::Capture *capture = Ekos::Manager::Instance()->captureModule();
@@ -906,9 +932,9 @@ void TestEkosCaptureWorkflow::testDustcapSource_data()
     QTest::addColumn<QString>("frametype");              /*!< frame type (Dark or Flat)           */
     QTest::addColumn<bool>("internalLight");             /*!< use internal or external flat light */
 
-    QTest::newRow("Flat, light=internal") << "Flat" << true;   // flat + light source integrated into the light panel
+//    QTest::newRow("Flat, light=internal") << "Flat" << true;   // flat + light source integrated into the light panel
     QTest::newRow("Flat, light=external") << "Flat" << false;  // flat + external light source used
-    QTest::newRow("Dark") << "Dark" << false;  // dark
+//    QTest::newRow("Dark") << "Dark" << false;  // dark
 }
 
 void TestEkosCaptureWorkflow::testWallSource_data()
@@ -952,9 +978,9 @@ void TestEkosCaptureWorkflow::testCaptureWaitingForTemperature_data()
 void TestEkosCaptureWorkflow::initTestCase()
 {
     KVERIFY_EKOS_IS_HIDDEN();
-    // set guiding agressiveness (needs to be set before the guiding module starts)
-    Options::setRAProportionalGain(0.25);
-    Options::setDECProportionalGain(0.25);
+    // limit guiding pulses to ensure that guiding deviations lead to aborted capture
+    Options::setRAMaximumPulseArcSec(5.0);
+    Options::setDECMaximumPulseArcSec(5.0);
 
     QStandardPaths::setTestModeEnabled(true);
 
@@ -971,10 +997,19 @@ void TestEkosCaptureWorkflow::cleanupTestCase()
 
 bool TestEkosCaptureWorkflow::prepareTestCase()
 {
-    Ekos::Manager * const ekos = Ekos::Manager::Instance();
-
-    // start the profile
+    // use the helper to start the profile
     KVERIFY_SUB(m_CaptureHelper->startEkosProfile());
+    // prepare optical trains for testing
+    m_CaptureHelper->prepareOpticalTrains();
+    // prepare the mount module for testing with OAG guiding
+    m_CaptureHelper->prepareMountModule();
+    // prepare for focusing tests
+    m_CaptureHelper->prepareFocusModule();
+    // prepare for alignment tests
+    m_CaptureHelper->prepareAlignmentModule();
+    // prepare for guiding tests
+    m_CaptureHelper->prepareGuidingModule();
+
     m_CaptureHelper->init();
 
     // clear image directory
@@ -984,52 +1019,6 @@ bool TestEkosCaptureWorkflow::prepareTestCase()
     Options::setVerboseLogging(false);
     Options::setLogToFile(false);
 
-    // set mount defaults
-    KWRAP_SUB(QTRY_VERIFY_WITH_TIMEOUT(ekos->mountModule() != nullptr, 5000));
-    // set primary scope to my favorite FSQ-85
-    KTRY_SET_DOUBLESPINBOX_SUB(ekos->mountModule(), primaryScopeFocalIN, 450.0);
-    KTRY_SET_DOUBLESPINBOX_SUB(ekos->mountModule(), primaryScopeApertureIN, 85.0);
-    // set guide scope to a Tak 7x50
-    KTRY_SET_DOUBLESPINBOX_SUB(ekos->mountModule(), guideScopeFocalIN, 170.0);
-    KTRY_SET_DOUBLESPINBOX_SUB(ekos->mountModule(), guideScopeApertureIN, 50.0);
-    // save values
-    KTRY_CLICK_SUB(ekos->mountModule(), saveB);
-    // disable meridian flip
-    KTRY_SET_CHECKBOX_SUB(ekos->mountModule(), meridianFlipCheckBox, false);
-    // switch to focus module
-    KWRAP_SUB(QTRY_VERIFY_WITH_TIMEOUT(ekos->focusModule() != nullptr, 5000));
-    // set filter to luminance
-    KTRY_SET_COMBO_SUB(ekos->focusModule(), FilterPosCombo, "Luminance");
-    // set exposure time to 3 sec
-    KTRY_SET_DOUBLESPINBOX_SUB(ekos->focusModule(), exposureIN, 3.0);
-    // use full field
-    KTRY_SET_CHECKBOX_SUB(ekos->focusModule(), useFullField, true);
-    // use iterative algorithm
-    KTRY_SET_COMBO_SUB(ekos->focusModule(), focusAlgorithmCombo, "Polynomial");
-    // step size 5000
-    KTRY_SET_SPINBOX_SUB(ekos->focusModule(), stepIN, 5000);
-    // set max travel to 100000
-    KTRY_SET_DOUBLESPINBOX_SUB(ekos->focusModule(), maxTravelIN, 100000);
-    // set low accuracy to 10%
-    KTRY_SET_DOUBLESPINBOX_SUB(ekos->focusModule(), toleranceIN, 10.0);
-    // set outstep multiple
-    KTRY_SET_DOUBLESPINBOX_SUB(ekos->focusModule(), initialFocusOutStepsIN, 2);
-    // set average over 1 frame for focusing
-    KTRY_SET_SPINBOX_SUB(ekos->focusModule(), focusFramesSpin, 1);
-    // ensure that guiding calibration is NOT reused
-    Options::setReuseGuideCalibration(false);
-    // 4 guide calibration steps are sufficient
-    Options::setAutoModeIterations(4);
-    // Set the guiding and ST4 device
-    KTRY_SET_COMBO_SUB(Ekos::Manager::Instance()->guideModule(), guiderCombo, m_CaptureHelper->m_GuiderDevice);
-    KTRY_SET_COMBO_SUB(Ekos::Manager::Instance()->guideModule(), ST4Combo, m_CaptureHelper->m_MountDevice);
-    // select primary scope (higher focal length seems better for the guiding simulator)
-    KTRY_SET_COMBO_INDEX_SUB(Ekos::Manager::Instance()->guideModule(), FOVScopeCombo, ISD::CCD::TELESCOPE_PRIMARY);
-    // disable guiding restrictions by default
-    KTRY_SET_CHECKBOX_SUB(ekos->captureModule(), startGuiderDriftS, false);
-    KTRY_SET_CHECKBOX_SUB(ekos->captureModule(), limitGuideDeviationS, false);
-
-
     // ensure that the scope is unparked
     Ekos::Mount *mount = Ekos::Manager::Instance()->mountModule();
     if (mount->parkStatus() == ISD::PARK_PARKED)
@@ -1037,14 +1026,14 @@ bool TestEkosCaptureWorkflow::prepareTestCase()
     KTRY_VERIFY_WITH_TIMEOUT_SUB(mount->parkStatus() == ISD::PARK_UNPARKED, 30000);
 
     // ensure that the dome is unparked
-    if (m_CaptureHelper->m_DomeDevice != nullptr)
-    {
-        Ekos::Dome *dome = Ekos::Manager::Instance()->domeModule();
-        KVERIFY_SUB(dome != nullptr);
-        if (dome->parkStatus() == ISD::PARK_PARKED)
-            dome->unpark();
-        KTRY_VERIFY_WITH_TIMEOUT_SUB(dome->parkStatus() == ISD::PARK_UNPARKED, 30000);
-    }
+    //    if (m_CaptureHelper->m_DomeDevice != nullptr)
+    //    {
+    //        Ekos::Dome *dome = Ekos::Manager::Instance()->domeModule();
+    //        KVERIFY_SUB(dome != nullptr);
+    //        if (dome->parkStatus() == ISD::PARK_PARKED)
+    //            dome->unpark();
+    //        KTRY_VERIFY_WITH_TIMEOUT_SUB(dome->parkStatus() == ISD::PARK_UNPARKED, 30000);
+    //    }
 
     // close INDI window
     GUIManager::Instance()->close();
@@ -1053,7 +1042,8 @@ bool TestEkosCaptureWorkflow::prepareTestCase()
     return true;
 }
 
-void TestEkosCaptureWorkflow::init() {
+void TestEkosCaptureWorkflow::init()
+{
     // reset counters
     image_count  = 0;
 
@@ -1061,9 +1051,12 @@ void TestEkosCaptureWorkflow::init() {
     KVERIFY_EKOS_IS_OPENED();
     // clear light panel
     m_CaptureHelper->m_LightPanelDevice = nullptr;
+    // clear rotator
+    m_CaptureHelper->m_RotatorDevice = nullptr;
 }
 
-void TestEkosCaptureWorkflow::cleanup() {
+void TestEkosCaptureWorkflow::cleanup()
+{
     if (Ekos::Manager::Instance()->focusModule() != nullptr)
         Ekos::Manager::Instance()->focusModule()->abort();
 
