@@ -8,13 +8,19 @@
 
 #include "test_ekos_helper.h"
 #include "ksutils.h"
+#include "Options.h"
+#include "ekos/profileeditor.h"
+#include "ekos/guide/internalguide/gmath.h"
+#include "ekos/auxiliary/opticaltrainmanager.h"
 
-TestEkosHelper::TestEkosHelper(QString guider) {
+TestEkosHelper::TestEkosHelper(QString guider)
+{
     m_Guider = guider;
     m_MountDevice = "Telescope Simulator";
     m_CCDDevice   = "CCD Simulator";
     if (guider != nullptr)
-        m_GuiderDevice = "Guide Simulator";
+        m_GuiderDevice = "CCD Simulator";
+    m_astrometry_available = false;
 }
 
 
@@ -46,49 +52,56 @@ void TestEkosHelper::fillProfile(bool *isDone)
     ProfileEditor* profileEditor = Ekos::Manager::Instance()->findChild<ProfileEditor*>("profileEditorDialog");
 
     // Select the mount device
-    if (m_MountDevice != nullptr) {
+    if (m_MountDevice != "")
+    {
         KTRY_PROFILEEDITOR_GADGET(QComboBox, mountCombo);
         setTreeviewCombo(mountCombo, m_MountDevice);
         qCInfo(KSTARS_EKOS_TEST) << "Fill profile: Mount selected.";
     }
 
     // Selet the CCD device
-    if (m_CCDDevice != nullptr) {
+    if (m_CCDDevice != "")
+    {
         KTRY_PROFILEEDITOR_GADGET(QComboBox, ccdCombo);
         setTreeviewCombo(ccdCombo, m_CCDDevice);
         qCInfo(KSTARS_EKOS_TEST) << "Fill profile: CCD selected.";
     }
 
     // Select the focuser device
-    if (m_FocuserDevice != nullptr) {
+    if (m_FocuserDevice != "")
+    {
         KTRY_PROFILEEDITOR_GADGET(QComboBox, focuserCombo);
         setTreeviewCombo(focuserCombo, m_FocuserDevice);
         qCInfo(KSTARS_EKOS_TEST) << "Fill profile: Focuser selected.";
     }
 
-    // Select the guider device
-    if (m_GuiderDevice != nullptr) {
+    // Select the guider device, if not empty and different from CCD
+    if (m_GuiderDevice != "" && m_GuiderDevice != m_CCDDevice)
+    {
         KTRY_PROFILEEDITOR_GADGET(QComboBox, guiderCombo);
         setTreeviewCombo(guiderCombo, m_GuiderDevice);
         qCInfo(KSTARS_EKOS_TEST) << "Fill profile: Guider selected.";
     }
 
     // Select the light panel device for flats capturing
-    if (m_LightPanelDevice != nullptr) {
+    if (m_LightPanelDevice != "")
+    {
         KTRY_PROFILEEDITOR_GADGET(QComboBox, aux1Combo);
         setTreeviewCombo(aux1Combo, m_LightPanelDevice);
         qCInfo(KSTARS_EKOS_TEST) << "Fill profile: Light panel selected.";
     }
 
     // Select the dome device
-    if (m_DomeDevice != nullptr) {
+    if (m_DomeDevice != "")
+    {
         KTRY_PROFILEEDITOR_GADGET(QComboBox, domeCombo);
         setTreeviewCombo(domeCombo, m_DomeDevice);
         qCInfo(KSTARS_EKOS_TEST) << "Fill profile: Dome selected.";
     }
 
     // Select the rotator device
-    if (m_RotatorDevice != nullptr) {
+    if (m_RotatorDevice != "")
+    {
         KTRY_PROFILEEDITOR_GADGET(QComboBox, aux2Combo);
         setTreeviewCombo(aux2Combo, m_RotatorDevice);
         qCInfo(KSTARS_EKOS_TEST) << "Fill profile: Rotator selected.";
@@ -124,7 +137,7 @@ bool TestEkosHelper::setupEkosProfile(QString name, bool isPHD2)
         if (editProfileB->isEnabled())
         {
             // start with a delay of 1 sec a new thread that edits the profile
-            QTimer::singleShot(1000, ekos, [&]{fillProfile(&isDone);});
+            QTimer::singleShot(1000, ekos, [&] {fillProfile(&isDone);});
             KTRY_CLICK_SUB(ekos, editProfileB);
         }
         else
@@ -138,17 +151,16 @@ bool TestEkosHelper::setupEkosProfile(QString name, bool isPHD2)
     {
         // start with a delay of 1 sec a new thread that edits the profile
         qCInfo(KSTARS_EKOS_TEST) << "Creating new profile " << name << " ...";
-        QTimer::singleShot(1000, ekos, [&]{createEkosProfile(name, isPHD2, &isDone);});
+        QTimer::singleShot(1000, ekos, [&] {createEkosProfile(name, isPHD2, &isDone);});
         // create new profile addProfileB
         KTRY_CLICK_SUB(ekos, addProfileB);
     }
 
-
     // Cancel the profile editor if test did not close the editor dialog within 10 sec
     QTimer * closeDialog = new QTimer(this);
     closeDialog->setSingleShot(true);
     closeDialog->setInterval(10000);
-    ekos->connect(closeDialog, &QTimer::timeout, [&]
+    ekos->connect(closeDialog, &QTimer::timeout, this, [&]
     {
         ProfileEditor* profileEditor = ekos->findChild<ProfileEditor*>("profileEditorDialog");
         if (profileEditor != nullptr)
@@ -172,41 +184,61 @@ void TestEkosHelper::connectModules()
 {
     Ekos::Manager * const ekos = Ekos::Manager::Instance();
 
+    // wait for modules startup
+    if (m_MountDevice != "")
+        QTRY_VERIFY_WITH_TIMEOUT(ekos->mountModule() != nullptr, 10000);
+    if (m_CCDDevice != "")
+    {
+        QTRY_VERIFY_WITH_TIMEOUT(ekos->captureModule() != nullptr, 10000);
+        QTRY_VERIFY_WITH_TIMEOUT(ekos->alignModule() != nullptr, 10000);
+    }
+    if (m_GuiderDevice != "")
+        QTRY_VERIFY_WITH_TIMEOUT(ekos->guideModule() != nullptr, 10000);
+    if (m_FocuserDevice != "")
+        QTRY_VERIFY_WITH_TIMEOUT(ekos->focusModule() != nullptr, 10000);
+
     // connect to the alignment process to receive align status changes
     connect(ekos->alignModule(), &Ekos::Align::newStatus, this, &TestEkosHelper::alignStatusChanged,
             Qt::UniqueConnection);
 
-    // connect to the mount process to rmount status changes
-    connect(ekos->mountModule(), &Ekos::Mount::newStatus, this,
-            &TestEkosHelper::mountStatusChanged, Qt::UniqueConnection);
+    if (m_MountDevice != "")
+    {
+        // connect to the mount process to rmount status changes
+        connect(ekos->mountModule(), &Ekos::Mount::newStatus, this,
+                &TestEkosHelper::mountStatusChanged, Qt::UniqueConnection);
 
-    // connect to the mount process to receive meridian flip status changes
-    connect(ekos->mountModule(), &Ekos::Mount::newMeridianFlipStatus, this,
-            &TestEkosHelper::meridianFlipStatusChanged, Qt::UniqueConnection);
+        // connect to the mount process to receive meridian flip status changes
+        connect(ekos->mountModule()->getMeridianFlipState().get(), &Ekos::MeridianFlipState::newMountMFStatus, this,
+                &TestEkosHelper::meridianFlipStatusChanged, Qt::UniqueConnection);
+    }
 
-    // connect to the guiding process to receive guiding status changes
-    connect(ekos->guideModule(), &Ekos::Guide::newStatus, this, &TestEkosHelper::guidingStatusChanged,
-            Qt::UniqueConnection);
-
-    connect(ekos->guideModule(), &Ekos::Guide::newAxisDelta, this, &TestEkosHelper::guideDeviationChanged,
-            Qt::UniqueConnection);
+    if (m_GuiderDevice != "")
+    {
+        // connect to the guiding process to receive guiding status changes
+        connect(ekos->guideModule(), &Ekos::Guide::newStatus, this, &TestEkosHelper::guidingStatusChanged,
+                Qt::UniqueConnection);
 
+        connect(ekos->guideModule(), &Ekos::Guide::newAxisDelta, this, &TestEkosHelper::guideDeviationChanged,
+                Qt::UniqueConnection);
+    }
 
     // connect to the capture process to receive capture status changes
-    connect(ekos->captureModule(), &Ekos::Capture::newStatus, this, &TestEkosHelper::captureStatusChanged,
-            Qt::UniqueConnection);
+    if (m_CCDDevice != "")
+        connect(ekos->captureModule(), &Ekos::Capture::newStatus, this, &TestEkosHelper::captureStatusChanged,
+                Qt::UniqueConnection);
 
     // connect to the scheduler process to receive scheduler status changes
     connect(ekos->schedulerModule(), &Ekos::Scheduler::newStatus, this, &TestEkosHelper::schedulerStatusChanged,
             Qt::UniqueConnection);
 
     // connect to the focus process to receive focus status changes
-    connect(ekos->focusModule(), &Ekos::Focus::newStatus, this, &TestEkosHelper::focusStatusChanged,
-            Qt::UniqueConnection);
+    if (m_FocuserDevice != "")
+        connect(ekos->focusModule(), &Ekos::Focus::newStatus, this, &TestEkosHelper::focusStatusChanged,
+                Qt::UniqueConnection);
 
     // connect to the dome process to receive dome status changes
-    connect(ekos->domeModule(), &Ekos::Dome::newStatus, this, &TestEkosHelper::domeStatusChanged,
-            Qt::UniqueConnection);
+    //    connect(ekos->domeModule(), &Ekos::Dome::newStatus, this, &TestEkosHelper::domeStatusChanged,
+    //            Qt::UniqueConnection);
 }
 
 bool TestEkosHelper::startEkosProfile()
@@ -229,8 +261,7 @@ bool TestEkosHelper::startEkosProfile()
     KTRY_EKOS_CLICK(processINDIB);
     // wait for the devices to come up
     KWRAP_SUB(QTRY_VERIFY_WITH_TIMEOUT(Ekos::Manager::Instance()->indiStatus() == Ekos::Success, 10000));
-
-    // connect to the alignment process to receive align status changes
+    // receive status updates from all devices
     connectModules();
 
     // Everything completed successfully
@@ -240,8 +271,8 @@ bool TestEkosHelper::startEkosProfile()
 bool TestEkosHelper::shutdownEkosProfile()
 {
     // disconnect to the dome process to receive dome status changes
-    disconnect(Ekos::Manager::Instance()->domeModule(), &Ekos::Dome::newStatus, this,
-               &TestEkosHelper::domeStatusChanged);
+    //    disconnect(Ekos::Manager::Instance()->domeModule(), &Ekos::Dome::newStatus, this,
+    //               &TestEkosHelper::domeStatusChanged);
     // disconnect to the focus process to receive focus status changes
     disconnect(Ekos::Manager::Instance()->focusModule(), &Ekos::Focus::newStatus, this,
                &TestEkosHelper::focusStatusChanged);
@@ -252,7 +283,7 @@ bool TestEkosHelper::shutdownEkosProfile()
     disconnect(Ekos::Manager::Instance()->mountModule(), &Ekos::Mount::newStatus, this,
                &TestEkosHelper::mountStatusChanged);
     // disconnect the mount process to receive meridian flip status changes
-    disconnect(Ekos::Manager::Instance()->mountModule(), &Ekos::Mount::newMeridianFlipStatus, this,
+    disconnect(Ekos::Manager::Instance()->mountModule()->getMeridianFlipState().get(), &Ekos::MeridianFlipState::newMountMFStatus, this,
                &TestEkosHelper::meridianFlipStatusChanged);
     // disconnect to the scheduler process to receive scheduler status changes
     disconnect(Ekos::Manager::Instance()->schedulerModule(), &Ekos::Scheduler::newStatus, this,
@@ -290,7 +321,7 @@ void TestEkosHelper::startPHD2()
 {
     phd2 = new QProcess(this);
     QStringList arguments;
-        // Start PHD2 with the proper configuration
+    // Start PHD2 with the proper configuration
     phd2->start(QString("phd2"), arguments);
     QVERIFY(phd2->waitForStarted(3000));
     QTest::qWait(2000);
@@ -353,6 +384,246 @@ void TestEkosHelper::cleanupPHD2()
         QVERIFY(QFile::rename(phd2_config_home_bak.filePath(), phd2_config_home.filePath()));
 }
 
+void TestEkosHelper::prepareOpticalTrains()
+{
+    Ekos::OpticalTrainManager *otm = Ekos::OpticalTrainManager::Instance();
+    // close window, we change everything programatically
+    otm->close();
+    // setup train with main scope and camera
+    QVariantMap primaryTrain = otm->getOpticalTrain(m_primaryTrain);
+    primaryTrain["mount"] = m_MountDevice;
+    primaryTrain["camera"] = m_CCDDevice;
+    primaryTrain["filterwheel"] = m_CCDDevice;
+    primaryTrain["focuser"] = m_FocuserDevice == "" ? "-" : m_FocuserDevice;
+    primaryTrain["rotator"] = m_RotatorDevice == "" ? "-" : m_RotatorDevice;
+    primaryTrain["lightbox"] = m_LightPanelDevice == "" ? "-" : m_LightPanelDevice;
+    KStarsData::Instance()->userdb()->UpdateOpticalTrain(primaryTrain, primaryTrain["id"].toInt());
+    if (m_GuiderDevice != "")
+    {
+        // setup guiding scope train
+        QVariantMap guidingTrain = otm->getOpticalTrain(m_guidingTrain);
+        guidingTrain["mount"] = m_MountDevice;
+        guidingTrain["camera"] = m_GuiderDevice;
+        guidingTrain["filterwheel"] = "-";
+        guidingTrain["focuser"] = "-";
+        guidingTrain["guider"] = m_MountDevice;
+        KStarsData::Instance()->userdb()->UpdateOpticalTrain(guidingTrain, guidingTrain["id"].toInt());
+    }
+    // ensure that the OTM initializes from the database
+    otm->refreshModel();
+    otm->refreshTrains();
+}
+
+void TestEkosHelper::prepareAlignmentModule()
+{
+
+    // check if astrometry files exist
+    QTRY_VERIFY(isAstrometryAvailable() == true);
+    // switch to alignment module
+    KTRY_SWITCH_TO_MODULE_WITH_TIMEOUT(Ekos::Manager::Instance()->alignModule(), 1000);
+    // select the primary train for alignment
+    KTRY_SET_COMBO(Ekos::Manager::Instance()->alignModule(), opticalTrainCombo, m_primaryTrain);
+    // select local solver
+    Ekos::Manager::Instance()->alignModule()->setSolverMode(Ekos::Align::SOLVER_LOCAL);
+    // select internal SEP method
+    Options::setSolveSextractorType(SSolver::EXTRACTOR_BUILTIN);
+    // select StellarSolver
+    Options::setSolverType(SSolver::SOLVER_STELLARSOLVER);
+    // select fast solve profile option
+    Options::setSolveOptionsProfile(SSolver::Parameters::SINGLE_THREAD_SOLVING);
+    // select the "Slew to Target" mode
+    KTRY_SET_RADIOBUTTON(Ekos::Manager::Instance()->alignModule(), slewR, true);
+    // disable rotator check in alignment
+    Options::setAstrometryUseRotator(false);
+}
+
+void TestEkosHelper::prepareFocusModule()
+{
+    // set focus mode defaults
+    KTRY_SWITCH_TO_MODULE_WITH_TIMEOUT(Ekos::Manager::Instance()->focusModule(), 1000);
+    // select the primary train for focusing
+    KTRY_SET_COMBO(Ekos::Manager::Instance()->focusModule(), opticalTrainCombo, m_primaryTrain);
+    // use full field
+    KTRY_SET_CHECKBOX(Ekos::Manager::Instance()->focusModule(), focusUseFullField, true);
+    //initial step size 5000
+    KTRY_SET_SPINBOX(Ekos::Manager::Instance()->focusModule(), focusTicks, 5000);
+    // max travel 50000
+    KTRY_SET_DOUBLESPINBOX(Ekos::Manager::Instance()->focusModule(), focusMaxTravel, 50000.0);
+    // focus tolerance 20% - make focus fast and robust, precision does not matter
+    KTRY_GADGET(Ekos::Manager::Instance()->focusModule(), QDoubleSpinBox, focusTolerance);
+    focusTolerance->setMaximum(20.0);
+    focusTolerance->setValue(20.0);
+    // use single pass linear algorithm
+    KTRY_SET_COMBO(Ekos::Manager::Instance()->focusModule(), focusAlgorithm, "Linear 1 Pass");
+    // select star detection
+    KTRY_SET_COMBO(Ekos::Manager::Instance()->focusModule(), focusSEPProfile, "1-Focus-Default");
+    // set annulus to 0% - 50%
+    KTRY_SET_DOUBLESPINBOX(Ekos::Manager::Instance()->focusModule(), focusFullFieldInnerRadius, 0.0);
+    KTRY_SET_DOUBLESPINBOX(Ekos::Manager::Instance()->focusModule(), focusFullFieldOuterRadius, 50.0);
+    // try to make focusing fast, precision is not relevant here
+    KTRY_SET_DOUBLESPINBOX(Ekos::Manager::Instance()->focusModule(), focusOutSteps, 1);
+    // select the Luminance filter
+    KTRY_SET_COMBO(Ekos::Manager::Instance()->focusModule(), focusFilter, "Luminance");
+    // select SEP algorithm for star detection
+    KTRY_SET_COMBO(Ekos::Manager::Instance()->focusModule(), focusDetection, "SEP");
+    // set exp time for current filter
+    KTRY_SET_DOUBLESPINBOX(Ekos::Manager::Instance()->focusModule(), focusExposure, 3.0);
+    // set exposure times for all filters
+    auto filtermanager = Ekos::Manager::Instance()->focusModule()->filterManager();
+    for (int pos = 0; pos < filtermanager->getFilterLabels().count(); pos++)
+    {
+        filtermanager->setFilterExposure(pos, 3.0);
+        filtermanager->setFilterLock(pos, "Luminance");
+    }
+    // gain 100
+    KTRY_SET_DOUBLESPINBOX(Ekos::Manager::Instance()->focusModule(), focusGain, 100.0);
+    // suspend guiding while focusing
+    KTRY_SET_CHECKBOX(Ekos::Manager::Instance()->focusModule(), focusSuspendGuiding, true);
+}
+
+void TestEkosHelper::prepareGuidingModule()
+{
+    // switch to guiding module
+    KTRY_SWITCH_TO_MODULE_WITH_TIMEOUT(Ekos::Manager::Instance()->guideModule(), 1000);
+
+    // preserve guiding calibration as good as possible
+    Options::setReuseGuideCalibration(true);
+    Options::setResetGuideCalibration(false);
+    // guide calibration captured with fsq-85 as guiding scope, clear if it creates problems
+    Options::setSerializedCalibration("Cal v1.0,bx=1,by=1,pw=0.0098,ph=0.0126,fl=450,ang=269.84,angR=270.362,angD=179.318,ramspas=104.501,decmspas=191.838,swap=0,ra= 143:57:35,dec=00:25:52,side=0,when=2022-09-11 12:05:16,calEnd");
+    // 0.5 pixel dithering
+    Options::setDitherPixels(0.5);
+    if (m_Guider == "PHD2")
+    {
+        KTRY_GADGET(Ekos::Manager::Instance()->guideModule(), QPushButton, externalConnectB);
+        // ensure that PHD2 is connected
+        if (externalConnectB->isEnabled())
+        {
+            // click "Connect"
+            KTRY_CLICK(Ekos::Manager::Instance()->guideModule(), externalConnectB);
+            // wait max 60 sec that PHD2 is connected (sometimes INDI connections hang)
+            QTRY_VERIFY_WITH_TIMEOUT(Ekos::Manager::Instance()->guideModule()->status() == Ekos::GUIDE_CONNECTED, 60000);
+            qCInfo(KSTARS_EKOS_TEST) << "PHD2 connected successfully.";
+        }
+    }
+    else
+    {
+        // select the secondary train for guiding
+        KTRY_SET_COMBO(Ekos::Manager::Instance()->guideModule(), opticalTrainCombo, m_guidingTrain);
+        // select multi-star
+        Options::setGuideAlgorithm(SEP_MULTISTAR);
+        // select small star profile
+        Options::setGuideOptionsProfile(2);
+        // set 2 sec guiding calibration pulse duration
+        Options::setCalibrationPulseDuration(2000);
+        // use three steps in each direction for calibration
+        Options::setAutoModeIterations(3);
+        // set the guiding aggressiveness
+        Options::setRAProportionalGain(0.5);
+        Options::setDECProportionalGain(0.5);
+        // use simulator's guide head
+        Options::setUseGuideHead(true);
+    }
+}
+
+Scope *TestEkosHelper::createScopeIfNecessary(QString model, QString vendor, QString type, double aperture,
+        double focallenght)
+{
+    QList<Scope *> scope_list;
+    KStarsData::Instance()->userdb()->GetAllScopes(scope_list);
+
+    for (Scope *scope : scope_list)
+    {
+        if (scope->model() == model && scope->vendor() == vendor && scope->type() == type && scope->aperture() == aperture
+                && scope->focalLength() == focallenght)
+            return scope;
+    }
+    // no match found, create it again
+    KStarsData::Instance()->userdb()->AddScope(model, vendor, type, aperture, focallenght);
+    Ekos::OpticalTrainManager::Instance()->refreshOpticalElements();
+    // find it
+    scope_list.clear();
+    KStarsData::Instance()->userdb()->GetAllScopes(scope_list);
+    for (Scope *scope : scope_list)
+    {
+        if (scope->model() == model && scope->vendor() == vendor && scope->type() == type && scope->aperture() == aperture
+                && scope->focalLength() == focallenght)
+            return scope;
+    }
+    // this should never happen
+    return nullptr;
+}
+
+OAL::Scope *TestEkosHelper::getScope(TestEkosHelper::ScopeType type)
+{
+    switch (type)
+    {
+        case SCOPE_FSQ85:
+            return fsq85;
+        case SCOPE_NEWTON_10F4:
+            return newton_10F4;
+        case SCOPE_TAKFINDER10x50:
+            return takfinder10x50;
+    }
+    // this should never happen
+    return fsq85;
+}
+
+void TestEkosHelper::prepareMountModule(ScopeType primary, ScopeType guiding)
+{
+    Ekos::OpticalTrainManager *otm = Ekos::OpticalTrainManager::Instance();
+    // set mount defaults
+    QTRY_VERIFY_WITH_TIMEOUT(Ekos::Manager::Instance()->mountModule() != nullptr, 5000);
+    QTRY_VERIFY_WITH_TIMEOUT(Ekos::Manager::Instance()->mountModule()->slewStatus() != IPState::IPS_ALERT, 1000);
+
+    // define a set of scopes
+    fsq85          = createScopeIfNecessary("FSQ-85", "Takahashi", "Refractor", 85.0, 450.0);
+    newton_10F4    = createScopeIfNecessary("ONTC", "Teleskop-Service", "Newtonian", 254.0, 1000.0);
+    takfinder10x50 = createScopeIfNecessary("Finder 7x50", "Takahashi", "Refractor", 50.0, 170.0);
+
+    QVERIFY(fsq85 != nullptr);
+    QVERIFY(newton_10F4 != nullptr);
+    QVERIFY(takfinder10x50 != nullptr);
+
+    OAL::Scope *primaryScope = getScope(primary);
+    OAL::Scope *guidingScope = getScope(guiding);
+
+    // setup the primary train
+    QVariantMap primaryTrain = otm->getOpticalTrain(m_primaryTrain);
+    primaryTrain["scope"] = primaryScope->name();
+    primaryTrain["reducer"] = 1.0;
+    KStarsData::Instance()->userdb()->UpdateOpticalTrain(primaryTrain, primaryTrain["id"].toInt());
+
+    // setup the guiding train
+    QVariantMap guidingTrain = otm->getOpticalTrain(m_guidingTrain);
+    guidingTrain["scope"] = guidingScope->name();
+    guidingTrain["reducer"] = 1.0;
+    KStarsData::Instance()->userdb()->UpdateOpticalTrain(guidingTrain, guidingTrain["id"].toInt());
+    // ensure that the OTM initializes from the database
+    otm->refreshModel();
+    otm->refreshTrains();
+
+    // Set the scope parameters also in the telescope simulator to ensure that the CCD simulator creates the right FOW
+    KTRY_INDI_PROPERTY(m_MountDevice, "Options", "TELESCOPE_INFO", scope_info);
+    INDI_E *primary_aperture = scope_info->getElement("TELESCOPE_APERTURE");
+    INDI_E *primary_focallength = scope_info->getElement("TELESCOPE_FOCAL_LENGTH");
+    INDI_E *guider_aperture = scope_info->getElement("GUIDER_APERTURE");
+    INDI_E *guider_focallength = scope_info->getElement("GUIDER_FOCAL_LENGTH");
+    QVERIFY(primary_aperture != nullptr);
+    QVERIFY(primary_focallength != nullptr);
+    QVERIFY(guider_aperture != nullptr);
+    QVERIFY(guider_focallength != nullptr);
+    primary_aperture->setValue(primaryTrain["aperture"].toDouble());
+    primary_focallength->setValue(primaryTrain["focal_length"].toDouble() * primaryTrain["reducer"].toDouble());
+    guider_aperture->setValue(guidingTrain["aperture"].toDouble());
+    guider_focallength->setValue(guidingTrain["focal_length"].toDouble() * guidingTrain["reducer"].toDouble());
+    scope_info->processSetButton();
+
+
+    // select the primary train for the mount
+    KTRY_SET_COMBO(Ekos::Manager::Instance()->mountModule(), opticalTrainCombo, m_primaryTrain);
+}
+
 bool TestEkosHelper::slewTo(double RA, double DEC, bool fast)
 {
     if (fast)
@@ -367,7 +638,7 @@ bool TestEkosHelper::slewTo(double RA, double DEC, bool fast)
     // wait a certain time until the mount slews
     QTest::qWait(3000);
     // wait until the mount is tracking
-    KTRY_VERIFY_WITH_TIMEOUT_SUB(Ekos::Manager::Instance()->mountModule()->status() == ISD::Telescope::MOUNT_TRACKING, 10000);
+    KTRY_VERIFY_WITH_TIMEOUT_SUB(Ekos::Manager::Instance()->mountModule()->status() == ISD::Mount::MOUNT_TRACKING, 10000);
 
     // everything succeeded
     return true;
@@ -384,7 +655,13 @@ bool TestEkosHelper::startGuiding(double expTime)
     // switch to guiding module
     KWRAP_SUB(KTRY_SWITCH_TO_MODULE_WITH_TIMEOUT(Ekos::Manager::Instance()->guideModule(), 1000));
     //set the exposure time
-    KTRY_SET_DOUBLESPINBOX_SUB(Ekos::Manager::Instance()->guideModule(), exposureIN, expTime);
+    KTRY_SET_DOUBLESPINBOX_SUB(Ekos::Manager::Instance()->guideModule(), guideExposure, expTime);
+    // auto star select
+    KTRY_SET_CHECKBOX_SUB(Ekos::Manager::Instance()->guideModule(), guideAutoStar, true);
+    // set the guide star box to size 32
+    KTRY_SET_COMBO_SUB(Ekos::Manager::Instance()->guideModule(), guideSquareSize, "32");
+    // use 1x1 binning for guiding
+    KTRY_SET_COMBO_SUB(Ekos::Manager::Instance()->guideModule(), guideBinning, "1x1");
 
     // start guiding
     KTRY_GADGET_SUB(Ekos::Manager::Instance()->guideModule(), QPushButton, guideB);
@@ -394,6 +671,7 @@ bool TestEkosHelper::startGuiding(double expTime)
     KTRY_CLICK_SUB(Ekos::Manager::Instance()->guideModule(), guideB);
     KVERIFY_EMPTY_QUEUE_WITH_TIMEOUT_SUB(expectedGuidingStates, 120000);
     qCInfo(KSTARS_EKOS_TEST) << "Guiding started.";
+    qCInfo(KSTARS_EKOS_TEST) << "Guiding calibration: " << Options::serializedCalibration();
     qCInfo(KSTARS_EKOS_TEST) << "Waiting 2sec for settle guiding ...";
     QTest::qWait(2000);
     // all checks succeeded, remember that guiding is running
@@ -434,16 +712,18 @@ bool TestEkosHelper::startAligning(double expTime)
 {
     KWRAP_SUB(KTRY_SWITCH_TO_MODULE_WITH_TIMEOUT(Ekos::Manager::Instance()->alignModule(), 1000));
     // set the exposure time to the given value
-    KTRY_SET_DOUBLESPINBOX_SUB(Ekos::Manager::Instance()->alignModule(), exposureIN, expTime);
+    KTRY_SET_DOUBLESPINBOX_SUB(Ekos::Manager::Instance()->alignModule(), alignExposure, expTime);
     // reduce the accuracy to avoid testing problems
-    KTRY_SET_SPINBOX_SUB(Ekos::Manager::Instance()->alignModule(), accuracySpin, 300);
+    KTRY_SET_SPINBOX_SUB(Ekos::Manager::Instance()->alignModule(), alignAccuracyThreshold, 300);
+    // select the Luminance filter
+    KTRY_SET_COMBO_SUB(Ekos::Manager::Instance()->alignModule(), alignFilter, "Luminance");
 
     // start alignment
     KTRY_GADGET_SUB(Ekos::Manager::Instance()->alignModule(), QPushButton, solveB);
-    // ensure that the guiding button is enabled (after MF it may take a while)
-    KTRY_VERIFY_WITH_TIMEOUT_SUB(solveB->isEnabled(), 10000);
+    // ensure that the solve button is enabled (after MF it may take a while)
+    KTRY_VERIFY_WITH_TIMEOUT_SUB(solveB->isEnabled(), 60000);
     KTRY_CLICK_SUB(Ekos::Manager::Instance()->alignModule(), solveB);
-    // success
+    // alignment started
     return true;
 }
 
@@ -462,16 +742,28 @@ bool TestEkosHelper::checkAstrometryFiles()
         if (! dir.entryList().isEmpty())
             return true;
     }
+    QWARN(QString("No astrometry index files found in %1").arg(
+              KSUtils::getAstrometryDataDirs().join(", ")).toStdString().c_str());
     return false;
 }
 
-void TestEkosHelper::init() {
+bool TestEkosHelper::isAstrometryAvailable()
+{
+    // avoid double checks if already found
+    if (! m_astrometry_available)
+        m_astrometry_available = checkAstrometryFiles();
+
+    return m_astrometry_available;
+}
+
+void TestEkosHelper::init()
+{
     // initialize the recorded states
     m_AlignStatus   = Ekos::ALIGN_IDLE;
     m_CaptureStatus = Ekos::CAPTURE_IDLE;
     m_FocusStatus   = Ekos::FOCUS_IDLE;
     m_GuideStatus   = Ekos::GUIDE_IDLE;
-    m_MFStatus      = Ekos::Mount::FLIP_NONE;
+    m_MFStatus      = Ekos::MeridianFlipState::MOUNT_FLIP_NONE;
     m_DomeStatus    = ISD::Dome::DOME_IDLE;
     // initialize the event queues
     expectedAlignStates.clear();
@@ -486,8 +778,11 @@ void TestEkosHelper::init() {
 
     // disable by default
     use_guiding = false;
+    // reset dithering flag
+    dithered = false;
 }
-void TestEkosHelper::cleanup() {
+void TestEkosHelper::cleanup()
+{
 
 }
 
@@ -499,7 +794,8 @@ void TestEkosHelper::cleanup() {
 void TestEkosHelper::setTreeviewCombo(QComboBox *combo, QString lookup)
 {
     // Match the text recursively in the model, this results in a model index with a parent
-    QModelIndexList const list = combo->model()->match(combo->model()->index(0, 0), Qt::DisplayRole, QVariant::fromValue(lookup), 1, Qt::MatchRecursive);
+    QModelIndexList const list = combo->model()->match(combo->model()->index(0, 0), Qt::DisplayRole,
+                                 QVariant::fromValue(lookup), 1, Qt::MatchRecursive);
     QVERIFY(0 < list.count());
     QModelIndex const &index = list.first();
     QCOMPARE(list.value(0).data().toString(), lookup);
@@ -548,7 +844,7 @@ void TestEkosHelper::alignStatusChanged(Ekos::AlignState status)
         expectedAlignStates.dequeue();
 }
 
-void TestEkosHelper::mountStatusChanged(ISD::Telescope::Status status)
+void TestEkosHelper::mountStatusChanged(ISD::Mount::Status status)
 {
     m_MountStatus = status;
     // check if the new state is the next one expected, then remove it from the stack
@@ -556,7 +852,7 @@ void TestEkosHelper::mountStatusChanged(ISD::Telescope::Status status)
         expectedMountStates.dequeue();
 }
 
-void TestEkosHelper::meridianFlipStatusChanged(Ekos::Mount::MeridianFlipStatus status)
+void TestEkosHelper::meridianFlipStatusChanged(Ekos::MeridianFlipState::MeridianFlipMountState status)
 {
     m_MFStatus = status;
     // check if the new state is the next one expected, then remove it from the stack
@@ -578,6 +874,10 @@ void TestEkosHelper::guidingStatusChanged(Ekos::GuideState status)
     // check if the new state is the next one expected, then remove it from the stack
     if (!expectedGuidingStates.isEmpty() && expectedGuidingStates.head() == status)
         expectedGuidingStates.dequeue();
+    // dithering detected?
+    if (status == Ekos::GUIDE_DITHERING || status == Ekos::GUIDE_DITHERING_SUCCESS || status == Ekos::GUIDE_DITHERING_ERROR)
+        dithered = true;
+
 }
 
 void TestEkosHelper::guideDeviationChanged(double delta_ra, double delta_dec)
