@@ -15,8 +15,7 @@
 #include "indi/indigroup.h"
 #include "indi/indiproperty.h"
 #include "indi/indielement.h"
-
-#include "ekos/profileeditor.h"
+#include "indi/guimanager.h"
 
 #include <QObject>
 
@@ -243,6 +242,14 @@ do {\
 #define KTRY_SET_COMBO(module, combo, value) \
     KTRY_GADGET(module, QComboBox, combo); combo->setCurrentText(value); QVERIFY(combo->currentText() == value)
 
+/** @brief Helper to set a combo box by index and verify whether it succeeded
+ * @param module KStars module that holds the combo box
+ * @param combo object name of the combo box
+ * @param value index the combo box should be selected
+ */
+#define KTRY_SET_COMBO_INDEX(module, combo, value) \
+    KTRY_GADGET(module, QComboBox, combo); combo->setCurrentIndex(value); QVERIFY(combo->currentIndex() == value)
+
 /** @brief Subroutine version of @see KTRY_SET_COMBO
  * @param module KStars module that holds the combo box
  * @param combo object name of the combo box
@@ -251,13 +258,13 @@ do {\
 #define KTRY_SET_COMBO_SUB(module, combo, value) \
     KWRAP_SUB(KTRY_GADGET(module, QComboBox, combo); combo->setCurrentText(value); QVERIFY(combo->currentText() == value))
 
-/** @brief Helper to set a combo box by indexand verify whether it succeeded
+/** @brief Helper to set a combo box by index and verify whether it succeeded
  * @param module KStars module that holds the combo box
  * @param combo object name of the combo box
  * @param value index the combo box should be selected
  */
 #define KTRY_SET_COMBO_INDEX_SUB(module, combo, value) \
-    KWRAP_SUB(KTRY_GADGET(module, QComboBox, combo); combo->setCurrentIndex(value); QVERIFY(combo->currentIndex() == value))
+    KWRAP_SUB(KTRY_SET_COMBO_INDEX(module, combo, value))
 
 /** @brief Helper to set a line edit box and verify whether it succeeded
  * @param module KStars module that holds the line edit box
@@ -348,20 +355,30 @@ class TestEkosHelper : public QObject
 public:
     explicit TestEkosHelper(QString guider = nullptr);
 
+    // main optical train
+    QString m_primaryTrain = i18n("Primary");
+    QString m_guidingTrain = i18n("Secondary");
+
+    // predefined set of scopes
+    OAL::Scope *fsq85, *newton_10F4, *takfinder10x50;
+    enum ScopeType {SCOPE_FSQ85, SCOPE_NEWTON_10F4, SCOPE_TAKFINDER10x50};
+    // map scope type to predefined scope
+    OAL::Scope *getScope(ScopeType type);
+
     // Mount device
-    QString m_MountDevice = nullptr;
+    QString m_MountDevice = "";
     // CCD device
-    QString m_CCDDevice = nullptr;
+    QString m_CCDDevice = "";
     // Rotator device
-    QString m_RotatorDevice = nullptr;
+    QString m_RotatorDevice = "";
     // Guiding device
-    QString m_GuiderDevice = nullptr;
+    QString m_GuiderDevice = "";
     // Focusing device
-    QString m_FocuserDevice = nullptr;
+    QString m_FocuserDevice = "";
     // Flat light panel device
-    QString m_LightPanelDevice = nullptr;
+    QString m_LightPanelDevice = "";
     // Dome device
-    QString m_DomeDevice = nullptr;
+    QString m_DomeDevice = "";
 
     // Guider (PHD2 or Internal)
     QString m_Guider = "Internal";
@@ -373,6 +390,8 @@ public:
 
     // guiding used?
     bool use_guiding = false;
+    // did a dithering start?
+    bool dithered = false;
 
     // sequence of alignment states that are expected
     QQueue<Ekos::AlignState> expectedAlignStates;
@@ -383,11 +402,11 @@ public:
     // sequence of guiding states that are expected
     QQueue<Ekos::GuideState> expectedGuidingStates;
     // sequence of mount states that are expected
-    QQueue<ISD::Telescope::Status> expectedMountStates;
+    QQueue<ISD::Mount::Status> expectedMountStates;
     // sequence of dome states that are expected
     QQueue<ISD::Dome::Status> expectedDomeStates;
     // sequence of meridian flip states that are expected
-    QQueue<Ekos::Mount::MeridianFlipStatus> expectedMeridianFlipStates;
+    QQueue<Ekos::MeridianFlipState::MeridianFlipMountState> expectedMeridianFlipStates;
     // sequence of scheduler states that are expected
     QQueue<Ekos::SchedulerState> expectedSchedulerStates;
 
@@ -453,6 +472,31 @@ public:
      */
     void cleanupPHD2();
 
+    /**
+     * @brief Prepare two optical trains for test purposes, one for capturing and one for guiding
+     */
+    void prepareOpticalTrains();
+
+    /**
+     * @brief Helper function that ensures that alignment works in a test environment
+     */
+    void prepareAlignmentModule();
+
+    /**
+     * @brief Helper function that ensures that focusing works in a test environment
+     */
+    void prepareFocusModule();
+
+    /**
+     * @brief Helper function that ensures that guiding works in a test environment
+     */
+    void prepareGuidingModule();
+
+    /**
+     * @brief Prepare the mount module with the given mount parameters
+     */
+    void prepareMountModule(ScopeType primary = SCOPE_FSQ85, ScopeType guiding = SCOPE_TAKFINDER10x50);
+
     /**
      * @brief Slew to a given position
      * @param fast set to true if slewing should be fast using sync first close to the position
@@ -483,6 +527,10 @@ public:
      */
     bool checkAstrometryFiles();
 
+    /**
+     * @brief Returns true iff astrometry is available and checks if necessary.
+     */
+    bool isAstrometryAvailable();
 
     /**
      * @brief Set a tree view combo to a given value
@@ -521,7 +569,7 @@ public:
     /**
      * @brief Retrieve the current mount meridian flip status.
      */
-    Ekos::Mount::MeridianFlipStatus getMeridianFlipStatus() {return m_MFStatus;}
+    Ekos::MeridianFlipState::MeridianFlipMountState getMeridianFlipStatus() {return m_MFStatus;}
     /**
      * @brief Retrieve the current scheduler status.
      */
@@ -536,12 +584,23 @@ public:
      */
     void connectModules();
 
+    /**
+     * @brief Search for a certain scope in the database and create it if necessary
+     * @param model scope model
+     * łparam vendor scope vendor
+     * @param type scope type (refractor, newton, ...
+     * @param aperture scope aperture in mm
+     * @param focallenght focal length in mm
+     * @return scope object
+     */
+    Scope *createScopeIfNecessary(QString model, QString vendor, QString type, double aperture, double focallenght);
+
 private:
     // current mount status
-    ISD::Telescope::Status m_MountStatus { ISD::Telescope::MOUNT_IDLE };
+    ISD::Mount::Status m_MountStatus { ISD::Mount::MOUNT_IDLE };
 
     // current mount meridian flip status
-    Ekos::Mount::MeridianFlipStatus m_MFStatus { Ekos::Mount::FLIP_NONE };
+    Ekos::MeridianFlipState::MeridianFlipMountState m_MFStatus { Ekos::MeridianFlipState::MOUNT_FLIP_NONE };
 
     // current alignment status
     Ekos::AlignState m_AlignStatus { Ekos::ALIGN_IDLE };
@@ -574,13 +633,13 @@ private:
      * @brief Slot to track the mount status
      * @param status new mount status
      */
-    void mountStatusChanged(ISD::Telescope::Status status);
+    void mountStatusChanged(ISD::Mount::Status status);
 
     /**
      * @brief Slot to track the meridian flip stage of the mount
      * @param status new meridian flip state
      */
-    void meridianFlipStatusChanged(Ekos::Mount::MeridianFlipStatus status);
+    void meridianFlipStatusChanged(Ekos::MeridianFlipState::MeridianFlipMountState status);
 
     /**
      * @brief Slot to track the focus status
@@ -619,4 +678,7 @@ private:
      */
     void domeStatusChanged(ISD::Dome::Status status);
 
+private:
+    bool m_astrometry_available;
+
 };
