@@ -19,11 +19,10 @@
 #include "auxiliary/dms.h"
 #include "ksutils.h"
 #include "indicom.h"
-#include "indi/guimanager.h"
-#include "ekos/guide/internalguide/gmath.h"
 #include "Options.h"
 
-TestEkosMeridianFlipBase::TestEkosMeridianFlipBase(QObject *parent) : TestEkosMeridianFlipBase::TestEkosMeridianFlipBase("Internal", parent){}
+TestEkosMeridianFlipBase::TestEkosMeridianFlipBase(QObject *parent) :
+    TestEkosMeridianFlipBase::TestEkosMeridianFlipBase("Internal", parent) {}
 
 TestEkosMeridianFlipBase::TestEkosMeridianFlipBase(QString guider, QObject *parent) : QObject(parent)
 {
@@ -35,124 +34,16 @@ bool TestEkosMeridianFlipBase::startEkosProfile()
 {
     // use the helper to start the profile
     m_CaptureHelper->startEkosProfile();
-
-    Ekos::Manager * const ekos = Ekos::Manager::Instance();
-
-    // set mount defaults
-    KWRAP_SUB(QTRY_VERIFY_WITH_TIMEOUT(ekos->mountModule() != nullptr, 5000));
-    KWRAP_SUB(QTRY_VERIFY_WITH_TIMEOUT(Ekos::Manager::Instance()->mountModule()->slewStatus() != IPState::IPS_ALERT, 1000));
-    // set primary scope to my favorite FSQ-85 with QE 0.73 Reducer
-    KTRY_SET_DOUBLESPINBOX_SUB(ekos->mountModule(), primaryScopeFocalIN, 339.0);
-    KTRY_SET_DOUBLESPINBOX_SUB(ekos->mountModule(), primaryScopeApertureIN, 85.0);
-    // set guide scope to a Tak 7x50
-    KTRY_SET_DOUBLESPINBOX_SUB(ekos->mountModule(), guideScopeFocalIN, 170.0);
-    KTRY_SET_DOUBLESPINBOX_SUB(ekos->mountModule(), guideScopeApertureIN, 50.0);
-    // save values
-    KTRY_CLICK_SUB(Ekos::Manager::Instance()->mountModule(), saveB);
-
-    // set focus mode defaults
-    KWRAP_SUB(KTRY_SWITCH_TO_MODULE_WITH_TIMEOUT(ekos->focusModule(), 1000));
-    // use full field
-    KTRY_SET_CHECKBOX_SUB(ekos->focusModule(), useFullField, true);
-    //initial step size 5000
-    KTRY_SET_SPINBOX_SUB(ekos->focusModule(), stepIN, 5000);
-    // max travel 50000
-    KTRY_SET_DOUBLESPINBOX_SUB(ekos->focusModule(), maxTravelIN, 50000.0);
-    // focus tolerance 20% - make focus fast and robust, precision does not matter
-    KTRY_GADGET_SUB(Ekos::Manager::Instance()->focusModule(), QDoubleSpinBox, toleranceIN);
-    toleranceIN->setMaximum(20.0);
-    toleranceIN->setValue(20.0);
-    // use single pass linear algorithm
-    KTRY_SET_COMBO_SUB(ekos->focusModule(), focusAlgorithmCombo, "Linear 1 Pass");
-    // select star detection
-    KTRY_SET_COMBO_SUB(Ekos::Manager::Instance()->focusModule(), focusOptionsProfiles, "1-Focus-Default");
-    // set annulus to 0% - 50%
-    KTRY_SET_DOUBLESPINBOX_SUB(ekos->focusModule(), fullFieldInnerRing, 0.0);
-    KTRY_SET_DOUBLESPINBOX_SUB(ekos->focusModule(), fullFieldOuterRing, 50.0);
-    // try to make focusing fast, precision is not relevant here
-    KTRY_SET_DOUBLESPINBOX_SUB(ekos->focusModule(), initialFocusOutStepsIN, 2);
-    // select the Luminance filter
-    KTRY_SET_COMBO_SUB(Ekos::Manager::Instance()->focusModule(), FilterPosCombo, "Luminance");
-    // select SEP algorithm for star detection
-    KTRY_SET_COMBO_SUB(Ekos::Manager::Instance()->focusModule(), focusDetectionCombo, "SEP");
-    // set exp time for current filter
-    KTRY_SET_DOUBLESPINBOX_SUB(Ekos::Manager::Instance()->focusModule(), exposureIN, 3.0);
-    // set exposure times for all filters
-    Ekos::FilterManager *filtermanager = Ekos::Manager::Instance()->getFilterManager();
-    for (int pos = 0; pos < filtermanager->getFilterLabels().count(); pos++)
-    {
-        filtermanager->setFilterExposure(pos, 3.0);
-        filtermanager->setFilterLock(pos, "Luminance");
-    }
-    // gain 100
-    KTRY_SET_DOUBLESPINBOX_SUB(Ekos::Manager::Instance()->focusModule(), gainIN, 100.0);
-    // initial step size 5000
-    KTRY_SET_SPINBOX_SUB(Ekos::Manager::Instance()->focusModule(), stepIN, 5000);
-    // suspend guiding while focusing
-    KTRY_SET_CHECKBOX_SUB(Ekos::Manager::Instance()->focusModule(), suspendGuideCheck, true);
-
-    // check if astrometry files exist
-    if (! checkAstrometryFiles())
-    {
-        QWARN(QString("No astrometry index files found in %1").arg(
-                  KSUtils::getAstrometryDataDirs().join(", ")).toStdString().c_str());
-        astrometry_available = false;
-    }
-    // switch to alignment module
-    KWRAP_SUB(KTRY_SWITCH_TO_MODULE_WITH_TIMEOUT(ekos->alignModule(), 1000));
-    // set the CCD
-    KTRY_SET_COMBO_SUB(ekos->alignModule(), CCDCaptureCombo, m_CaptureHelper->m_CCDDevice);
-    // select the Luminance filter
-    KTRY_SET_COMBO_SUB(ekos->alignModule(), FilterPosCombo, "Luminance");
-    // select local solver
-    ekos->alignModule()->setSolverMode(Ekos::Align::SOLVER_LOCAL);
-    // select internal SEP method
-    Options::setSolveSextractorType(SSolver::EXTRACTOR_BUILTIN);
-    // select StellarSolver
-    Options::setSolverType(SSolver::SOLVER_STELLARSOLVER);
-    // select fast solve profile option
-    Options::setSolveOptionsProfile(SSolver::Parameters::SINGLE_THREAD_SOLVING);
-    // select the "Slew to Target" mode
-    KTRY_SET_RADIOBUTTON_SUB(ekos->alignModule(), slewR, true);
-
-    // switch to guiding module
-    KWRAP_SUB(KTRY_SWITCH_TO_MODULE_WITH_TIMEOUT(Ekos::Manager::Instance()->guideModule(), 1000));
-
-    // preserve guiding calibration as good as possible
-    Options::setResetGuideCalibration(false);
-    Options::setReuseGuideCalibration(true);
-
-    if (m_CaptureHelper->m_Guider == "PHD2")
-    {
-        KTRY_GADGET_SUB(Ekos::Manager::Instance()->guideModule(), QPushButton, externalConnectB);
-        // ensure that PHD2 is connected
-        if (externalConnectB->isEnabled())
-        {
-            // click "Connect"
-            KTRY_CLICK_SUB(Ekos::Manager::Instance()->guideModule(), externalConnectB);
-            // wait max 60 sec that PHD2 is connected (sometimes INDI connections hang)
-            KTRY_VERIFY_WITH_TIMEOUT_SUB(Ekos::Manager::Instance()->guideModule()->status() == Ekos::GUIDE_CONNECTED, 60000);
-            qCInfo(KSTARS_EKOS_TEST) << "PHD2 connected successfully.";
-        }
-    }
-    else
-    {
-        // select multi-star
-        Options::setGuideAlgorithm(SEP_MULTISTAR);
-        // select small star profile
-        Options::setGuideOptionsProfile(2);
-        // auto star select
-        KTRY_SET_CHECKBOX_SUB(Ekos::Manager::Instance()->guideModule(), autoStarCheck, true);
-        // set the guide star box to size 32
-        KTRY_SET_COMBO_SUB(Ekos::Manager::Instance()->guideModule(), boxSizeCombo, "32");
-        // Set the guiding and ST4 device
-        KTRY_SET_COMBO_SUB(Ekos::Manager::Instance()->guideModule(), guiderCombo, "CCD Simulator Guider");
-        KTRY_SET_COMBO_SUB(Ekos::Manager::Instance()->guideModule(), ST4Combo, m_CaptureHelper->m_MountDevice);
-        // select primary scope (higher focal length seems better for the guiding simulator)
-        KTRY_SET_COMBO_INDEX_SUB(Ekos::Manager::Instance()->guideModule(), FOVScopeCombo, ISD::CCD::TELESCOPE_PRIMARY);
-        // use two steps in each direction for calibration
-        Options::setAutoModeIterations(2);
-    }
+    // prepare optical trains for testing
+    m_CaptureHelper->prepareOpticalTrains();
+    // prepare the mount module for testing (OAG guiding seems more robust)
+    m_CaptureHelper->prepareMountModule(TestEkosHelper::SCOPE_FSQ85, TestEkosHelper::SCOPE_FSQ85);
+    // prepare for focusing tests
+    m_CaptureHelper->prepareFocusModule();
+    // prepare for alignment tests
+    m_CaptureHelper->prepareAlignmentModule();
+    // prepare for guiding tests
+    m_CaptureHelper->prepareGuidingModule();
 
     // Everything completed successfully
     return true;
@@ -190,6 +81,7 @@ void TestEkosMeridianFlipBase::cleanupTestCase()
 
 void TestEkosMeridianFlipBase::init()
 {
+    // initialize the capture helper
     m_CaptureHelper->init();
 
     // disable by default
@@ -198,13 +90,6 @@ void TestEkosMeridianFlipBase::init()
     use_aligning      = false;
     dithering_checked = false;
 
-    // clear the queues
-    m_CaptureHelper->expectedAlignStates.clear();
-    m_CaptureHelper->expectedCaptureStates.clear();
-    m_CaptureHelper->expectedFocusStates.clear();
-    m_CaptureHelper->expectedGuidingStates.clear();
-    m_CaptureHelper->expectedMeridianFlipStates.clear();
-
     // reset initial focuser position
     initialFocusPosition = -1;
 
@@ -219,9 +104,6 @@ void TestEkosMeridianFlipBase::init()
     // start the profile (depending on the selected guider)
     QVERIFY(startEkosProfile());
 
-    // initialize the capture helper
-    m_CaptureHelper->init();
-
     // disable FITS viewer
     Options::setUseFITSViewer(false);
 
@@ -236,9 +118,8 @@ void TestEkosMeridianFlipBase::init()
     GUIManager::Instance()->close();
 
     // clear guiding calibration to ensure proper guiding
-    KTRY_CLICK(Ekos::Manager::Instance()->guideModule(), clearCalibrationB);
-    // reuse calibration data
-    // Options::setSerializedCalibration("Cal v1.0,bx=1,by=1,pw=0.0052,ph=0.0052,fl=339,ang=287.544,angR=299.377,angD=185.712,ramspas=200.06,decmspas=135.882,swap=0,ra= 203:48:10,dec=00:25:52,side=0,when=2022-07-08 20:19:14,calEnd");
+    // TestEkosHelper::prepareGuidingModule() stores a guiding calibration, override if necessary
+    // KTRY_CLICK(Ekos::Manager::Instance()->guideModule(), clearCalibrationB);
 
     // switch to mount module
     KTRY_SWITCH_TO_MODULE_WITH_TIMEOUT(Ekos::Manager::Instance()->mountModule(), 1000);
@@ -247,8 +128,8 @@ void TestEkosMeridianFlipBase::init()
     QVERIFY(enableMeridianFlip(0.0));
 
     // expected beginning of the meridian flip
-    m_CaptureHelper->expectedMeridianFlipStates.enqueue(Ekos::Mount::FLIP_PLANNED);
-    m_CaptureHelper->expectedMeridianFlipStates.enqueue(Ekos::Mount::FLIP_RUNNING);
+    m_CaptureHelper->expectedMeridianFlipStates.enqueue(Ekos::MeridianFlipState::MOUNT_FLIP_PLANNED);
+    m_CaptureHelper->expectedMeridianFlipStates.enqueue(Ekos::MeridianFlipState::MOUNT_FLIP_RUNNING);
 }
 
 void TestEkosMeridianFlipBase::cleanup()
@@ -281,8 +162,8 @@ void TestEkosMeridianFlipBase::cleanup()
  * ********************************************************************************* */
 
 void TestEkosMeridianFlipBase::prepareTestData(double exptime, QList<QString> locationList, QList<bool> culminationList,
-                                               QList<std::pair<QString, int> > filterList, QList<bool> focusList, QList<bool> autofocusList,
-                                               QList<bool> guideList, QList<bool> ditherList)
+        QList<std::pair<QString, int> > filterList, QList<bool> focusList, QList<bool> autofocusList,
+        QList<bool> guideList, QList<bool> ditherList)
 {
 #if QT_VERSION < QT_VERSION_CHECK(5,9,0)
     QSKIP("Bypassing fixture test on old Qt");
@@ -339,9 +220,7 @@ bool TestEkosMeridianFlipBase::positionMountForMF(int secsToMF, bool fast)
 #if QT_VERSION < QT_VERSION_CHECK(5,9,0)
     QSKIP("Bypassing fixture test on old Qt");
     Q_UNUSED(secsToMF)
-    Q_UNUSED(calibrate)
-    Q_UNUSED(initialFocus)
-    Q_UNUSED(guideDeviation)
+    Q_UNUSED(fast)
 #else
     Ekos::Mount *mount = Ekos::Manager::Instance()->mountModule();
 
@@ -352,22 +231,23 @@ bool TestEkosMeridianFlipBase::positionMountForMF(int secsToMF, bool fast)
     // wait a certain time until the mount slews
     QTest::qWait(3000);
     // wait until the mount is tracking
-    KTRY_VERIFY_WITH_TIMEOUT_SUB(Ekos::Manager::Instance()->mountModule()->status() == ISD::Telescope::MOUNT_TRACKING, 60000);
+    KTRY_VERIFY_WITH_TIMEOUT_SUB(Ekos::Manager::Instance()->mountModule()->status() == ISD::Mount::MOUNT_TRACKING, 60000);
     // check whether a meridian flip is announced
-    KTRY_GADGET_SUB(mount, QLabel, meridianFlipStatusText);
-    KTRY_VERIFY_WITH_TIMEOUT_SUB(meridianFlipStatusText->text().length() > 0, 20000);
-    KTRY_VERIFY_TEXTFIELD_STARTS_WITH_TIMEOUT_SUB(meridianFlipStatusText, "Meridian flip in", 20000);
+    const QString flipmsg = Ekos::Manager::Instance()->mountModule()->meridianFlipStatusWidget->getStatus();
+    const int prefixlength = QString("Meridian flip in").length();
+    KWRAP_SUB(QTRY_VERIFY_WITH_TIMEOUT(flipmsg.length() >= prefixlength && flipmsg.left(prefixlength).compare("Meridian flip in") == 0, 20000));
 #endif
     // all checks succeeded
     return true;
 }
 
-bool TestEkosMeridianFlipBase::prepareMFTestcase(bool initialFocus, bool guideDeviation)
+bool TestEkosMeridianFlipBase::prepareMFTestcase(bool initialFocus, bool guideDeviation, bool initialGuideDeviation)
 {
 #if QT_VERSION < QT_VERSION_CHECK(5,9,0)
     QSKIP("Bypassing fixture test on old Qt");
     Q_UNUSED(initialFocus)
     Q_UNUSED(guideDeviation)
+    Q_UNUSED(initialGuideDeviation)
 #else
     // switch to capture module
     KWRAP_SUB(KTRY_SWITCH_TO_MODULE_WITH_TIMEOUT(Ekos::Manager::Instance()->captureModule(), 1000));
@@ -390,10 +270,10 @@ bool TestEkosMeridianFlipBase::prepareMFTestcase(bool initialFocus, bool guideDe
     Options::setDitherEnabled(dither);
 
     // set guide deviation guard
-    KTRY_SET_CHECKBOX_SUB(Ekos::Manager::Instance()->captureModule(), limitGuideDeviationS, guideDeviation);
+    KTRY_SET_CHECKBOX_SUB(Ekos::Manager::Instance()->captureModule(), limitGuideDeviationS, initialGuideDeviation);
     KTRY_SET_CHECKBOX_SUB(Ekos::Manager::Instance()->captureModule(), startGuiderDriftS, guideDeviation);
 
-    // create the capture sequences
+    // create the capture directory
     QTemporaryDir destination;
     KWRAP_SUB(QVERIFY(destination.isValid()));
     KWRAP_SUB(QVERIFY(destination.autoRemove()));
@@ -428,16 +308,17 @@ bool TestEkosMeridianFlipBase::prepareMFTestcase(bool initialFocus, bool guideDe
 #endif
 }
 
-bool TestEkosMeridianFlipBase::prepareCaptureTestcase(int secsToMF, bool initialFocus, bool guideDeviation)
+bool TestEkosMeridianFlipBase::prepareCaptureTestcase(int secsToMF, bool initialFocus, bool guideDeviation, bool initialGuideDeviation)
 {
 #if QT_VERSION < QT_VERSION_CHECK(5,9,0)
     QSKIP("Bypassing fixture test on old Qt");
     Q_UNUSED(secsToMF)
     Q_UNUSED(initialFocus)
     Q_UNUSED(guideDeviation)
+    Q_UNUSED(initialGuideDeviation)
 #else
     // execute preparation steps
-    KWRAP_SUB(QVERIFY(prepareMFTestcase(initialFocus, guideDeviation)));
+    KWRAP_SUB(QVERIFY(prepareMFTestcase(initialFocus, guideDeviation, initialGuideDeviation)));
 
     // slew close to the meridian
     QFETCH(bool, guide);
@@ -448,8 +329,9 @@ bool TestEkosMeridianFlipBase::prepareCaptureTestcase(int secsToMF, bool initial
     return true;
 }
 
-bool TestEkosMeridianFlipBase::prepareSchedulerTestcase(int secsToMF, bool useFocus, Ekos::Scheduler::SchedulerAlgorithm algorithm,
-                                                        SchedulerJob::CompletionCondition completionCondition, int iterations)
+bool TestEkosMeridianFlipBase::prepareSchedulerTestcase(int secsToMF, bool useFocus,
+        Ekos::Scheduler::SchedulerAlgorithm algorithm,
+        SchedulerJob::CompletionCondition completionCondition, int iterations)
 {
 #if QT_VERSION < QT_VERSION_CHECK(5,9,0)
     QSKIP("Bypassing fixture test on old Qt");
@@ -460,7 +342,7 @@ bool TestEkosMeridianFlipBase::prepareSchedulerTestcase(int secsToMF, bool useFo
     Q_UNUSED(iterations)
 #else
     // execute all similar preparation steps for a capture test case except for target slew
-    KVERIFY_SUB(prepareMFTestcase(useFocus, false));
+    KVERIFY_SUB(prepareMFTestcase(useFocus, false, false));
 
     // determine the target and sync close to it
     findMFTestTarget(secsToMF, false);
@@ -494,18 +376,19 @@ bool TestEkosMeridianFlipBase::prepareSchedulerTestcase(int secsToMF, bool useFo
     Options::setStopEkosAfterShutdown(false);
 
     // set the completion condition
-    switch (completionCondition) {
-    case SchedulerJob::FINISH_REPEAT:
-        // repeat the job for a fixed amount
-        KTRY_SET_RADIOBUTTON_SUB(scheduler, repeatCompletionR, true);
-        KTRY_SET_SPINBOX_SUB(scheduler, repeatsSpin, iterations);
-        break;
-    case SchedulerJob::FINISH_LOOP:
-        KTRY_SET_RADIOBUTTON_SUB(scheduler, loopCompletionR, true);
-        break;
-    default:
-        QWARN(QString("Unsupported completion condition %1!").arg(completionCondition).toStdString().c_str());
-        return false;
+    switch (completionCondition)
+    {
+        case SchedulerJob::FINISH_REPEAT:
+            // repeat the job for a fixed amount
+            KTRY_SET_RADIOBUTTON_SUB(scheduler, repeatCompletionR, true);
+            KTRY_SET_SPINBOX_SUB(scheduler, repeatsSpin, iterations);
+            break;
+        case SchedulerJob::FINISH_LOOP:
+            KTRY_SET_RADIOBUTTON_SUB(scheduler, loopCompletionR, true);
+            break;
+        default:
+            QWARN(QString("Unsupported completion condition %1!").arg(completionCondition).toStdString().c_str());
+            return false;
     }
     // add scheduler job
     KTRY_CLICK_SUB(scheduler, addToQueueB);
@@ -514,34 +397,17 @@ bool TestEkosMeridianFlipBase::prepareSchedulerTestcase(int secsToMF, bool useFo
     return true;
 }
 
-bool TestEkosMeridianFlipBase::checkAstrometryFiles()
-{
-    // first check the current configuration
-    QStringList dataDirs = KSUtils::getAstrometryDataDirs();
-
-    // search if configured directories contain some index files
-    for(QString dirname : dataDirs)
-    {
-        QDir dir(dirname);
-        dir.setNameFilters(QStringList("index*"));
-        dir.setFilter(QDir::Files);
-        if (! dir.entryList().isEmpty())
-            return true;
-    }
-    return false;
-}
-
-
 bool TestEkosMeridianFlipBase::checkDithering()
 {
     // if dithering is not selected, to nothing
     if (! dithering_checked)
         return true;
 
-    // check if dithering took place (failure doesn't matter in a test case)
-    KTRY_VERIFY_WITH_TIMEOUT_SUB(m_CaptureHelper->getGuidingStatus() == Ekos::GUIDE_DITHERING ||
-                                 m_CaptureHelper->getGuidingStatus() == Ekos::GUIDE_DITHERING_SUCCESS ||
-                                 m_CaptureHelper->getGuidingStatus() == Ekos::GUIDE_DITHERING_ERROR, 60000);
+    // reset the dithering flag
+    m_CaptureHelper->dithered = false;
+
+    // wait for 120s if dithering happens
+    KTRY_VERIFY_WITH_TIMEOUT_SUB(m_CaptureHelper->dithered == true, 120000);
     // all checks succeeded
     return true;
 }
@@ -555,8 +421,9 @@ bool TestEkosMeridianFlipBase::checkRefocusing()
         KTRY_VERIFY_WITH_TIMEOUT_SUB(m_CaptureHelper->getFocusStatus() == Ekos::FOCUS_PROGRESS, 60000);
         KVERIFY_EMPTY_QUEUE_WITH_TIMEOUT_SUB(m_CaptureHelper->expectedFocusStates, 60000);
         // check if focus completion is reached (successful or not)
-            KTRY_VERIFY_WITH_TIMEOUT_SUB(m_CaptureHelper->getFocusStatus() == Ekos::FOCUS_COMPLETE || m_CaptureHelper->getFocusStatus() == Ekos::FOCUS_FAILED ||
-                                         m_CaptureHelper->getFocusStatus() == Ekos::FOCUS_ABORTED, 120000);
+        KTRY_VERIFY_WITH_TIMEOUT_SUB(m_CaptureHelper->getFocusStatus() == Ekos::FOCUS_COMPLETE
+                                     || m_CaptureHelper->getFocusStatus() == Ekos::FOCUS_FAILED ||
+                                     m_CaptureHelper->getFocusStatus() == Ekos::FOCUS_ABORTED, 120000);
     }
     // focusing might have suspended guiding
     if (m_CaptureHelper->use_guiding)
@@ -565,25 +432,16 @@ bool TestEkosMeridianFlipBase::checkRefocusing()
     return true;
 }
 
-bool TestEkosMeridianFlipBase::startAligning(double expTime)
+bool TestEkosMeridianFlipBase::executeAlignment(double expTime)
 {
     // mark alignment
     use_aligning = true;
 
-    KWRAP_SUB(KTRY_SWITCH_TO_MODULE_WITH_TIMEOUT(Ekos::Manager::Instance()->alignModule(), 1000));
-    // set the exposure time to the given value
-    KTRY_SET_DOUBLESPINBOX_SUB(Ekos::Manager::Instance()->alignModule(), exposureIN, expTime);
-    // reduce the accuracy to avoid testing problems
-    KTRY_SET_SPINBOX_SUB(Ekos::Manager::Instance()->alignModule(), accuracySpin, 300);
-    KTRY_SET_COMBO_SUB(Ekos::Manager::Instance()->alignModule(), FilterPosCombo, "Luminance");
+    m_CaptureHelper->expectedAlignStates.enqueue(Ekos::ALIGN_COMPLETE);
+    KVERIFY_SUB(m_CaptureHelper->startAligning(expTime));
+    KVERIFY_EMPTY_QUEUE_WITH_TIMEOUT_SUB(m_CaptureHelper->expectedAlignStates, 60000);
 
-    // start alignment
-    KTRY_GADGET_SUB(Ekos::Manager::Instance()->alignModule(), QPushButton, solveB);
-    // ensure that the guiding button is enabled (after MF it may take a while)
-    KTRY_VERIFY_WITH_TIMEOUT_SUB(solveB->isEnabled(), 10000);
-    KTRY_CLICK_SUB(Ekos::Manager::Instance()->alignModule(), solveB);
-    KTRY_VERIFY_WITH_TIMEOUT_SUB(m_CaptureHelper->getAlignStatus() == Ekos::ALIGN_COMPLETE, 60000);
-    // all checks succeeded
+    // alignment succeeded
     return true;
 }
 
@@ -594,13 +452,14 @@ bool TestEkosMeridianFlipBase::stopAligning()
             m_CaptureHelper->getAlignStatus() == Ekos::ALIGN_FAILED || m_CaptureHelper->getAlignStatus() == Ekos::ALIGN_COMPLETE)
         return true;
 
-    // switch to guiding module
+    // switch to alignment module
     KWRAP_SUB(KTRY_SWITCH_TO_MODULE_WITH_TIMEOUT(Ekos::Manager::Instance()->alignModule(), 1000));
 
     // stop alignment
     KTRY_GADGET_SUB(Ekos::Manager::Instance()->alignModule(), QPushButton, stopB);
     KTRY_CLICK_SUB(Ekos::Manager::Instance()->alignModule(), stopB);
-    KTRY_VERIFY_WITH_TIMEOUT_SUB(m_CaptureHelper->getAlignStatus() == Ekos::ALIGN_IDLE || m_CaptureHelper->getAlignStatus() == Ekos::ALIGN_ABORTED ||
+    KTRY_VERIFY_WITH_TIMEOUT_SUB(m_CaptureHelper->getAlignStatus() == Ekos::ALIGN_IDLE
+                                 || m_CaptureHelper->getAlignStatus() == Ekos::ALIGN_ABORTED ||
                                  m_CaptureHelper->getAlignStatus() == Ekos::ALIGN_FAILED || m_CaptureHelper->getAlignStatus() == Ekos::ALIGN_COMPLETE, 5000);
     // all checks succeeded
     return true;
@@ -627,8 +486,9 @@ bool TestEkosMeridianFlipBase::startCapturing()
     KWRAP_SUB(KTRY_SWITCH_TO_MODULE_WITH_TIMEOUT(Ekos::Manager::Instance()->captureModule(), 1000));
 
     // check if capture is in a stopped state
-    KWRAP_SUB(QVERIFY(m_CaptureHelper->getCaptureStatus() == Ekos::CAPTURE_IDLE || m_CaptureHelper->getCaptureStatus() == Ekos::CAPTURE_ABORTED
-                      || m_CaptureHelper->getCaptureStatus() == Ekos::CAPTURE_SUSPENDED|| m_CaptureHelper->getCaptureStatus() == Ekos::CAPTURE_COMPLETE));
+    Ekos::CaptureState capturestate = m_CaptureHelper->getCaptureStatus();
+    KWRAP_SUB(QVERIFY(capturestate == Ekos::CAPTURE_IDLE || capturestate == Ekos::CAPTURE_ABORTED ||
+                      capturestate == Ekos::CAPTURE_SUSPENDED || capturestate == Ekos::CAPTURE_COMPLETE));
 
     // press start
     m_CaptureHelper->expectedCaptureStates.enqueue(Ekos::CAPTURE_CAPTURING);
@@ -636,7 +496,7 @@ bool TestEkosMeridianFlipBase::startCapturing()
     KTRY_CLICK_SUB(Ekos::Manager::Instance()->captureModule(), startB);
 
     // check if capturing has started
-    KVERIFY_EMPTY_QUEUE_WITH_TIMEOUT_SUB(m_CaptureHelper->expectedCaptureStates, 5000);
+    KVERIFY_EMPTY_QUEUE_WITH_TIMEOUT_SUB(m_CaptureHelper->expectedCaptureStates, 15000);
 #endif
     // all checks succeeded
     return true;
@@ -645,8 +505,10 @@ bool TestEkosMeridianFlipBase::startCapturing()
 bool TestEkosMeridianFlipBase::stopCapturing()
 {
     // check if capture is in a stopped state
-    if (m_CaptureHelper->getCaptureStatus() == Ekos::CAPTURE_IDLE || m_CaptureHelper->getCaptureStatus() == Ekos::CAPTURE_ABORTED ||
-            m_CaptureHelper->getCaptureStatus() == Ekos::CAPTURE_COMPLETE || m_CaptureHelper->getCaptureStatus() == Ekos::CAPTURE_PAUSED ||
+    if (m_CaptureHelper->getCaptureStatus() == Ekos::CAPTURE_IDLE
+            || m_CaptureHelper->getCaptureStatus() == Ekos::CAPTURE_ABORTED ||
+            m_CaptureHelper->getCaptureStatus() == Ekos::CAPTURE_COMPLETE
+            || m_CaptureHelper->getCaptureStatus() == Ekos::CAPTURE_PAUSED ||
             m_CaptureHelper->getCaptureStatus() == Ekos::CAPTURE_MERIDIAN_FLIP)
         return true;
 
@@ -668,15 +530,16 @@ bool TestEkosMeridianFlipBase::startScheduler()
 {
     Ekos::Scheduler *scheduler = Ekos::Manager::Instance()->schedulerModule();
     // set expected states
-    m_CaptureHelper->expectedMountStates.append(ISD::Telescope::MOUNT_SLEWING);
-    m_CaptureHelper->expectedMountStates.append(ISD::Telescope::MOUNT_TRACKING);
+    m_CaptureHelper->expectedMountStates.append(ISD::Mount::MOUNT_SLEWING);
+    m_CaptureHelper->expectedMountStates.append(ISD::Mount::MOUNT_TRACKING);
     if (m_CaptureHelper->use_guiding == true)
         m_CaptureHelper->expectedGuidingStates.append(Ekos::GUIDE_GUIDING);
     m_CaptureHelper->expectedCaptureStates.append(Ekos::CAPTURE_CAPTURING);
 
     // switch to the scheduler module and start
     KWRAP_SUB(KTRY_SWITCH_TO_MODULE_WITH_TIMEOUT(scheduler, 1000));
-    KVERIFY_SUB(m_CaptureHelper->getSchedulerStatus() == Ekos::SCHEDULER_IDLE || m_CaptureHelper->getSchedulerStatus() == Ekos::SCHEDULER_ABORTED);
+    KVERIFY_SUB(m_CaptureHelper->getSchedulerStatus() == Ekos::SCHEDULER_IDLE
+                || m_CaptureHelper->getSchedulerStatus() == Ekos::SCHEDULER_ABORTED);
     KTRY_CLICK_SUB(scheduler, startB);
 
     // check mount slew and tracking
@@ -695,7 +558,8 @@ bool TestEkosMeridianFlipBase::stopScheduler()
     Ekos::Scheduler *scheduler = Ekos::Manager::Instance()->schedulerModule();
     // switch to the capture module
     KWRAP_SUB(KTRY_SWITCH_TO_MODULE_WITH_TIMEOUT(scheduler, 1000));
-    if (m_CaptureHelper->getSchedulerStatus() == Ekos::SCHEDULER_RUNNING || m_CaptureHelper->getSchedulerStatus() == Ekos::SCHEDULER_PAUSED)
+    if (m_CaptureHelper->getSchedulerStatus() == Ekos::SCHEDULER_RUNNING
+            || m_CaptureHelper->getSchedulerStatus() == Ekos::SCHEDULER_PAUSED)
         KTRY_CLICK_SUB(scheduler, startB);
     // all checks succeeded
     return true;
@@ -739,7 +603,8 @@ bool TestEkosMeridianFlipBase::stopFocusing()
     KTRY_GADGET_SUB(Ekos::Manager::Instance()->focusModule(), QPushButton, stopFocusB);
     if (stopFocusB->isEnabled())
         KTRY_CLICK_SUB(Ekos::Manager::Instance()->focusModule(), stopFocusB);
-    KTRY_VERIFY_WITH_TIMEOUT_SUB(m_CaptureHelper->getFocusStatus() == Ekos::FOCUS_IDLE || m_CaptureHelper->getFocusStatus() == Ekos::FOCUS_COMPLETE ||
+    KTRY_VERIFY_WITH_TIMEOUT_SUB(m_CaptureHelper->getFocusStatus() == Ekos::FOCUS_IDLE
+                                 || m_CaptureHelper->getFocusStatus() == Ekos::FOCUS_COMPLETE ||
                                  m_CaptureHelper->getFocusStatus() == Ekos::FOCUS_ABORTED || m_CaptureHelper->getFocusStatus() == Ekos::FOCUS_FAILED, 15000);
 
     // all checks succeeded
@@ -749,11 +614,9 @@ bool TestEkosMeridianFlipBase::stopFocusing()
 bool TestEkosMeridianFlipBase::enableMeridianFlip(double delay)
 {
     Ekos::Manager * const ekos = Ekos::Manager::Instance();
-    KTRY_SET_CHECKBOX_SUB(ekos->mountModule(), meridianFlipCheckBox, true);
-    // set the delay value to degrees
-    KTRY_SET_RADIOBUTTON_SUB(ekos->mountModule(), meridianFlipDegreesR, true);
+    KTRY_SET_CHECKBOX_SUB(ekos->mountModule(), executeMeridianFlip, true);
     // set the delay in degrees
-    KTRY_SET_DOUBLESPINBOX_SUB(ekos->mountModule(), meridianFlipTimeBox, 15.0 * delay / 3600.0);
+    KTRY_SET_DOUBLESPINBOX_SUB(ekos->mountModule(), meridianFlipOffsetDegrees, 15.0 * delay / 3600.0);
     // all checks succeeded
     return true;
 }
@@ -775,7 +638,7 @@ bool TestEkosMeridianFlipBase::checkMFExecuted(int startDelay)
     if (dithering_checked)
         Options::setDitherFrames(1);
     // check if the meridian flip is completed latest after one minute
-    m_CaptureHelper->expectedMeridianFlipStates.enqueue(Ekos::Mount::FLIP_COMPLETED);
+    m_CaptureHelper->expectedMeridianFlipStates.enqueue(Ekos::MeridianFlipState::MOUNT_FLIP_COMPLETED);
     KVERIFY_EMPTY_QUEUE_WITH_TIMEOUT_SUB(m_CaptureHelper->expectedMeridianFlipStates, 60000);
     // all checks succeeded
     return true;
@@ -816,7 +679,7 @@ bool TestEkosMeridianFlipBase::checkPostMFBehavior()
 
 int TestEkosMeridianFlipBase::secondsToMF()
 {
-    const QString flipmsg = Ekos::Manager::Instance()->mountModule()->meridianFlipStatusText->text();
+    const QString flipmsg = Ekos::Manager::Instance()->mountModule()->meridianFlipStatusWidget->getStatus();
 
     QRegExp mfPattern("Meridian flip in (\\d+):(\\d+):(\\d+)");
 
@@ -852,7 +715,7 @@ void TestEkosMeridianFlipBase::findMFTestTarget(int secsToMF, bool fast)
     // calculate a feasible declination depending to the location's latitude
     // for the upper culmination, we use an azimuth of 45 deg, for the lower culmination half way between pole and horizont
     double lat = KStarsData::Instance()->geo()->lat()->Degrees();
-    target = new SkyPoint(range24(meridianRA + delta), culmination ? (lat-45) : (90-lat/2));
+    target = new SkyPoint(range24(meridianRA + delta), culmination ? (lat - 45) : (90 - lat / 2));
     if (fast)
     {
         // reset mount model
