@@ -91,8 +91,9 @@ protected:
      * @brief General preparations used both for pure capturing test cases as well as those with the scheduler.
      * @param initialFocus execute upfront focusing
      * @param guideDeviation select "Abort if Guide Deviation"
+     * @param initialGuideDeviation select "Only Start if Guide Deviation <"
      */
-    bool prepareMFTestcase(bool initialFocus, bool guideDeviation);
+    bool prepareMFTestcase(bool initialFocus, bool guideDeviation, bool initialGuideDeviation);
 
     /**
      * @brief Helper function that reads capture sequence test data, creates entries in the capture module,
@@ -100,8 +101,9 @@ protected:
      * @param secsToMF seconds until the meridian will be crossed
      * @param initialFocus execute upfront focusing
      * @param guideDeviation select "Abort if Guide Deviation"
+     * @param initialGuideDeviation select "Only Start if Guide Deviation <"
      */
-    bool prepareCaptureTestcase(int secsToMF, bool initialFocus, bool guideDeviation);
+    bool prepareCaptureTestcase(int secsToMF, bool initialFocus, bool guideDeviation, bool initialGuideDeviation);
 
     /**
      * @brief Prepare the scheduler with a single based upon the capture sequences filled
@@ -128,12 +130,6 @@ protected:
     void prepareTestData(double exptime, QList<QString> locationList, QList<bool> culminationList, QList<std::pair<QString, int> > filterList,
                          QList<bool> focusList, QList<bool> autofocusList, QList<bool> guideList, QList<bool> ditherList);
 
-    /**
-     * @brief Check if astrometry files exist.
-     * @return true iff astrometry files found
-     */
-    bool checkAstrometryFiles();
-
     /**
      * @brief Check if guiding and dithering is restarted if required.
      */
@@ -148,7 +144,7 @@ protected:
      * @brief Helper function for start of alignment
      * @param exposure time
      */
-    bool startAligning(double expTime);
+    bool executeAlignment(double expTime);
 
     /**
      * @brief Helper function to stop guiding
