@@ -9,14 +9,14 @@
 
 #pragma once
 
+#include "ekos/auxiliary/filtermanager.h"
 #include "ui_align.h"
 #include "ekos/ekos.h"
-#include "indi/indiccd.h"
+#include "indi/indicamera.h"
 #include "indi/indistd.h"
-#include "indi/inditelescope.h"
+#include "indi/indimount.h"
 #include "indi/indidome.h"
 #include "ksuserdb.h"
-#include "ekos/auxiliary/filtermanager.h"
 #include "ekos/auxiliary/darkprocessor.h"
 
 #include <QTime>
@@ -68,7 +68,7 @@ class ManualRotator;
  * Align module provide Polar Align Helper tool which enables easy-to-follow polar alignment procedure given wide FOVs (> 1.5 degrees)
  * Legacy polar aligment is deprecated.
  *@author Jasem Mutlaq
- *@version 1.5
+ *@version 2.0
  */
 class Align : public QWidget, public Ui::Align
 {
@@ -76,8 +76,9 @@ class Align : public QWidget, public Ui::Align
         Q_CLASSINFO("D-Bus Interface", "org.kde.kstars.Ekos.Align")
         Q_PROPERTY(Ekos::AlignState status READ status NOTIFY newStatus)
         Q_PROPERTY(QStringList logText READ logText NOTIFY newLog)
-        Q_PROPERTY(QString camera READ camera WRITE setCamera)
-        Q_PROPERTY(QString filterWheel READ filterWheel WRITE setFilterWheel)
+        Q_PROPERTY(QString opticalTrain READ opticalTrain WRITE setOpticalTrain)
+        Q_PROPERTY(QString camera READ camera)
+        Q_PROPERTY(QString filterWheel READ filterWheel)
         Q_PROPERTY(QString filter READ filter WRITE setFilter)
         Q_PROPERTY(double exposure READ exposure WRITE setExposure)
         Q_PROPERTY(QList<double> fov READ fov)
@@ -86,7 +87,7 @@ class Align : public QWidget, public Ui::Align
         //Q_PROPERTY(QString solverArguments READ solverArguments WRITE setSolverArguments)
 
     public:
-        explicit Align(ProfileInfo *activeProfile);
+        explicit Align(const QSharedPointer<ProfileInfo> &activeProfile);
         virtual ~Align() override;
 
         typedef enum { GOTO_SYNC, GOTO_SLEW, GOTO_NOTHING } GotoMode;
@@ -116,7 +117,6 @@ class Align : public QWidget, public Ui::Align
              * @param device CCD device name
              * @return Returns true if device if found and selected, false otherwise.
              */
-        Q_SCRIPTABLE bool setCamera(const QString &device);
         Q_SCRIPTABLE QString camera();
 
         /** DBUS interface function.
@@ -124,7 +124,6 @@ class Align : public QWidget, public Ui::Align
              * @param device The filter device name
              * @return Returns true if filter device is found and set, false otherwise.
              */
-        Q_SCRIPTABLE bool setFilterWheel(const QString &device);
         Q_SCRIPTABLE QString filterWheel();
 
         /** DBUS interface function.
@@ -179,17 +178,7 @@ class Align : public QWidget, public Ui::Align
         Q_SCRIPTABLE Q_NOREPLY void setExposure(double value);
         Q_SCRIPTABLE double exposure()
         {
-            return exposureIN->value();
-        }
-
-        /** DBUS interface function.
-             * Sets the telescope type (PRIMARY or GUIDE) that should be used for FOV calculations. This value is loaded form driver settings by default.
-             * @param index 0 for PRIMARY telescope, 1 for GUIDE telescope
-             */
-        Q_SCRIPTABLE Q_NOREPLY void setFOVTelescopeType(int index);
-        int FOVTelescopeType()
-        {
-            return FOVScopeCombo->currentIndex();
+            return alignExposure->value();
         }
 
         /** DBUS interface function.
@@ -207,57 +196,57 @@ class Align : public QWidget, public Ui::Align
         /** @}*/
 
         /**
-             * @brief Add CCD to the list of available CCD.
-             * @param newCCD pointer to CCD device.
+             * @brief Add Camera to the list of available Cameras.
+             * @param device pointer to camera device.
+             * @return True if added successfully, false if duplicate or failed to add.
              */
-        void addCCD(ISD::GDInterface *newCCD);
+        bool setCamera(ISD::Camera *device);
 
         /**
-             * @brief addFilter Add filter to the list of available filters.
-             * @param newFilter pointer to filter device.
+             * @brief addFilterWheel Add new filter wheel filter device.
+             * @param device pointer to filter device.
+             * @return True if added successfully, false if duplicate or failed to add.
              */
-        void addFilter(ISD::GDInterface *newFilter);
+        bool setFilterWheel(ISD::FilterWheel *device);
 
         /**
-             * @brief Set the current telescope
-             * @param newTelescope pointer to telescope device.
+             * @brief Add new mount
+             * @param device pointer to mount device.
+             * @return True if added successfully, false if duplicate or failed to add.
              */
-        void setTelescope(ISD::GDInterface *newTelescope);
+        bool setMount(ISD::Mount *device);
 
         /**
-             * @brief Set the current dome
-             * @param newDome pointer to telescope device.
+             * @brief Add new Dome
+             * @param device pointer to dome device.
+             * @return True if added successfully, false if duplicate or failed to add.
              */
-        void setDome(ISD::GDInterface *newDome);
-
-        void setRotator(ISD::GDInterface *newRotator);
-
-        void removeDevice(ISD::GDInterface *device);
+        bool setDome(ISD::Dome *device);
 
         /**
-             * @brief Set telescope and guide scope info. All measurements is in millimeters.
-             * @param primaryFocalLength Primary Telescope Focal Length. Set to 0 to skip setting this value.
-             * @param primaryAperture Primary Telescope Aperture. Set to 0 to skip setting this value.
-             * @param guideFocalLength Guide Telescope Focal Length. Set to 0 to skip setting this value.
-             * @param guideAperture Guide Telescope Aperture. Set to 0 to skip setting this value.
-        */
-        void setTelescopeInfo(double primaryFocalLength, double primaryAperture, double guideFocalLength, double guideAperture);
+             * @brief Add new Rotator
+             * @param device pointer to rotator device.
+             * @return True if added successfully, false if duplicate or failed to add.
+             */
+        bool setRotator(ISD::Rotator *device);
+
+        void removeDevice(const QSharedPointer<ISD::GenericDevice> &device);
 
         /**
              * @brief setAstrometryDevice
              * @param newAstrometry
              */
-        void setAstrometryDevice(ISD::GDInterface *newAstrometry);
+        void setAstrometryDevice(const QSharedPointer<ISD::GenericDevice> &device);
 
         /**
              * @brief CCD information is updated, sync them.
              */
-        void syncCCDInfo();
+        void syncCameraInfo();
 
         /**
          * @brief syncCCDControls Update camera controls like gain, offset, ISO..etc.
          */
-        void syncCCDControls();
+        void syncCameraControls();
 
         /**
              * @brief Generate arguments we pass to the remote solver.
@@ -299,7 +288,10 @@ class Align : public QWidget, public Ui::Align
          */
         void getCalculatedFOVScale(double &fov_w, double &fov_h, double &fov_scale);
 
-        void setFilterManager(const QSharedPointer<FilterManager> &manager);
+        void setupFilterManager();
+        void setupPlot();
+        void setupSolutionTable();
+        void setupOptions();
 
         /**
              * @brief Sync the telescope to the solved alignment coordinate.
@@ -367,22 +359,22 @@ class Align : public QWidget, public Ui::Align
              * @brief Check CCD and make sure information is updated and FOV is re-calculated.
              * @param CCDNum By default, we check the already selected CCD in the dropdown menu. If CCDNum is specified, the check is made against this specific CCD in the dropdown menu. CCDNum is the index of the CCD in the dropdown menu.
              */
-        void checkCCD(int CCDNum = -1);
+        void checkCamera();
 
         /**
              * @brief Check Filter and make sure information is updated accordingly.
              * @param filterNum By default, we check the already selected filter in the dropdown menu. If filterNum is specified, the check is made against this specific filter in the dropdown menu.
              *  filterNum is the index of the filter in the dropdown menu.
              */
-        void checkFilter(int filterNum = -1);
+        void checkFilter();
 
         /**
-             * @brief checkCCDExposureProgress Track the progress of CCD exposure
+             * @brief checkCameraExposureProgress Track the progress of CCD exposure
              * @param targetChip Target chip under exposure
              * @param remaining how many seconds remaining
              * @param state status of exposure
              */
-        void checkCCDExposureProgress(ISD::CCDChip *targetChip, double remaining, IPState state);
+        void checkCameraExposureProgress(ISD::CameraChip *targetChip, double remaining, IPState state);
         /**
              * @brief Process new FITS received from CCD.
              * @param bp pointer to blob property
@@ -466,24 +458,15 @@ class Align : public QWidget, public Ui::Align
           */
         void setTarget(const SkyPoint &targetCoord);
 
-        /**
-         * @brief Clear the target, make it invalid.
-         */
-        Q_SCRIPTABLE Q_NOREPLY void clearTarget()
-        {
-            m_targetCoordValid = false;
-        }
-
         /**
          * @brief Set the coordinates that the mount reports as its position
          * @param position current mount position
          */
         void setTelescopeCoordinates(const SkyPoint &position)
         {
-            telescopeCoord = position;
+            m_TelescopeCoord = position;
         }
 
-
         Q_SCRIPTABLE Q_NOREPLY void setTargetPositionAngle(double value);
 
         /** DBUS interface function.
@@ -506,12 +489,6 @@ class Align : public QWidget, public Ui::Align
 
         void solverComplete();
 
-        /**
-         * @brief If the target is valid (see m_targetCoordValid), simply return. If the target is not valid,
-         * use thecurrent mount coordinates as target coordinates.
-         */
-        void updateTargetCoords();
-
         /**
              * @brief Process solver failure.
              */
@@ -533,11 +510,7 @@ class Align : public QWidget, public Ui::Align
         // Update Capture Module status
         void setCaptureStatus(Ekos::CaptureState newState);
         // Update Mount module status
-        void setMountStatus(ISD::Telescope::Status newState);
-
-        // Align Settings
-        QJsonObject getSettings() const;
-        void setSettings(const QJsonObject &settings);
+        void setMountStatus(ISD::Mount::Status newState);
 
         void zoomAlignView();
         void setAlignZoom(double scale);
@@ -545,18 +518,25 @@ class Align : public QWidget, public Ui::Align
         // Manual Rotator Dialog
         void toggleManualRotator(bool toggled);
 
+        // Settings
+        QVariantMap getAllSettings() const;
+        void setAllSettings(const QVariantMap &settings);
+
+        // Trains
+        QString opticalTrain() const
+        {
+            return opticalTrainCombo->currentText();
+        }
+        void setOpticalTrain(const QString &value)
+        {
+            opticalTrainCombo->setCurrentText(value);
+        }
+
     private slots:
-
-        void setDefaultCCD(QString ccd);
-
-        void saveSettleTime();
-
         // Solver timeout
         void checkAlignmentTimeout();
         void setAlignTableResult(AlignResult result);
 
-        void updateTelescopeType(int index);
-
         // External View
         void showFITSViewer();
         void toggleAlignWidgetFullScreen();
@@ -565,7 +545,7 @@ class Align : public QWidget, public Ui::Align
          * @brief prepareCapture Set common settings for capture for align module
          * @param targetChip target Chip
          */
-        void prepareCapture(ISD::CCDChip *targetChip);
+        void prepareCapture(ISD::CameraChip *targetChip);
 
         //Solutions Display slots
         void buildTarget();
@@ -577,10 +557,11 @@ class Align : public QWidget, public Ui::Align
         void slotRemoveSolutionPoint();
         void slotAutoScaleGraph();
 
+        // Model
         void slotMountModel();
 
-        // Settings
-        void syncSettings();
+        // Capture Timeout
+        void processCaptureTimeout();
 
     protected slots:
         /**
@@ -608,13 +589,52 @@ class Align : public QWidget, public Ui::Align
         // Send new solver results
         void newSolverResults(double orientation, double ra, double dec, double pixscale);
 
+        // Train changed
+        void trainChanged();
+
         // Settings
-        void settingsUpdated(const QJsonObject &settings);
+        void settingsUpdated(const QVariantMap &settings);
 
         // Manual Rotator
         void manualRotatorChanged(double currentPA, double targetPA, double threshold);
 
     private:
+
+        void setupOpticalTrainManager();
+        void refreshOpticalTrain();        
+
+        ////////////////////////////////////////////////////////////////////
+        /// Settings
+        ////////////////////////////////////////////////////////////////////
+
+        /**
+         * @brief Connect GUI elements to sync settings once updated.
+         */
+        void connectSettings();
+        /**
+         * @brief Stop updating settings when GUI elements are updated.
+         */
+        void disconnectSettings();
+        /**
+         * @brief loadSettings Load setting from Options and set them accordingly.
+         */
+        void loadGlobalSettings();
+
+        /**
+         * @brief syncSettings When checkboxes, comboboxes, or spin boxes are updated, save their values in the
+         * global and per-train settings.
+         */
+        void syncSettings();
+
+        /**
+         * @brief syncControl Sync setting to widget. The value depends on the widget type.
+         * @param settings Map of all settings
+         * @param key name of widget to sync
+         * @param widget pointer of widget to set
+         * @return True if sync successful, false otherwise
+         */
+        bool syncControl(const QVariantMap &settings, const QString &key, QWidget * widget);
+
         /**
          * @brief Retrieve the align status indicator
          */
@@ -678,17 +698,17 @@ class Align : public QWidget, public Ui::Align
         /**
          * @brief initPolarAlignmentAssistant Initialize Polar Alignment Asssistant Tool
          */
-        void initPolarAlignmentAssistant();
+        void setupPolarAlignmentAssistant();
 
         /**
          * @brief initManualRotator Initialize Manual Rotator Tool
          */
-        void initManualRotator();
+        void setupManualRotator();
 
         /**
          * @brief initDarkProcessor Initialize Dark Processor
          */
-        void initDarkProcessor();
+        void setuptDarkProcessor();
 
         bool matchPAHStage(uint32_t stage);
 
@@ -725,13 +745,17 @@ class Align : public QWidget, public Ui::Align
         // FOV
         double m_CameraPixelWidth { -1 };
         double m_CameraPixelHeight { -1 };
-        double m_TelescopeFocalLength { -1 };
-        double m_TelescopeAperture { -1 };
+        uint16_t m_CameraWidth { 0 };
+        uint16_t m_CameraHeight { 0 };
+
+        double m_FocalLength {-1};
+        double m_Aperture {-1};
+        double m_FocalRatio {-1};
+        double m_Reducer = {-1};
+
         double m_FOVWidth { 0 };
         double m_FOVHeight { 0 };
         double m_FOVPixelScale { 0 };
-        uint16_t m_CameraWidth { 0 };
-        uint16_t m_CameraHeight { 0 };
 
         // Keep track of solver results
         double sOrientation { INVALID_VALUE };
@@ -739,15 +763,11 @@ class Align : public QWidget, public Ui::Align
         double sDEC { INVALID_VALUE };
 
         /// Solver alignment coordinates
-        SkyPoint alignCoord;
+        SkyPoint m_AlignCoord;
         /// Target coordinates we need to slew to
-        SkyPoint m_targetCoord;
-        /// do we have valid target coordinates?
-        bool m_targetCoordValid = false;
+        SkyPoint m_TargetCoord;
         /// Current telescope coordinates
-        SkyPoint telescopeCoord;
-        /// Coord from Load & Slew
-        SkyPoint loadSlewCoord;
+        SkyPoint m_TelescopeCoord;
         /// Difference between solution and target coordinate
         double m_TargetDiffTotal { 1e6 };
         double m_TargetDiffRA { 1e6 };
@@ -776,19 +796,15 @@ class Align : public QWidget, public Ui::Align
         // Online and Offline parsers
         AstrometryParser* parser { nullptr };
         std::unique_ptr<RemoteAstrometryParser> remoteParser;
-        ISD::GDInterface *remoteParserDevice { nullptr };
+        QSharedPointer<ISD::GenericDevice> m_RemoteParserDevice;
 
         // Pointers to our devices
-        ISD::Telescope *currentTelescope { nullptr };
-        ISD::Dome *currentDome { nullptr };
-        ISD::CCD *currentCCD { nullptr };
-        ISD::GDInterface *currentRotator { nullptr };
-        QList<ISD::CCD *> CCDs;
+        ISD::Mount *m_Mount { nullptr };
+        ISD::Dome *m_Dome { nullptr };
+        ISD::Camera *m_Camera { nullptr };
+        ISD::Rotator *m_Rotator { nullptr };
+        ISD::FilterWheel *m_FilterWheel { nullptr };
 
-        /// Optional device filter
-        ISD::GDInterface *currentFilter { nullptr };
-        /// They're generic GDInterface because they could be either ISD::CCD or ISD::Filter
-        QList<ISD::GDInterface *> Filters;
         int currentFilterPosition { -1 };
         /// True if we need to change filter position and wait for result before continuing capture
         bool filterPositionPending { false };
@@ -816,7 +832,7 @@ class Align : public QWidget, public Ui::Align
         CaptureState m_CaptureState { CAPTURE_IDLE };
 
         // Track which upload mode the CCD is set to. If set to UPLOAD_LOCAL, then we need to switch it to UPLOAD_CLIENT in order to do focusing, and then switch it back to UPLOAD_LOCAL
-        ISD::CCD::UploadMode rememberUploadMode { ISD::CCD::UPLOAD_CLIENT };
+        ISD::Camera::UploadMode rememberUploadMode { ISD::Camera::UPLOAD_CLIENT };
 
         GotoMode m_CurrentGotoMode;
 
@@ -857,9 +873,7 @@ class Align : public QWidget, public Ui::Align
         QCPCurve *concentricRings { nullptr };
 
         // Telescope Settings
-        ISD::CCD::TelescopeType rememberTelescopeType = { ISD::CCD::TELESCOPE_UNKNOWN };
-        double primaryFL = -1, primaryAperture = -1, guideFL = -1, guideAperture = -1;
-        double primaryEffectiveFL = -1, guideEffectiveFL = -1;
+        double effectiveFocalLength = -1;
         bool m_isRateSynced = false;
         bool domeReady = true;
 
@@ -867,17 +881,14 @@ class Align : public QWidget, public Ui::Align
         bool m_RememberCameraFastExposure = { false };
 
         // Controls
-        double GainSpinSpecialValue {INVALID_VALUE};
+        double alignGainSpecialValue {INVALID_VALUE};
         double TargetCustomGainValue {-1};
 
-        // Filter Manager
-        QSharedPointer<FilterManager> filterManager;
-
         // Data
         QSharedPointer<FITSData> m_ImageData;
 
         // Active Profile
-        ProfileInfo *m_ActiveProfile { nullptr };
+        QSharedPointer<ProfileInfo> m_ActiveProfile;
 
         // Threshold to notify settle time is 3 seconds
         static constexpr uint16_t DELAY_THRESHOLD_NOTIFY { 3000 };
@@ -892,5 +903,11 @@ class Align : public QWidget, public Ui::Align
         // Dark Processor
         QPointer<DarkProcessor> m_DarkProcessor;
 
+        // Filter Manager
+        QSharedPointer<FilterManager> m_FilterManager;
+
+        // Settings
+        QVariantMap m_Settings;
+        QVariantMap m_GlobalSettings;
 };
 }
