@@ -7,18 +7,18 @@
 #pragma once
 
 #include "ui_capture.h"
+#include "capturemodulestate.h"
 #include "capturedeviceadaptor.h"
 #include "sequencejobstate.h"
+#include "ekos/manager/meridianflipstate.h"
 #include "customproperties.h"
-#include "oal/filter.h"
 #include "ekos/ekos.h"
 #include "ekos/mount/mount.h"
-#include "indi/indiccd.h"
-#include "indi/indicap.h"
+#include "indi/indicamera.h"
+#include "indi/indidustcap.h"
 #include "indi/indidome.h"
 #include "indi/indilightbox.h"
-#include "indi/inditelescope.h"
-#include "ekos/auxiliary/filtermanager.h"
+#include "indi/indimount.h"
 #include "ekos/scheduler/schedulerjob.h"
 #include "ekos/auxiliary/darkprocessor.h"
 #include "dslrinfodialog.h"
@@ -87,15 +87,14 @@ class Capture : public QWidget, public Ui::Capture
         Q_PROPERTY(Ekos::CaptureState status READ status NOTIFY newStatus)
         Q_PROPERTY(QString targetName MEMBER m_TargetName)
         Q_PROPERTY(QString observerName MEMBER m_ObserverName)
-        Q_PROPERTY(QString camera READ camera WRITE setCamera)
-        Q_PROPERTY(QString filterWheel READ filterWheel WRITE setFilterWheel)
+        Q_PROPERTY(QString opticalTrain READ opticalTrain WRITE setOpticalTrain)
+        Q_PROPERTY(QString camera READ camera)
+        Q_PROPERTY(QString filterWheel READ filterWheel)
         Q_PROPERTY(QString filter READ filter WRITE setFilter)
         Q_PROPERTY(bool coolerControl READ hasCoolerControl WRITE setCoolerControl)
         Q_PROPERTY(QStringList logText READ logText NOTIFY newLog)
 
     public:
-        typedef enum { MF_NONE, MF_REQUESTED, MF_READY, MF_INITIATED, MF_FLIPPING, MF_SLEWING, MF_COMPLETED, MF_ALIGNING, MF_GUIDING } MFStage;
-
         typedef enum
         {
             ADU_LEAST_SQUARES,
@@ -117,14 +116,12 @@ class Capture : public QWidget, public Ui::Capture
              * select the CCD device from the available CCD drivers.
              * @param device The CCD device name
              */
-        Q_SCRIPTABLE bool setCamera(const QString &device);
         Q_SCRIPTABLE QString camera();
 
         /** DBUS interface function.
              * select the filter device from the available filter drivers. The filter device can be the same as the CCD driver if the filter functionality was embedded within the driver.
              * @param device The filter device name
              */
-        Q_SCRIPTABLE bool setFilterWheel(const QString &device);
         Q_SCRIPTABLE QString filterWheel();
 
         /** DBUS interface function.
@@ -283,27 +280,86 @@ class Capture : public QWidget, public Ui::Capture
 
         Q_SCRIPTABLE Ekos::CaptureState status()
         {
-            return m_State;
+            return m_captureModuleState->getCaptureState();
         }
 
         /** @}*/
 
-        void addCCD(ISD::GDInterface *newCCD);
-        void addFilter(ISD::GDInterface *newFilter);
-        void setDome(ISD::GDInterface *device);
-        void setDustCap(ISD::GDInterface *device);
-        void setLightBox(ISD::GDInterface *device);
-        void removeDevice(ISD::GDInterface *device);
-        void addGuideHead(ISD::GDInterface *newCCD);
-        void syncFrameType(ISD::GDInterface *ccd);
-        void setTelescope(ISD::GDInterface *newTelescope);
-        void setRotator(ISD::GDInterface *newRotator);
+
+        /**
+         * @brief Add new Camera
+         * @param device pointer to camera device.
+         * @return True if added successfully, false if duplicate or failed to add.
+        */
+        bool setCamera(ISD::Camera *device);
+
+        /**
+         * @brief Add new Filter Wheel
+         * @param device pointer to filter wheel device.
+         * @return True if added successfully, false if duplicate or failed to add.
+        */
+        bool setFilterWheel(ISD::FilterWheel *device);
+
+        /**
+         * @brief Add new Dome
+         * @param device pointer to Dome device.
+         * @return True if added successfully, false if duplicate or failed to add.
+        */
+        bool setDome(ISD::Dome *device);
+
+        /**
+         * @brief Add new Dust Cap
+         * @param device pointer to Dust Cap device.
+         * @return True if added successfully, false if duplicate or failed to add.
+        */
+        bool setDustCap(ISD::DustCap *device);
+
+        /**
+         * @brief Add new Light Box
+         * @param device pointer to Light Box device.
+         * @return True if added successfully, false if duplicate or failed to add.
+        */
+        bool setLightBox(ISD::LightBox *device);
+
+        /**
+         * @brief Add new Mount
+         * @param device pointer to Mount device.
+         * @return True if added successfully, false if duplicate or failed to add.
+        */
+        bool setMount(ISD::Mount *device);
+
+        /**
+         * @brief Add new Rotator
+         * @param device pointer to rotator device.
+         * @return True if added successfully, false if duplicate or failed to add.
+        */
+        bool setRotator(ISD::Rotator *device);
+
+        void removeDevice(const QSharedPointer<ISD::GenericDevice> &device);
+        void addGuideHead(ISD::Camera *device);
+        void syncFrameType(const QString &name);
         void setRotatorReversed(bool toggled);
-        void setFilterManager(const QSharedPointer<FilterManager> &manager);
+
+        // Filter Manager
+        void setupFilterManager();
+        void refreshFilterManager(ISD::FilterWheel *device);
+
         void syncTelescopeInfo();
-        void syncCCDControls();
+        void syncCameraInfo();
         void syncFilterInfo();
 
+        void setupOpticalTrainManager();
+        void refreshOpticalTrain();
+
+        QString opticalTrain() const
+        {
+            return opticalTrainCombo->currentText();
+        }
+        void setOpticalTrain(const QString &value)
+        {
+            opticalTrainCombo->setCurrentText(value);
+        }
+
         // Restart driver
         void reconnectDriver(const QString &camera, const QString &filterWheel);
 
@@ -325,6 +381,11 @@ class Capture : public QWidget, public Ui::Capture
             return m_SequenceArray;
         }
 
+        const QSharedPointer<FilterManager> &filterManager() const
+        {
+            return m_FilterManager;
+        }
+
         /**
          * @brief setSettings Set capture settings
          * @param settings list of settings
@@ -550,16 +611,22 @@ class Capture : public QWidget, public Ui::Capture
         void processData(const QSharedPointer<FITSData> &data);
 
         /**
-             * @brief checkCCD Refreshes the CCD information in the capture module.
-             * @param CCDNum The CCD index in the CCD combo box to select as the active CCD.
+             * @brief checkCamera Refreshes the CCD information in the capture module.
              */
-        void checkCCD(int CCDNum = -1);
+        void checkCamera();
+
+
+        /**
+             * @brief Connect or disconnect the camera device
+             * @param connection flag if connect (=true) or disconnect (=false)
+             */
+        void connectCamera(bool connection);
 
         /**
              * @brief checkFilter Refreshes the filter wheel information in the capture module.
              * @param filterNum The filter wheel index in the filter device combo box to set as the active filter.
              */
-        void checkFilter(int filterNum = -1);
+        void checkFilter();
 
         /**
              * @brief processCCDNumber Process number properties arriving from CCD. Currently, only CCD and Guider frames are processed.
@@ -680,16 +747,16 @@ class Capture : public QWidget, public Ui::Capture
         void setAlignStatus(Ekos::AlignState state);
         void setAlignResults(double orientation, double ra, double de, double pixscale);
         // Update Mount module status
-        void setMountStatus(ISD::Telescope::Status newState);
+        void setMountStatus(ISD::Mount::Status newState);
 
-        void setGuideChip(ISD::CCDChip *guideChip);
+        void setGuideChip(ISD::CameraChip *guideChip);
         //void setGeneratedPreviewFITS(const QString &previewFITS);
 
         // Clear Camera Configuration
         void clearCameraConfiguration();
 
         // Meridian flip
-        void meridianFlipStatusChanged(Mount::MeridianFlipStatus status);
+        void meridianFlipStatusChanged(MeridianFlipState::MeridianFlipMountState status);
 
         /**
          * @brief registerNewModule Register an Ekos module as it arrives via DBus
@@ -703,7 +770,13 @@ class Capture : public QWidget, public Ui::Capture
          */
         void generateDarkFlats();
 
-    private slots:
+        /**
+         * @brief Access to the meridian flip state machine
+         */
+        QSharedPointer<MeridianFlipState> getMeridianFlipState();;
+        void setMeridianFlipState(QSharedPointer<MeridianFlipState> newMf_state);;
+
+private slots:
 
         /**
              * @brief setDirty Set dirty bit to indicate sequence queue file was modified and needs saving.
@@ -712,12 +785,10 @@ class Capture : public QWidget, public Ui::Capture
 
         void checkFrameType(int index);
         void resetFrame();
-        void setExposureProgress(ISD::CCDChip *tChip, double value, IPState state);
+        void setExposureProgress(ISD::CameraChip *tChip, double value, IPState state);
         void updateCaptureCountDown(int deltaMillis);
         void checkSeqBoundary(const QString &path);
         void saveFITSDirectory();
-        void setDefaultCCD(QString ccd);
-        void setDefaultFilterWheel(QString filterWheel);
         void setNewRemoteFile(QString file);
 
         // Sequence Queue
@@ -766,7 +837,7 @@ class Capture : public QWidget, public Ui::Capture
         IPState checkLightFramePendingTasks();
 
         // Send image info
-        //void sendNewImage(const QString &filename, ISD::CCDChip *myChip);
+        //void sendNewImage(const QString &filename, ISD::CameraChip *myChip);
 
         // Capture
         IPState setCaptureComplete();
@@ -797,6 +868,8 @@ class Capture : public QWidget, public Ui::Capture
         Q_SCRIPTABLE void newStatus(Ekos::CaptureState status);
         Q_SCRIPTABLE void captureComplete(const QVariantMap &metadata);
 
+        void newFilterManagerStatus(Ekos::FilterState state);
+
         void ready();
 
         // signals to the sequence job
@@ -816,15 +889,19 @@ class Capture : public QWidget, public Ui::Capture
         void newDownloadProgress(double);
         void sequenceChanged(const QJsonArray &sequence);
         void settingsUpdated(const QJsonObject &settings);
-        void newMeridianFlipStatus(Mount::MeridianFlipStatus status);
         void newMeridianFlipSetup(bool activate, double hours);
         void dslrInfoRequested(const QString &cameraName);
         void driverTimedout(const QString &deviceName);
 
-        // Signals for the Analyze tab.        
+        // Signals for the Analyze tab.
         void captureStarting(double exposureSeconds, const QString &filter);
         void captureAborted(double exposureSeconds);
 
+        // Filter Manager
+        void filterManagerUpdated(ISD::FilterWheel *device);
+
+        void trainChanged();
+
     private:
         /**
          * @brief Set the currently active sequence job. Use this function to ensure
@@ -878,15 +955,7 @@ class Capture : public QWidget, public Ui::Capture
         bool isModelinDSLRInfo(const QString &model);
 
         /* Meridian Flip */
-        /**
-         * @brief Check if a meridian flip has already been started
-         * @return true iff the scope has started the meridian flip
-         */
-        inline bool checkMeridianFlipRunning()
-        {
-            return meridianFlipStage == MF_INITIATED || meridianFlipStage == MF_FLIPPING || meridianFlipStage == MF_SLEWING;
-        }
-
+        QSharedPointer<MeridianFlipState> mf_state;
         /**
          * @brief Check whether a meridian flip has been requested and trigger it
          * @return true iff a meridian flip has been triggered
@@ -951,7 +1020,7 @@ class Capture : public QWidget, public Ui::Capture
          * @brief processCaptureError Handle when image capture fails
          * @param type error type
          */
-        void processCaptureError(ISD::CCD::ErrorType type);
+        void processCaptureError(ISD::Camera::ErrorType type);
 
         /**
          * @brief setDarkFlatExposure Given a dark flat job, find the exposure suitable from it by searching for
@@ -1005,12 +1074,7 @@ class Capture : public QWidget, public Ui::Capture
         // DO NOT SET IT MANUALLY, USE {@see setActiveJob()} INSTEAD!
         SequenceJob *activeJob { nullptr };
         QSharedPointer<CaptureDeviceAdaptor> m_captureDeviceAdaptor;
-        QSharedPointer<SequenceJobState::CaptureState> m_captureState;
-
-        QList<ISD::CCD *> CCDs;
-
-        // They're generic GDInterface because they could be either ISD::CCD or ISD::Filter
-        QList<ISD::GDInterface *> Filters;
+        QSharedPointer<CaptureModuleState> m_captureModuleState;
 
         QList<SequenceJob *> jobs;
 
@@ -1057,14 +1121,6 @@ class Capture : public QWidget, public Ui::Capture
 
         bool refocusAfterMeridianFlip { false };// set to true at meridian flip to request refocus
 
-        // Meridian flip
-        SkyPoint initialMountCoords;
-        bool resumeAlignmentAfterFlip { false };
-        bool resumeGuidingAfterFlip { false };
-        MFStage meridianFlipStage { MF_NONE };
-
-        QString MFStageString(MFStage stage);
-
         // Flat field automation
         QVector<double> ExpRaw, ADURaw;
         double targetADU { 0 };
@@ -1075,7 +1131,6 @@ class Capture : public QWidget, public Ui::Capture
         bool preDomePark { false };
         FlatFieldDuration flatFieldDuration { DURATION_MANUAL };
         FlatFieldSource flatFieldSource { SOURCE_MANUAL };
-        bool dustCapLightEnabled { false };
         bool lightBoxLightEnabled { false };
         QMap<ScriptTypes, QString> m_Scripts;
 
@@ -1086,25 +1141,32 @@ class Capture : public QWidget, public Ui::Capture
         bool suspendGuideOnDownload { false };
         QJsonArray m_SequenceArray;
 
-        // State
-        CaptureState m_State { CAPTURE_IDLE };
-        FocusState m_FocusState { FOCUS_IDLE };
-        GuideState m_GuideState { GUIDE_IDLE };
-        IPState m_DitheringState {IPS_IDLE};
-        AlignState m_AlignState { ALIGN_IDLE };
-        FilterState m_FilterManagerState { FILTER_IDLE };
-
         PauseFunctionPointer pauseFunction;
 
         // CCD Chip frame settings
-        QMap<ISD::CCDChip *, QVariantMap> frameSettings;
+        QMap<ISD::CameraChip *, QVariantMap> frameSettings;
+
+        /// CCD device needed for focus operation
+        ISD::Camera *m_Camera { nullptr };
+        /// Optional device filter
+        ISD::FilterWheel *m_FilterWheel { nullptr };
+        /// Dust Cap
+        ISD::DustCap *m_DustCap { nullptr };
+        /// LightBox
+        ISD::LightBox *m_LightBox { nullptr };
+        /// Rotator
+        ISD::Rotator *m_Rotator { nullptr };
+        /// Dome
+        ISD::Dome *m_Dome { nullptr };
+        /// Mount
+        ISD::Mount *m_Mount { nullptr };
 
         // Post capture script
         QProcess m_CaptureScript;
         uint8_t m_CaptureScriptType {0};
 
         // Rotator Settings
-        std::unique_ptr<RotatorSettings> rotatorSettings;
+        std::unique_ptr<RotatorSettings> m_RotatorControlPanel;
 
         // How many images to capture before dithering operation is executed?
         uint ditherCounter { 0 };
@@ -1122,7 +1184,7 @@ class Capture : public QWidget, public Ui::Capture
         SchedulerJob::CapturedFramesMap capturedFramesMap;
 
         // Execute the meridian flip
-        void setMeridianFlipStage(MFStage stage);
+        void setMeridianFlipStage(MeridianFlipState::MFStage stage);
         void processFlipCompleted();
 
         // Controls
@@ -1139,5 +1201,7 @@ class Capture : public QWidget, public Ui::Capture
         QTimer downloadProgressTimer;
         QVariantMap m_Metadata;
         void processGuidingFailed();
+
+        QSharedPointer<FilterManager> m_FilterManager;
 };
 }
