@@ -13,9 +13,9 @@
 
 namespace Ekos
 {
-CaptureDeviceAdaptor::CaptureDeviceAdaptor()
+CaptureDeviceAdaptor::CaptureDeviceAdaptor(QSharedPointer<CaptureModuleState> captureModuleState)
 {
-
+    m_captureModuleState = captureModuleState;
 }
 
 void CaptureDeviceAdaptor::connectDome()
@@ -43,13 +43,25 @@ void CaptureDeviceAdaptor::setLightBox(ISD::LightBox *device)
         return;
 
     m_ActiveLightBox = device;
-    if (currentSequenceJobState != nullptr && !currentSequenceJobState->m_CaptureState.isNull())
-        currentSequenceJobState->m_CaptureState->hasLightBox = (device != nullptr);
+    if (currentSequenceJobState != nullptr && !currentSequenceJobState->m_CaptureModuleState.isNull())
+    {
+        currentSequenceJobState->m_CaptureModuleState->hasLightBox = (device != nullptr);
+        currentSequenceJobState->m_CaptureModuleState->setLightBoxLightState(CaptureModuleState::CAP_LIGHT_UNKNOWN);
+    }
 }
 
 void CaptureDeviceAdaptor::setDustCap(ISD::DustCap *device)
 {
+    if (m_ActiveDustCap == device)
+        return;
+
     m_ActiveDustCap = device;
+
+    if (currentSequenceJobState != nullptr && !currentSequenceJobState->m_CaptureModuleState.isNull())
+    {
+        currentSequenceJobState->m_CaptureModuleState->hasDustCap = (device != nullptr);
+        currentSequenceJobState->m_CaptureModuleState->setDustCapState(CaptureModuleState::CAP_UNKNOWN);
+    }
 }
 
 void CaptureDeviceAdaptor::connectDustCap()
@@ -63,16 +75,13 @@ void CaptureDeviceAdaptor::connectDustCap()
             &CaptureDeviceAdaptor::askManualScopeLightOpen);
     connect(currentSequenceJobState, &SequenceJobState::setLightBoxLight, this,
             &CaptureDeviceAdaptor::setLightBoxLight);
-    connect(currentSequenceJobState, &SequenceJobState::setDustCapLight, this,
-            &CaptureDeviceAdaptor::setDustCapLight);
     connect(currentSequenceJobState, &SequenceJobState::parkDustCap, this, &CaptureDeviceAdaptor::parkDustCap);
 
     connect(this, &CaptureDeviceAdaptor::manualScopeLightCover, currentSequenceJobState,
             &SequenceJobState::manualScopeLightCover);
     connect(this, &CaptureDeviceAdaptor::lightBoxLight, currentSequenceJobState, &SequenceJobState::lightBoxLight);
-    connect(this, &CaptureDeviceAdaptor::dustCapLight, currentSequenceJobState, &SequenceJobState::dustCapLight);
     connect(this, &CaptureDeviceAdaptor::dustCapStatusChanged, currentSequenceJobState,
-            &SequenceJobState::dustCapStatusChanged);
+            &SequenceJobState::dustCapStateChanged);
 }
 
 void CaptureDeviceAdaptor::disconnectDustCap()
@@ -86,19 +95,16 @@ void CaptureDeviceAdaptor::disconnectDustCap()
                &CaptureDeviceAdaptor::askManualScopeLightOpen);
     disconnect(currentSequenceJobState, &SequenceJobState::setLightBoxLight, this,
                &CaptureDeviceAdaptor::setLightBoxLight);
-    disconnect(currentSequenceJobState, &SequenceJobState::setDustCapLight, this,
-               &CaptureDeviceAdaptor::setDustCapLight);
     disconnect(currentSequenceJobState, &SequenceJobState::parkDustCap, this, &CaptureDeviceAdaptor::parkDustCap);
 
     disconnect(this, &CaptureDeviceAdaptor::manualScopeLightCover, currentSequenceJobState,
                &SequenceJobState::manualScopeLightCover);
     disconnect(this, &CaptureDeviceAdaptor::lightBoxLight, currentSequenceJobState, &SequenceJobState::lightBoxLight);
-    disconnect(this, &CaptureDeviceAdaptor::dustCapLight, currentSequenceJobState, &SequenceJobState::dustCapLight);
     disconnect(this, &CaptureDeviceAdaptor::dustCapStatusChanged, currentSequenceJobState,
-               &SequenceJobState::dustCapStatusChanged);
+               &SequenceJobState::dustCapStateChanged);
 }
 
-void CaptureDeviceAdaptor::setTelescope(ISD::Telescope *device)
+void CaptureDeviceAdaptor::setMount(ISD::Mount *device)
 {
     if (m_ActiveTelescope == device)
         return;
@@ -109,8 +115,8 @@ void CaptureDeviceAdaptor::setTelescope(ISD::Telescope *device)
     // connect new device
     if (device != nullptr)
     {
-        connect(device, &ISD::Telescope::newStatus, this, &CaptureDeviceAdaptor::scopeStatusChanged);
-        connect(device, &ISD::Telescope::newParkStatus, this, &CaptureDeviceAdaptor::scopeParkStatusChanged);
+        connect(device, &ISD::Mount::newStatus, this, &CaptureDeviceAdaptor::scopeStatusChanged);
+        connect(device, &ISD::Mount::newParkStatus, this, &CaptureDeviceAdaptor::scopeParkStatusChanged);
     }
 
     m_ActiveTelescope = device;
@@ -157,7 +163,7 @@ void CaptureDeviceAdaptor::setDome(ISD::Dome *device)
     m_ActiveDome = device;
 }
 
-void CaptureDeviceAdaptor::setRotator(ISD::GDInterface *device)
+void CaptureDeviceAdaptor::setRotator(ISD::Rotator *device)
 {
     if (m_ActiveRotator == device)
         return;
@@ -171,12 +177,15 @@ void CaptureDeviceAdaptor::setRotator(ISD::GDInterface *device)
     // connect new device
     if (m_ActiveRotator != nullptr)
     {
-        connect(m_ActiveRotator, &ISD::GDInterface::numberUpdated, this, &CaptureDeviceAdaptor::updateRotatorNumber,
+        connect(m_ActiveRotator, &ISD::Rotator::newAbsoluteAngle, this, &CaptureDeviceAdaptor::newRotatorAngle,
                 Qt::UniqueConnection);
-        connect(m_ActiveRotator, &ISD::GDInterface::switchUpdated, this, &CaptureDeviceAdaptor::updateRotatorSwitch,
+        connect(m_ActiveRotator, &ISD::Rotator::reverseToggled, this, &CaptureDeviceAdaptor::rotatorReverseToggled,
                 Qt::UniqueConnection);
-    }
 
+        emit newRotatorAngle(device->absoluteAngle(), device->absoluteAngleState());
+        emit rotatorReverseToggled(device->isReversed());
+
+    }
 }
 
 void CaptureDeviceAdaptor::connectRotator()
@@ -198,14 +207,14 @@ void CaptureDeviceAdaptor::disconnectRotator()
 void CaptureDeviceAdaptor::setRotatorAngle(double *rawAngle)
 {
     if (m_ActiveRotator != nullptr)
-        m_ActiveRotator->runCommand(INDI_SET_ROTATOR_ANGLE, rawAngle);
+        m_ActiveRotator->setAbsoluteAngle(*rawAngle);
 }
 
 void CaptureDeviceAdaptor::reverseRotator(bool toggled)
 {
     if (m_ActiveRotator != nullptr)
     {
-        m_ActiveRotator->runCommand(INDI_REVERSE_ROTATOR, &toggled);
+        m_ActiveRotator->setReversed(toggled);
         m_ActiveRotator->setConfig(SAVE_CONFIG);
     }
 }
@@ -213,37 +222,10 @@ void CaptureDeviceAdaptor::reverseRotator(bool toggled)
 void CaptureDeviceAdaptor::readRotatorAngle()
 {
     if (m_ActiveRotator != nullptr)
-    {
-        auto nvp = m_ActiveRotator->getBaseDevice()->getNumber("ABS_ROTATOR_ANGLE");
-        emit newRotatorAngle(nvp->at(0)->getValue(), nvp->s);
-    }
+        emit newRotatorAngle(m_ActiveRotator->absoluteAngle(), m_ActiveRotator->absoluteAngleState());
 }
 
-bool CaptureDeviceAdaptor::isRotatorReversed()
-{
-    if (m_ActiveRotator != nullptr)
-    {
-        auto svp = m_ActiveRotator->getBaseDevice()->getSwitch("ROTATOR_REVERSE");
-        if (svp)
-            return svp->sp[0].s == ISS_ON;
-    }
-
-    return false;
-}
-
-void CaptureDeviceAdaptor::updateRotatorNumber(INumberVectorProperty *nvp)
-{
-    if (!strcmp(nvp->name, "ABS_ROTATOR_ANGLE"))
-        emit newRotatorAngle(nvp->np[0].value, nvp->s);
-}
-
-void CaptureDeviceAdaptor::updateRotatorSwitch(ISwitchVectorProperty *svp)
-{
-    if (!strcmp(svp->name, "ROTATOR_REVERSE"))
-        emit newRotatorReversed(svp->sp[0].s == ISS_ON);
-}
-
-void CaptureDeviceAdaptor::setActiveCCD(ISD::CCD *device)
+void CaptureDeviceAdaptor::setActiveCamera(ISD::Camera *device)
 {
     if (m_ActiveCamera == device)
         return;
@@ -251,7 +233,7 @@ void CaptureDeviceAdaptor::setActiveCCD(ISD::CCD *device)
     // disconnect device events if the new device is not empty
     if (m_ActiveCamera != nullptr)
     {
-        disconnect(m_ActiveCamera, &ISD::CCD::newTemperatureValue, this,
+        disconnect(m_ActiveCamera, &ISD::Camera::newTemperatureValue, this,
                    &Ekos::CaptureDeviceAdaptor::newCCDTemperatureValue);
     }
 
@@ -262,12 +244,12 @@ void CaptureDeviceAdaptor::setActiveCCD(ISD::CCD *device)
     if (m_ActiveCamera != nullptr)
     {
         // publish device events
-        connect(m_ActiveCamera, &ISD::CCD::newTemperatureValue, this,
+        connect(m_ActiveCamera, &ISD::Camera::newTemperatureValue, this,
                 &Ekos::CaptureDeviceAdaptor::newCCDTemperatureValue, Qt::UniqueConnection);
     }
 }
 
-void CaptureDeviceAdaptor::connectActiveCCD()
+void CaptureDeviceAdaptor::connectActiveCamera()
 {
     //connect state machine to device adaptor
     connect(currentSequenceJobState, &SequenceJobState::setCCDTemperature, this,
@@ -285,7 +267,7 @@ void CaptureDeviceAdaptor::connectActiveCCD()
             &Ekos::SequenceJobState::setCurrentCCDTemperature, Qt::UniqueConnection);
 }
 
-void CaptureDeviceAdaptor::disconnectActiveCCD()
+void CaptureDeviceAdaptor::disconnectActiveCamera()
 {
     disconnect(currentSequenceJobState, &SequenceJobState::setCCDTemperature, this,
                &CaptureDeviceAdaptor::setCCDTemperature);
@@ -338,16 +320,21 @@ void CaptureDeviceAdaptor::enableCCDBatchMode(bool enable)
         m_ActiveChip->setBatchMode(enable);
 }
 
-void CaptureDeviceAdaptor::setActiveChip(ISD::CCDChip *device)
+void CaptureDeviceAdaptor::setActiveChip(ISD::CameraChip *device)
 {
     m_ActiveChip = device;
 }
 
-void CaptureDeviceAdaptor::setFilterWheel(ISD::GDInterface *device)
+void CaptureDeviceAdaptor::setFilterWheel(ISD::FilterWheel *device)
 {
     m_ActiveFilterWheel = device;
 }
 
+void CaptureDeviceAdaptor::setFilterManager(QSharedPointer<FilterManager> device)
+{
+    m_FilterManager = device;
+}
+
 void CaptureDeviceAdaptor::askManualScopeLightCover(QString question, QString title)
 {
     // do not ask again
@@ -363,11 +350,13 @@ void CaptureDeviceAdaptor::askManualScopeLightCover(QString question, QString ti
         emit manualScopeLightCover(true, true);
         KSMessageBox::Instance()->disconnect(this);
         m_ManualCoveringAsked = true;
+        m_ManualOpeningAsked = false;
     });
 
     // Cancel
     connect(KSMessageBox::Instance(), &KSMessageBox::rejected, this, [this]()
     {
+        m_ManualCoveringAsked = false;
         emit manualScopeLightCover(true, false);
         KSMessageBox::Instance()->disconnect(this);
     });
@@ -378,9 +367,18 @@ void CaptureDeviceAdaptor::askManualScopeLightCover(QString question, QString ti
 
 void CaptureDeviceAdaptor::askManualScopeLightOpen()
 {
+    // do not ask again
+    if (m_ManualOpeningAsked == true)
+    {
+        emit manualScopeLightCover(false, true);
+        return;
+    }
+
     // Continue
     connect(KSMessageBox::Instance(), &KSMessageBox::accepted, this, [this]()
     {
+        m_ManualOpeningAsked = true;
+        m_ManualCoveringAsked = false;
         emit manualScopeLightCover(false, true);
         KSMessageBox::Instance()->disconnect(this);
     });
@@ -388,6 +386,7 @@ void CaptureDeviceAdaptor::askManualScopeLightOpen()
     // Cancel
     connect(KSMessageBox::Instance(), &KSMessageBox::rejected, this, [this]()
     {
+        m_ManualOpeningAsked = false;
         emit manualScopeLightCover(false, false);
         KSMessageBox::Instance()->disconnect(this);
     });
@@ -399,7 +398,7 @@ void CaptureDeviceAdaptor::askManualScopeLightOpen()
 
 void CaptureDeviceAdaptor::setLightBoxLight(bool on)
 {
-    m_ActiveLightBox->SetLightEnabled(on);
+    m_ActiveLightBox->setLightEnabled(on);
     emit lightBoxLight(on);
 }
 
@@ -407,33 +406,23 @@ void CaptureDeviceAdaptor::parkDustCap(bool park)
 {
     // park
     if (park == true)
-        if (m_ActiveDustCap->Park())
+        if (m_ActiveDustCap->park())
             emit dustCapStatusChanged(ISD::DustCap::CAP_PARKING);
         else
             emit dustCapStatusChanged(ISD::DustCap::CAP_ERROR);
     // unpark
-    else if (m_ActiveDustCap->UnPark())
+    else if (m_ActiveDustCap->unpark())
         emit dustCapStatusChanged(ISD::DustCap::CAP_UNPARKING);
     else
         emit dustCapStatusChanged(ISD::DustCap::CAP_ERROR);
 }
 
-void CaptureDeviceAdaptor::setDustCapLight(bool on)
-{
-    // If light is not on, set it.
-    if (m_ActiveDustCap->hasLight())
-    {
-        m_ActiveDustCap->SetLightEnabled(on);
-        emit dustCapLight(on);
-    }
-}
-
 void CaptureDeviceAdaptor::slewTelescope(SkyPoint &target)
 {
     if (m_ActiveTelescope != nullptr)
     {
         m_ActiveTelescope->Slew(&target);
-        emit scopeStatusChanged(ISD::Telescope::MOUNT_SLEWING);
+        emit scopeStatusChanged(ISD::Mount::MOUNT_SLEWING);
     }
 }
 
@@ -442,7 +431,7 @@ void CaptureDeviceAdaptor::setScopeTracking(bool on)
     if (m_ActiveTelescope != nullptr)
     {
         m_ActiveTelescope->setTrackEnabled(on);
-        emit scopeStatusChanged(on ? ISD::Telescope::MOUNT_TRACKING : ISD::Telescope::MOUNT_IDLE);
+        emit scopeStatusChanged(on ? ISD::Mount::MOUNT_TRACKING : ISD::Mount::MOUNT_IDLE);
     }
 }
 
@@ -452,15 +441,15 @@ void Ekos::CaptureDeviceAdaptor::setScopeParked(bool parked)
     {
         if (parked == true)
         {
-            if (m_ActiveTelescope->Park())
-                emit scopeStatusChanged(ISD::Telescope::MOUNT_PARKING);
+            if (m_ActiveTelescope->park())
+                emit scopeStatusChanged(ISD::Mount::MOUNT_PARKING);
             else
-                emit scopeStatusChanged(ISD::Telescope::MOUNT_ERROR);
+                emit scopeStatusChanged(ISD::Mount::MOUNT_ERROR);
         }
         else
         {
-            if (m_ActiveTelescope->UnPark() == false)
-                emit scopeStatusChanged(ISD::Telescope::MOUNT_ERROR);
+            if (m_ActiveTelescope->unpark() == false)
+                emit scopeStatusChanged(ISD::Mount::MOUNT_ERROR);
         }
     }
 }
@@ -471,15 +460,15 @@ void Ekos::CaptureDeviceAdaptor::setDomeParked(bool parked)
     {
         if (parked == true)
         {
-            if (m_ActiveDome->Park())
-                emit scopeStatusChanged(ISD::Telescope::MOUNT_PARKING);
+            if (m_ActiveDome->park())
+                emit domeStatusChanged(ISD::Dome::DOME_PARKING);
             else
-                emit scopeStatusChanged(ISD::Telescope::MOUNT_ERROR);
+                emit domeStatusChanged(ISD::Dome::DOME_ERROR);
         }
         else
         {
-            if (m_ActiveTelescope->UnPark() == false)
-                emit scopeStatusChanged(ISD::Telescope::MOUNT_ERROR);
+            if (m_ActiveDome->unpark() == false)
+                emit domeStatusChanged(ISD::Dome::DOME_ERROR);
         }
     }
 
@@ -487,7 +476,7 @@ void Ekos::CaptureDeviceAdaptor::setDomeParked(bool parked)
 
 void CaptureDeviceAdaptor::flatSyncFocus(int targetFilterID)
 {
-    if (filterManager->syncAbsoluteFocusPosition(targetFilterID - 1))
+    if (getFilterManager()->syncAbsoluteFocusPosition(targetFilterID - 1))
         emit flatSyncFocusChanged(true);
     else
         emit flatSyncFocusChanged(false);
