@@ -49,7 +49,7 @@ SequenceJob::SequenceJob()
 }
 
 SequenceJob::SequenceJob(const QSharedPointer<CaptureDeviceAdaptor> cp,
-                         const QSharedPointer<SequenceJobState::CaptureState> sharedState) : SequenceJob()
+                         const QSharedPointer<CaptureModuleState> sharedState) : SequenceJob()
 {
     captureDeviceAdaptor = cp;
     // initialize the state machine
@@ -73,8 +73,8 @@ SequenceJob::SequenceJob(XMLEle *root): SequenceJob()
     XMLEle *subEP = nullptr;
 
     // set own unconnected state machine
-    QSharedPointer<SequenceJobState::CaptureState> sharedState;
-    sharedState.reset(new SequenceJobState::CaptureState);
+    QSharedPointer<CaptureModuleState> sharedState;
+    sharedState.reset(new CaptureModuleState);
     stateMachine = new SequenceJobState(sharedState);
 
     // We expect all data read from the XML to be in the C locale - QLocale::c().
@@ -160,7 +160,7 @@ SequenceJob::SequenceJob(XMLEle *root): SequenceJob()
         }
         else if (!strcmp(tagXMLEle(ep), "UploadMode"))
         {
-            setUploadMode(static_cast<ISD::CCD::UploadMode>(atoi(pcdataXMLEle(ep))));
+            setUploadMode(static_cast<ISD::Camera::UploadMode>(atoi(pcdataXMLEle(ep))));
         }
         else if (!strcmp(tagXMLEle(ep), "Calibration"))
         {
@@ -371,52 +371,57 @@ void SequenceJob::connectDeviceAdaptor()
 {
     captureDeviceAdaptor->setCurrentSequenceJobState(stateMachine);
     captureDeviceAdaptor->connectRotator();
-    captureDeviceAdaptor->connectActiveCCD();
+    captureDeviceAdaptor->connectActiveCamera();
     captureDeviceAdaptor->connectTelescope();
     captureDeviceAdaptor->connectDome();
     captureDeviceAdaptor->connectDustCap();
     // connect state machine with device adaptor
-    connect(stateMachine, &SequenceJobState::readCurrentState, captureDeviceAdaptor.data(), &CaptureDeviceAdaptor::readCurrentState);
+    connect(stateMachine, &SequenceJobState::readCurrentState, captureDeviceAdaptor.data(),
+            &CaptureDeviceAdaptor::readCurrentState);
     connect(stateMachine, &SequenceJobState::flatSyncFocus, captureDeviceAdaptor.data(), &CaptureDeviceAdaptor::flatSyncFocus);
     // connect device adaptor with state machine
-    connect(captureDeviceAdaptor.data(), &CaptureDeviceAdaptor::flatSyncFocusChanged, stateMachine, &SequenceJobState::flatSyncFocusChanged);
+    connect(captureDeviceAdaptor.data(), &CaptureDeviceAdaptor::flatSyncFocusChanged, stateMachine,
+            &SequenceJobState::flatSyncFocusChanged);
 }
 
 void SequenceJob::disconnectDeviceAdaptor()
 {
     captureDeviceAdaptor->disconnectRotator();
-    captureDeviceAdaptor->disconnectActiveCCD();
+    captureDeviceAdaptor->disconnectActiveCamera();
     captureDeviceAdaptor->disconnectTelescope();
     captureDeviceAdaptor->disconnectDome();
     captureDeviceAdaptor->disconnectDustCap();
-    disconnect(stateMachine, &SequenceJobState::readCurrentState, captureDeviceAdaptor.data(), &CaptureDeviceAdaptor::readCurrentState);
-    disconnect(stateMachine, &SequenceJobState::flatSyncFocus, captureDeviceAdaptor.data(), &CaptureDeviceAdaptor::flatSyncFocus);
-    disconnect(captureDeviceAdaptor.data(), &CaptureDeviceAdaptor::flatSyncFocusChanged, stateMachine, &SequenceJobState::flatSyncFocusChanged);
+    disconnect(stateMachine, &SequenceJobState::readCurrentState, captureDeviceAdaptor.data(),
+               &CaptureDeviceAdaptor::readCurrentState);
+    disconnect(stateMachine, &SequenceJobState::flatSyncFocus, captureDeviceAdaptor.data(),
+               &CaptureDeviceAdaptor::flatSyncFocus);
+    disconnect(captureDeviceAdaptor.data(), &CaptureDeviceAdaptor::flatSyncFocusChanged, stateMachine,
+               &SequenceJobState::flatSyncFocusChanged);
 }
 
 CAPTUREResult SequenceJob::capture(bool autofocusReady, FITSMode mode)
 {
     captureDeviceAdaptor.data()->getActiveChip()->setBatchMode(!getCoreProperty(SequenceJob::SJ_Preview).toBool());
-    captureDeviceAdaptor.data()->getActiveCCD()->setISOMode(getCoreProperty(SJ_TimeStampPrefixEnabled).toBool());
-    captureDeviceAdaptor.data()->getActiveCCD()->setSeqPrefix(getCoreProperty(SJ_FullPrefix).toString());
+    captureDeviceAdaptor.data()->getActiveCamera()->setISOMode(getCoreProperty(SJ_TimeStampPrefixEnabled).toBool());
+    captureDeviceAdaptor.data()->getActiveCamera()->setSeqPrefix(getCoreProperty(SJ_FullPrefix).toString());
 
     auto placeholderPath = Ekos::PlaceholderPath(getCoreProperty(SJ_LocalDirectory).toString() + "/sequence.esq");
     placeholderPath.setGenerateFilenameSettings(*this);
-    captureDeviceAdaptor.data()->getActiveCCD()->setPlaceholderPath(placeholderPath);
+    captureDeviceAdaptor.data()->getActiveCamera()->setPlaceholderPath(placeholderPath);
 
     if (getCoreProperty(SequenceJob::SJ_Preview).toBool())
     {
-        if (captureDeviceAdaptor.data()->getActiveCCD()->getUploadMode() != ISD::CCD::UPLOAD_CLIENT)
-            captureDeviceAdaptor.data()->getActiveCCD()->setUploadMode(ISD::CCD::UPLOAD_CLIENT);
+        if (captureDeviceAdaptor.data()->getActiveCamera()->getUploadMode() != ISD::Camera::UPLOAD_CLIENT)
+            captureDeviceAdaptor.data()->getActiveCamera()->setUploadMode(ISD::Camera::UPLOAD_CLIENT);
     }
     else
-        captureDeviceAdaptor.data()->getActiveCCD()->setUploadMode(m_UploadMode);
+        captureDeviceAdaptor.data()->getActiveCamera()->setUploadMode(m_UploadMode);
 
     QMapIterator<QString, QMap<QString, QVariant>> i(m_CustomProperties);
     while (i.hasNext())
     {
         i.next();
-        INDI::Property *customProp = captureDeviceAdaptor.data()->getActiveCCD()->getProperty(i.key());
+        INDI::Property *customProp = captureDeviceAdaptor.data()->getActiveCamera()->getProperty(i.key());
         if (customProp)
         {
             QMap<QString, QVariant> elements = i.value();
@@ -434,7 +439,7 @@ CAPTUREResult SequenceJob::capture(bool autofocusReady, FITSMode mode)
                         if (oneSwitch)
                             oneSwitch->setState(static_cast<ISState>(j.value().toInt()));
                     }
-                    captureDeviceAdaptor.data()->getActiveCCD()->getDriverInfo()->getClientManager()->sendNewSwitch(sp);
+                    captureDeviceAdaptor.data()->getActiveCamera()->getDriverInfo()->getClientManager()->sendNewSwitch(sp);
                 }
                 break;
                 case INDI_TEXT:
@@ -447,7 +452,7 @@ CAPTUREResult SequenceJob::capture(bool autofocusReady, FITSMode mode)
                         if (oneText)
                             oneText->setText(j.value().toString().toLatin1().constData());
                     }
-                    captureDeviceAdaptor.data()->getActiveCCD()->getDriverInfo()->getClientManager()->sendNewText(tp);
+                    captureDeviceAdaptor.data()->getActiveCamera()->getDriverInfo()->getClientManager()->sendNewText(tp);
                 }
                 break;
                 case INDI_NUMBER:
@@ -460,7 +465,7 @@ CAPTUREResult SequenceJob::capture(bool autofocusReady, FITSMode mode)
                         if (oneNumber)
                             oneNumber->setValue(j.value().toDouble());
                     }
-                    captureDeviceAdaptor.data()->getActiveCCD()->getDriverInfo()->getClientManager()->sendNewNumber(np);
+                    captureDeviceAdaptor.data()->getActiveCamera()->getDriverInfo()->getClientManager()->sendNewNumber(np);
                 }
                 break;
                 default:
@@ -471,7 +476,8 @@ CAPTUREResult SequenceJob::capture(bool autofocusReady, FITSMode mode)
 
     const auto remoteDirectory = getCoreProperty(SJ_RemoteDirectory).toString();
     if (captureDeviceAdaptor.data()->getActiveChip()->isBatchMode() && remoteDirectory.isEmpty() == false)
-        captureDeviceAdaptor.data()->getActiveCCD()->updateUploadSettings(remoteDirectory + getCoreProperty(SJ_DirectoryPostfix).toString());
+        captureDeviceAdaptor.data()->getActiveCamera()->updateUploadSettings(remoteDirectory + getCoreProperty(
+                    SJ_DirectoryPostfix).toString());
 
     const int ISOIndex = getCoreProperty(SJ_ISOIndex).toInt();
     if (ISOIndex != -1)
@@ -483,13 +489,13 @@ CAPTUREResult SequenceJob::capture(bool autofocusReady, FITSMode mode)
     const auto gain = getCoreProperty(SJ_Gain).toDouble();
     if (gain >= 0)
     {
-        captureDeviceAdaptor.data()->getActiveCCD()->setGain(gain);
+        captureDeviceAdaptor.data()->getActiveCamera()->setGain(gain);
     }
 
     const auto offset = getCoreProperty(SJ_Offset).toDouble();
     if (offset >= 0)
     {
-        captureDeviceAdaptor.data()->getActiveCCD()->setOffset(offset);
+        captureDeviceAdaptor.data()->getActiveCamera()->setOffset(offset);
     }
 
     const auto frameType = getFrameType();
@@ -498,7 +504,7 @@ CAPTUREResult SequenceJob::capture(bool autofocusReady, FITSMode mode)
     if (stateMachine->targetFilterID != -1 && captureDeviceAdaptor.data()->getFilterWheel() != nullptr &&
             (frameType == FRAME_FLAT || frameType == FRAME_LIGHT))
     {
-        if (stateMachine->targetFilterID != stateMachine->m_CaptureState->currentFilterID)
+        if (stateMachine->targetFilterID != stateMachine->m_CaptureModuleState->currentFilterID)
         {
             emit prepareState(CAPTURE_CHANGING_FILTER);
 
@@ -513,7 +519,7 @@ CAPTUREResult SequenceJob::capture(bool autofocusReady, FITSMode mode)
     }
 
     // Only attempt to set ROI and Binning if CCD transfer format is FITS
-    if (captureDeviceAdaptor.data()->getActiveCCD()->getEncodingFormat() == QLatin1String("FITS"))
+    if (captureDeviceAdaptor.data()->getActiveCamera()->getEncodingFormat() == QLatin1String("FITS"))
     {
         int currentBinX = 1, currentBinY = 1;
         captureDeviceAdaptor.data()->getActiveChip()->getBinning(&currentBinX, &currentBinY);
@@ -543,8 +549,8 @@ CAPTUREResult SequenceJob::capture(bool autofocusReady, FITSMode mode)
         }
     }
 
-    captureDeviceAdaptor.data()->getActiveCCD()->setCaptureFormat(getCoreProperty(SJ_Format).toString());
-    captureDeviceAdaptor.data()->getActiveCCD()->setEncodingFormat(getCoreProperty(SJ_Encoding).toString());
+    captureDeviceAdaptor.data()->getActiveCamera()->setCaptureFormat(getCoreProperty(SJ_Format).toString());
+    captureDeviceAdaptor.data()->getActiveCamera()->setEncodingFormat(getCoreProperty(SJ_Encoding).toString());
     captureDeviceAdaptor.data()->getActiveChip()->setFrameType(getFrameType());
 
     // In case FITS Viewer is not enabled. Then for flat frames, we still need to keep the data
@@ -593,7 +599,7 @@ void SequenceJob::setCaptureRetires(int value)
 
 int SequenceJob::getCurrentFilter() const
 {
-    return stateMachine->m_CaptureState->currentFilterID;
+    return stateMachine->m_CaptureModuleState->currentFilterID;
 }
 
 void SequenceJob::setCurrentFilter(int value)
@@ -603,12 +609,12 @@ void SequenceJob::setCurrentFilter(int value)
 }
 
 // Setter: Set upload mode
-void SequenceJob::setUploadMode(ISD::CCD::UploadMode value)
+void SequenceJob::setUploadMode(ISD::Camera::UploadMode value)
 {
     m_UploadMode = value;
 }
 // Getter: get upload mode
-ISD::CCD::UploadMode SequenceJob::getUploadMode() const
+ISD::Camera::UploadMode SequenceJob::getUploadMode() const
 {
     return m_UploadMode;
 }
@@ -658,27 +664,22 @@ bool SequenceJob::getJobProgressIgnored() const
 
 void SequenceJob::setLightBox(ISD::LightBox *lightBox)
 {
-    stateMachine->m_CaptureState->hasLightBox = (lightBox != nullptr);
+    stateMachine->m_CaptureModuleState->hasLightBox = (lightBox != nullptr);
 }
 
 void SequenceJob::setDustCap(ISD::DustCap *dustCap)
 {
-    stateMachine->m_CaptureState->hasDustCap = (dustCap != nullptr);
+    stateMachine->m_CaptureModuleState->hasDustCap = (dustCap != nullptr);
 }
 
-void SequenceJob::setTelescope(ISD::Telescope *scope)
+void SequenceJob::addMount(ISD::Mount *scope)
 {
-    stateMachine->m_CaptureState->hasTelescope = (scope != nullptr);
+    stateMachine->m_CaptureModuleState->hasTelescope = (scope != nullptr);
 }
 
 void SequenceJob::setDome(ISD::Dome *dome)
 {
-    stateMachine->m_CaptureState->hasDome = (dome != nullptr);
-}
-
-void SequenceJob::setFilterManager(const QSharedPointer<FilterManager> &manager)
-{
-    m_FilterManager = manager;
+    stateMachine->m_CaptureModuleState->hasDome = (dome != nullptr);
 }
 
 void SequenceJob::setFrameType(CCDFrameType value)
@@ -772,18 +773,18 @@ void SequenceJob::setCoreProperty(PropertyID id, const QVariant &value)
                 m_CoreProperties[id] = remoteDir;
             }
         }
-            break;
+        break;
 
         case SJ_GuiderActive:
-        // Inform the state machine if guiding is running. This is necessary during the preparation phase
-        // where the state machine might wait for guide deviations if enforcing initial guiding drift is selected.
-        // If guiding aborts after the preparation has started, the state machine might wait infinitely for an
-        // updated guide drift.
-        if (m_CoreProperties[SJ_GuiderActive] != value)
-        {
-            stateMachine->setEnforceInitialGuidingDrift(value.toBool() &&
-                                                        m_CoreProperties[SJ_EnforceStartGuiderDrift].toBool());
-        }
+            // Inform the state machine if guiding is running. This is necessary during the preparation phase
+            // where the state machine might wait for guide deviations if enforcing initial guiding drift is selected.
+            // If guiding aborts after the preparation has started, the state machine might wait infinitely for an
+            // updated guide drift.
+            if (m_CoreProperties[SJ_GuiderActive] != value)
+            {
+                stateMachine->setEnforceInitialGuidingDrift(value.toBool() &&
+                        m_CoreProperties[SJ_EnforceStartGuiderDrift].toBool());
+            }
             break;
         default:
             break;
