@@ -9,7 +9,7 @@
 #include "sequencejobstate.h"
 #include "capturedeviceadaptor.h"
 #include "indi/indistd.h"
-#include "indi/indiccd.h"
+#include "indi/indicamera.h"
 #include "ekos/auxiliary/filtermanager.h"
 #include "skyobjects/skypoint.h"
 
@@ -97,7 +97,7 @@ class SequenceJob : public QObject
         ////////////////////////////////////////////////////////////////////////
         /// Constructors
         ////////////////////////////////////////////////////////////////////////
-        SequenceJob(const QSharedPointer<CaptureDeviceAdaptor> cp, const QSharedPointer<SequenceJobState::CaptureState> sharedState);
+        SequenceJob(const QSharedPointer<CaptureDeviceAdaptor> cp, const QSharedPointer<CaptureModuleState> sharedState);
         SequenceJob(XMLEle *root);
         ~SequenceJob() = default;
 
@@ -117,6 +117,10 @@ class SequenceJob : public QObject
         ////////////////////////////////////////////////////////////////////////
         /// Job Status Functions
         ////////////////////////////////////////////////////////////////////////
+        void setFilterManager(const QSharedPointer<FilterManager> &manager)
+        {
+            m_FilterManager = manager;
+        }
         const QString &getStatusString()
         {
             return StatusStrings[getStatus()];
@@ -148,8 +152,6 @@ class SequenceJob : public QObject
         void setCurrentFilter(int value);
         // Getter: Get Current Filter Slot
         int getCurrentFilter() const;
-        // Setter: Set active filter manager.
-        void setFilterManager(const QSharedPointer<FilterManager> &manager);
 
         ////////////////////////////////////////////////////////////////////////
         /// GUI Related Functions
@@ -176,9 +178,9 @@ class SequenceJob : public QObject
         CCDFrameType getFrameType() const;
 
         // Setter: Set upload mode
-        void setUploadMode(ISD::CCD::UploadMode value);
+        void setUploadMode(ISD::Camera::UploadMode value);
         // Getter: get upload mode
-        ISD::CCD::UploadMode getUploadMode() const;
+        ISD::Camera::UploadMode getUploadMode() const;
 
         // Setter: Set flat field source
         void setFlatFieldSource(FlatFieldSource value);
@@ -186,7 +188,7 @@ class SequenceJob : public QObject
         FlatFieldSource getFlatFieldSource() const;
 
         // Setter: Set Wall SkyPoint Azimuth coords
-        void setWallCoord(const SkyPoint& value);
+        void setWallCoord(const SkyPoint &value);
         // Getter: Get Flat field source wall coords
         const SkyPoint &getWallCoord() const;
 
@@ -212,7 +214,7 @@ class SequenceJob : public QObject
         /**
          * @brief Set the telescope device
          */
-        void setTelescope(ISD::Telescope *scope);
+        void addMount(ISD::Mount *scope);
 
         /**
          * @brief Set the dome device
@@ -331,7 +333,7 @@ class SequenceJob : public QObject
          */
         IPState checkFlatFramePendingTasksCompleted();
 
-signals:
+    signals:
         // All preparations necessary for capturing are completed
         void prepareComplete();
         // Abort capturing
@@ -347,7 +349,7 @@ signals:
         void updateGuiderDrift(double deviation_rms);
 
 
-private:
+    private:
         // should not be used from outside
         SequenceJob();
 
@@ -368,17 +370,12 @@ private:
         // Capture Scripts
         QMap<ScriptTypes, QString> m_Scripts;
         // Upload Mode
-        ISD::CCD::UploadMode m_UploadMode { ISD::CCD::UPLOAD_CLIENT };
+        ISD::Camera::UploadMode m_UploadMode { ISD::Camera::UPLOAD_CLIENT };
         // Transfer Format
         QString m_TransferFormat { "FITS" };
         // capture frame type (light, flat, dark, bias)
         CCDFrameType m_FrameType { FRAME_LIGHT };
 
-        //////////////////////////////////////////////////////////////
-        /// Filter Manager
-        //////////////////////////////////////////////////////////////
-        QSharedPointer<FilterManager> m_FilterManager;
-
         //////////////////////////////////////////////////////////////
         /// Status Variable
         //////////////////////////////////////////////////////////////
@@ -393,6 +390,7 @@ private:
         /// State machines encapsulating the state of this capture sequence job
         //////////////////////////////////////////////////////////////
         QSharedPointer<CaptureDeviceAdaptor> captureDeviceAdaptor;
+        QSharedPointer<FilterManager> m_FilterManager;
         SequenceJobState *stateMachine { nullptr };
         /**
          * @brief Create all event connections between the state machine and the command processor
