@@ -16,11 +16,6 @@ enum COMMANDS
 {
     GET_CONNECTION,
     GET_STATES,
-    GET_CAMERAS,
-    GET_MOUNTS,
-    GET_FILTER_WHEELS,
-    GET_DOMES,
-    GET_CAPS,
     GET_STELLARSOLVER_PROFILES,
     GET_DRIVERS,
     GET_DEVICES,
@@ -62,6 +57,16 @@ enum COMMANDS
     DELETE_SCOPE,
     UPDATE_SCOPE,
 
+    // Trains
+    TRAIN_GET_ALL,
+    TRAIN_GET_PROFILES,
+    TRAIN_UPDATE,
+    TRAIN_SET,
+    TRAIN_ADD,
+    TRAIN_DELETE,
+    TRAIN_CONFIGURATION_REQUESTED,
+    TRAIN_ACCEPT,
+
     // Capture
     CAPTURE_PREVIEW,
     CAPTURE_TOGGLE_VIDEO,
@@ -95,13 +100,10 @@ enum COMMANDS
     MOUNT_SET_MOTION,
     MOUNT_SET_TRACKING,
     MOUNT_SET_SLEW_RATE,
-    MOUNT_SET_ALTITUDE_LIMITS,
-    MOUNT_SET_HA_LIMIT,
-    MOUNT_SET_MERIDIAN_FLIP,
-    MOUNT_SET_AUTO_PARK,
-    MOUNT_SET_EVERYDAY_AUTO_PARK,
     MOUNT_CLEAR,
-    MOUNT_GET_SETTINGS,
+    MOUNT_GET_ALL_SETTINGS,
+    MOUNT_SET_ALL_SETTINGS,
+    MOUNT_TOGGLE_AUTOPARK,
 
     // Dome
     DOME_PARK,
@@ -122,14 +124,10 @@ enum COMMANDS
     FOCUS_IN,
     FOCUS_OUT,
     FOCUS_LOOP,
-    FOCUS_SET_SETTINGS,
+    FOCUS_SET_ALL_SETTINGS,
+    FOCUS_GET_ALL_SETTINGS,
     FOCUS_SET_CROSSHAIR,
-    FOCUS_SET_PRIMARY_SETTINGS,
-    FOCUS_SET_PROCESS_SETTINGS,
-    FOCUS_SET_MECHANICS_SETTINGS,
-    FOCUS_GET_PRIMARY_SETTINGS,
-    FOCUS_GET_PROCESS_SETTINGS,
-    FOCUS_GET_MECHANICS_SETTINGS,
+
 
     // Guide
     GUIDE_START,
@@ -138,14 +136,18 @@ enum COMMANDS
     GUIDE_STOP,
     GUIDE_CLEAR,
     GUIDE_REPORT,
-    GUIDE_SET_SETTINGS,
+    GUIDE_SET_ALL_SETTINGS,
+    GUIDE_GET_ALL_SETTINGS,
+    GUIDE_SET_CALIBRATION_SETTINGS,
 
     // Align
     ALIGN_SOLVE,
     ALIGN_STOP,
     ALIGN_LOAD_AND_SLEW,
     ALIGN_SET_FILE_EXTENSION,
-    ALIGN_SET_SETTINGS,
+    ALIGN_SET_ALL_SETTINGS,
+    ALIGN_GET_ALL_SETTINGS,
+    ALIGN_SET_ASTROMETRY_SETTINGS,
     ALIGN_MANUAL_ROTATOR_STATUS,
     ALIGN_MANUAL_ROTATOR_TOGGLE,
 
@@ -169,8 +171,7 @@ enum COMMANDS
     PAH_REFRESH,
     PAH_SET_CROSSHAIR,
     PAH_SELECT_STAR_DONE,
-    PAH_REFRESHING_DONE,
-    PAH_SET_SETTINGS,
+    PAH_REFRESHING_DONE,    
     PAH_RESET_VIEW,
     PAH_SLEW_DONE,
     PAH_PAH_SET_ZOOM,
@@ -192,6 +193,12 @@ enum COMMANDS
     DSLR_SET_INFO,
     DSLR_SET_MODE,
 
+    //DSLR Lens
+    DSLR_ADD_LENS,
+    DSLR_DELETE_LENS,
+    DSLR_UPDATE_LENS,
+    GET_DSLR_LENSES,
+
     // Low-level device Access
     DEVICE_GET,
     DEVICE_RESTART,
@@ -225,11 +232,10 @@ enum COMMANDS
     DARK_LIBRARY_STOP,
     DARK_LIBRARY_SET_CAMERA_PRESETS,
     DARK_LIBRARY_GET_CAMERA_PRESETS,
-    DARK_LIBRARY_SET_SETTINGS,
-    DARK_LIBRARY_GET_DARK_SETTINGS,
-    DARK_LIBRARY_SET_DEFECT_SETTINGS,
-    DARK_LIBRARY_SET_DEFECT_PIXELS,
+    DARK_LIBRARY_SET_ALL_SETTINGS,
+    DARK_LIBRARY_GET_ALL_SETTINGS,
     DARK_LIBRARY_GET_DEFECT_SETTINGS,
+    DARK_LIBRARY_SET_DEFECT_PIXELS,
     DARK_LIBRARY_SAVE_MAP,
     DARK_LIBRARY_GET_VIEW_MASTERS,
     DARK_LIBRARY_GET_MASTERS_IMAGE,
@@ -241,11 +247,6 @@ static QMap<COMMANDS, QString> const commands =
 {
     {GET_CONNECTION, "get_connection"},
     {GET_STATES, "get_states"},
-    {GET_CAMERAS, "get_cameras"},
-    {GET_MOUNTS, "get_mounts"},
-    {GET_FILTER_WHEELS, "get_filter_wheels"},
-    {GET_DOMES, "get_domes"},
-    {GET_CAPS, "get_caps"},
     {GET_STELLARSOLVER_PROFILES, "get_stellarsolver_profiles"},
     {GET_DRIVERS, "get_drivers"},
     {GET_DEVICES, "get_devices"},
@@ -285,6 +286,15 @@ static QMap<COMMANDS, QString> const commands =
     {DELETE_SCOPE, "scope_delete"},
     {UPDATE_SCOPE, "scope_update"},
 
+    {TRAIN_GET_ALL, "train_get_all"},
+    {TRAIN_GET_PROFILES, "train_get_profiles"},
+    {TRAIN_UPDATE, "train_update"},
+    {TRAIN_SET, "train_set"},
+    {TRAIN_ADD, "train_add"},
+    {TRAIN_DELETE, "train_delete"},
+    {TRAIN_CONFIGURATION_REQUESTED, "train_configuration_requested"},
+    {TRAIN_ACCEPT, "train_accept"},
+
     {CAPTURE_PREVIEW, "capture_preview"},
     {CAPTURE_TOGGLE_VIDEO, "capture_toggle_video"},
     {CAPTURE_TOGGLE_CAMERA, "capture_toggle_camera"},
@@ -316,13 +326,10 @@ static QMap<COMMANDS, QString> const commands =
     {MOUNT_SET_MOTION, "mount_set_motion"},
     {MOUNT_SET_TRACKING, "mount_set_tracking"},
     {MOUNT_SET_SLEW_RATE, "mount_set_slew_rate"},
-    {MOUNT_SET_ALTITUDE_LIMITS, "mount_set_altitude_limits"},
-    {MOUNT_SET_HA_LIMIT, "mount_set_ha_limit"},
-    {MOUNT_SET_MERIDIAN_FLIP, "mount_set_meridian_flip"},
-    {MOUNT_SET_AUTO_PARK, "mount_set_auto_park"},
-    {MOUNT_SET_EVERYDAY_AUTO_PARK, "mount_set_everyday_auto_park"},
     {MOUNT_CLEAR, "mount_clear"},
-    {MOUNT_GET_SETTINGS, "mount_get_settings"},
+    {MOUNT_GET_ALL_SETTINGS, "mount_get_all_settings"},
+    {MOUNT_SET_ALL_SETTINGS, "mount_set_all_settings"},
+    {MOUNT_TOGGLE_AUTOPARK, "mount_toggle_autopark"},
 
     {DOME_PARK, "dome_park"},
     {DOME_UNPARK, "dome_unpark"},
@@ -340,14 +347,10 @@ static QMap<COMMANDS, QString> const commands =
     {FOCUS_IN, "focus_in"},
     {FOCUS_OUT, "focus_out"},
     {FOCUS_LOOP, "focus_loop"},
-    {FOCUS_SET_SETTINGS, "focus_set_settings"},
-    {FOCUS_SET_CROSSHAIR, "focus_set_crosshair"},
-    {FOCUS_SET_PRIMARY_SETTINGS, "focus_set_primary_settings"},
-    {FOCUS_SET_PROCESS_SETTINGS, "focus_set_process_settings"},
-    {FOCUS_SET_MECHANICS_SETTINGS, "focus_set_mechanics_settings"},
-    {FOCUS_GET_PRIMARY_SETTINGS, "focus_get_primary_settings"},
-    {FOCUS_GET_PROCESS_SETTINGS, "focus_get_process_settings"},
-    {FOCUS_GET_MECHANICS_SETTINGS, "focus_get_mechanics_settings"},
+    {FOCUS_SET_ALL_SETTINGS, "focus_set_all_settings"},
+    {FOCUS_GET_ALL_SETTINGS, "focus_get_all_settings"},
+    {FOCUS_SET_CROSSHAIR, "focus_set_crosshair"},    
+
 
     {SCHEDULER_SET_PRIMARY_SETTINGS, "scheduler_set_primary_settings"},
     {SCHEDULER_SET_JOB_STARTUP_CONDITIONS, "scheduler_set_job_startup_conditions"},
@@ -368,19 +371,22 @@ static QMap<COMMANDS, QString> const commands =
     {GUIDE_STOP, "guide_stop"},
     {GUIDE_CLEAR, "guide_clear"},
     {GUIDE_REPORT, "guide_report"},
-    {GUIDE_SET_SETTINGS, "guide_set_settings"},
+    {GUIDE_SET_ALL_SETTINGS, "guide_set_all_settings"},
+    {GUIDE_GET_ALL_SETTINGS, "guide_get_all_settings"},
+    {GUIDE_SET_CALIBRATION_SETTINGS, "guide_set_calibration_settings"},
 
     {ALIGN_SOLVE, "align_solve"},
     {ALIGN_STOP, "align_stop"},
     {ALIGN_LOAD_AND_SLEW, "align_load_and_slew"},
     {ALIGN_SET_FILE_EXTENSION, "align_set_file_extension"},
-    {ALIGN_SET_SETTINGS, "align_set_settings"},
+    {ALIGN_SET_ALL_SETTINGS, "align_set_all_settings"},
+    {ALIGN_GET_ALL_SETTINGS, "align_get_all_settings"},
+    {ALIGN_SET_ASTROMETRY_SETTINGS, "align_set_astrometry_settings"},
     {ALIGN_MANUAL_ROTATOR_STATUS, "align_manual_rotator_status"},
     {ALIGN_MANUAL_ROTATOR_TOGGLE, "align_manual_rotator_toggle"},
 
     {PAH_START, "polar_start"},
-    {PAH_STOP, "polar_stop"},
-    {PAH_SET_SETTINGS, "polar_set_settings"},
+    {PAH_STOP, "polar_stop"},    
     {PAH_REFRESH, "polar_refresh"},
     {PAH_SET_CROSSHAIR, "polar_set_crosshair"},
     {PAH_SELECT_STAR_DONE, "polar_star_select_done"},
@@ -403,6 +409,11 @@ static QMap<COMMANDS, QString> const commands =
     {DSLR_SET_INFO, "dslr_set_info"},
     {DSLR_SET_MODE, "dslr_set_mode"},
 
+    {GET_DSLR_LENSES, "get_dslr_lenses"},
+    {DSLR_ADD_LENS, "dslr_add_lens"},
+    {DSLR_DELETE_LENS, "dslr_delete_lens"},
+    {DSLR_UPDATE_LENS, "dslr_update_lens"},
+
     {DEVICE_GET, "device_get"},
     {DEVICE_RESTART, "device_restart"},
     {DEVICE_BLOB_GET, "device_blob_get"},
@@ -431,11 +442,10 @@ static QMap<COMMANDS, QString> const commands =
     {DARK_LIBRARY_STOP, "dark_library_stop"},
     {DARK_LIBRARY_SET_CAMERA_PRESETS, "dark_library_set_camera_presets"},
     {DARK_LIBRARY_GET_CAMERA_PRESETS, "dark_library_get_camera_presets"},
-    {DARK_LIBRARY_SET_SETTINGS, "dark_library_set_settings"},
-    {DARK_LIBRARY_GET_DARK_SETTINGS, "dark_library_get_dark_settings"},
-    {DARK_LIBRARY_SET_DEFECT_SETTINGS, "dark_library_set_defect_settings"},
-    {DARK_LIBRARY_SET_DEFECT_PIXELS, "dark_library_set_defect_pixels"},
+    {DARK_LIBRARY_SET_ALL_SETTINGS, "dark_library_set_all_settings"},
+    {DARK_LIBRARY_GET_ALL_SETTINGS, "dark_library_get_all_settings"},
     {DARK_LIBRARY_GET_DEFECT_SETTINGS, "dark_library_get_defect_settings"},
+    {DARK_LIBRARY_SET_DEFECT_PIXELS, "dark_library_set_defect_pixels"},
     {DARK_LIBRARY_GET_VIEW_MASTERS, "dark_library_get_view_masters"},
     {DARK_LIBRARY_GET_MASTERS_IMAGE, "dark_library_get_masters_image"},
     {DARK_LIBRARY_CLEAR_MASTERS_ROW, "dark_library_clear_masters_row"},
