@@ -47,15 +47,13 @@ class Message : public QObject
         void updateCapStatus(const QJsonObject &status);
 
         // Send devices as they come
-        void sendEvent(const QString &message, KSNotification::EventType event);
-        void sendCameras();
-        void sendMounts();
+        void sendEvent(const QString &message, KSNotification::EventSource source, KSNotification::EventType event);
         void sendScopes();
-        void sendFilterWheels();
-        void sendDomes();
-        void sendCaps();
+        void sendDSLRLenses();        
         void sendDrivers();
         void sendDevices();
+        void sendTrains();
+        void sendTrainProfiles();
         void sendSchedulerJobList(QJsonArray jobsList);
 
         // Scheduler
@@ -85,24 +83,31 @@ class Message : public QObject
         // Alignment
         void setAlignStatus(Ekos::AlignState newState);
         void setAlignSolution(const QVariantMap &solution);
-        void sendAlignSettings(const QJsonObject &settings);
+        void sendAlignSettings(const QVariantMap &settings);
 
         // Guide
-        void sendGuideSettings(const QJsonObject &settings);
+        void sendGuideSettings(const QVariantMap &settings);
 
         // Focus
-        void sendFocusSettings(const QJsonObject &settings);
+        void sendFocusSettings(const QVariantMap &settings);
+
+        // Mount
+        void sendMountSettings(const QVariantMap &settings);
+
+        // Dark Library
+        void sendDarkLibrarySettings(const QVariantMap &settings);
 
         //Scheduler
         void sendSchedulerSettings(const QJsonObject &settings);
 
         // Polar
-        void setPAHStage(Ekos::PolarAlignmentAssistant::PAHStage stage);
+        void setPAHStage(Ekos::PolarAlignmentAssistant::Stage stage);
         void setPAHMessage(const QString &message);
         void setPolarResults(QLineF correctionVector, double polarError, double azError, double altError);
         void setUpdatedErrors(double total, double az, double alt);
         void setPAHEnabled(bool enabled);
         void setBoundingRect(QRect rect, QSize view, double currentZoom);
+
         // Capture
         void sendCaptureSequence(const QJsonArray &sequenceArray);
         void sendCaptureSettings(const QJsonObject &settings);
@@ -113,6 +118,9 @@ class Message : public QObject
         // Port Selection
         void requestPortSelection(bool show);
 
+        // Trains
+        void requestOpticalTrains(bool show);
+
         // Dialogs
         void sendDialog(const QJsonObject &message);
         void processDialogResponse(const QJsonObject &payload);
@@ -169,12 +177,6 @@ class Message : public QObject
         // Polar
         void processPolarCommands(const QString &command, const QJsonObject &payload);
 
-        // Dome
-        void processDomeCommands(const QString &command, const QJsonObject &payload);
-
-        // Cap
-        void processCapCommands(const QString &command, const QJsonObject &payload);
-
         // Profile
         void processProfileCommands(const QString &command, const QJsonObject &payload);
 
@@ -187,6 +189,9 @@ class Message : public QObject
         // DSLRs
         void processDSLRCommands(const QString &command, const QJsonObject &payload);
 
+        // Trains
+        void processTrainCommands(const QString &command, const QJsonObject &payload);
+
         // Filter Manager commands
         void processFilterManagerCommands(const QString &command, const QJsonObject &payload);
 
