@@ -6,19 +6,19 @@
 
 #pragma once
 
+#include "ekos/auxiliary/filtermanager.h"
 #include "ui_focus.h"
 #include "focusprofileplot.h"
 #include "ekos/ekos.h"
-#include "ekos/auxiliary/filtermanager.h"
 #include "ekos/auxiliary/stellarsolverprofileeditor.h"
 #include "ekos/auxiliary/darkprocessor.h"
 #include "ekos/mount/mount.h"
 #include "fitsviewer/fitsviewer.h"
-#include "indi/indiccd.h"
+#include "indi/indicamera.h"
 #include "indi/indifocuser.h"
 #include "indi/indistd.h"
 #include "indi/indiweather.h"
-#include "indi/inditelescope.h"
+#include "indi/indimount.h"
 
 #include <QtDBus/QtDBus>
 #include <parameters.h>
@@ -42,9 +42,10 @@ class Focus : public QWidget, public Ui::Focus
         Q_CLASSINFO("D-Bus Interface", "org.kde.kstars.Ekos.Focus")
         Q_PROPERTY(Ekos::FocusState status READ status NOTIFY newStatus)
         Q_PROPERTY(QStringList logText READ logText NOTIFY newLog)
-        Q_PROPERTY(QString camera READ camera WRITE setCamera)
-        Q_PROPERTY(QString focuser READ focuser WRITE setFocuser)
-        Q_PROPERTY(QString filterWheel READ filterWheel WRITE setFilterWheel)
+        Q_PROPERTY(QString opticalTrain READ opticalTrain WRITE setOpticalTrain)
+        Q_PROPERTY(QString camera READ camera)
+        Q_PROPERTY(QString focuser READ focuser)
+        Q_PROPERTY(QString filterWheel READ filterWheel)
         Q_PROPERTY(QString filter READ filter WRITE setFilter)
         Q_PROPERTY(double HFR READ getHFR NOTIFY newHFR)
         Q_PROPERTY(double exposure READ exposure WRITE setExposure)
@@ -53,9 +54,9 @@ class Focus : public QWidget, public Ui::Focus
         Focus();
         ~Focus();
 
-        typedef enum { FOCUS_NONE, FOCUS_IN, FOCUS_OUT } FocusDirection;
-        typedef enum { FOCUS_MANUAL, FOCUS_AUTO } FocusType;
-        typedef enum { FOCUS_ITERATIVE, FOCUS_POLYNOMIAL, FOCUS_LINEAR, FOCUS_LINEAR1PASS } FocusAlgorithm;
+        typedef enum { FOCUS_NONE, FOCUS_IN, FOCUS_OUT } Direction;
+        typedef enum { FOCUS_MANUAL, FOCUS_AUTO } Type;
+        typedef enum { FOCUS_ITERATIVE, FOCUS_POLYNOMIAL, FOCUS_LINEAR, FOCUS_LINEAR1PASS } Algorithm;
         //typedef enum { FOCUSER_TEMPERATURE, OBSERVATORY_TEMPERATURE, NO_TEMPERATURE } TemperatureSource;
 
         /** @defgroup FocusDBusInterface Ekos DBus Interface - Focus Module
@@ -69,7 +70,6 @@ class Focus : public QWidget, public Ui::Focus
              * @param device The CCD device name
              * @return Returns true if CCD device is found and set, false otherwise.
              */
-        Q_SCRIPTABLE bool setCamera(const QString &device);
         Q_SCRIPTABLE QString camera();
 
         /** DBUS interface function.
@@ -77,7 +77,6 @@ class Focus : public QWidget, public Ui::Focus
              * @param device The focuser device name
              * @return Returns true if focuser device is found and set, false otherwise.
              */
-        Q_SCRIPTABLE bool setFocuser(const QString &device);
         Q_SCRIPTABLE QString focuser();
 
         /** DBUS interface function.
@@ -85,7 +84,6 @@ class Focus : public QWidget, public Ui::Focus
              * @param device The filter device name
              * @return Returns true if filter device is found and set, false otherwise.
              */
-        Q_SCRIPTABLE bool setFilterWheel(const QString &device);
         Q_SCRIPTABLE QString filterWheel();
 
         /** DBUS interface function.
@@ -101,7 +99,7 @@ class Focus : public QWidget, public Ui::Focus
              */
         Q_SCRIPTABLE bool canAutoFocus()
         {
-            return (focusType == FOCUS_AUTO);
+            return (m_FocusType == FOCUS_AUTO);
         }
 
         /** DBUS interface function.
@@ -119,7 +117,7 @@ class Focus : public QWidget, public Ui::Focus
         Q_SCRIPTABLE Q_NOREPLY void setExposure(double value);
         Q_SCRIPTABLE double exposure()
         {
-            return exposureIN->value();
+            return focusExposure->value();
         }
 
         /** DBUS interface function.
@@ -129,12 +127,6 @@ class Focus : public QWidget, public Ui::Focus
              */
         Q_SCRIPTABLE Q_NOREPLY void setBinning(int binX, int binY);
 
-        /** DBUS interface function.
-             * Set image filter to apply to the image after capture.
-             * @param value Image filter (Auto Stretch, High Contrast, Equalize, High Pass)
-             */
-        Q_SCRIPTABLE Q_NOREPLY void setImageFilter(const QString &value);
-
         /** DBUS interface function.
              * Set Auto Focus options. The options must be set before starting the autofocus operation. If no options are set, the options loaded from the user configuration are used.
              * @param enable If true, Ekos will attempt to automatically select the best focus star in the frame. If it fails to select a star, the user will be asked to select a star manually.
@@ -176,35 +168,44 @@ class Focus : public QWidget, public Ui::Focus
         /**
              * @brief Add CCD to the list of available CCD.
              * @param newCCD pointer to CCD device.
+             * @return True if added successfully, false if duplicate or failed to add.
              */
-        void addCCD(ISD::GDInterface *newCCD);
+        bool setCamera(ISD::Camera *device);
 
         /**
              * @brief addFocuser Add focuser to the list of available focusers.
              * @param newFocuser pointer to focuser device.
+             * @return True if added successfully, false if duplicate or failed to add.
              */
-        void addFocuser(ISD::GDInterface *newFocuser);
+        bool setFocuser(ISD::Focuser *device);
 
         /**
              * @brief addFilter Add filter to the list of available filters.
              * @param newFilter pointer to filter device.
+             * @return True if added successfully, false if duplicate or failed to add.
              */
-        void addFilter(ISD::GDInterface *newFilter);
+        bool setFilterWheel(ISD::FilterWheel *device);
 
 
         /**
              * @brief addTemperatureSource Add temperature source to the list of available sources.
              * @param newSource Device with temperature reporting capability
+             * @return True if added successfully, false if duplicate or failed to add.
              */
-        void addTemperatureSource(ISD::GDInterface *newSource);
+        bool addTemperatureSource(const QSharedPointer<ISD::GenericDevice> &device);
 
         /**
          * @brief removeDevice Remove device from Focus module
          * @param deviceRemoved pointer to device
          */
-        void removeDevice(ISD::GDInterface *deviceRemoved);
+        void removeDevice(const QSharedPointer<ISD::GenericDevice> &deviceRemoved);
 
-        void setFilterManager(const QSharedPointer<FilterManager> &manager);
+        const QSharedPointer<FilterManager> &filterManager() const
+        {
+            return m_FilterManager;
+        }        
+        void setupFilterManager();
+        void refreshFilterManager(ISD::FilterWheel *device);
 
         void clearLog();
         QStringList logText()
@@ -216,18 +217,9 @@ class Focus : public QWidget, public Ui::Focus
             return m_LogText.join("\n");
         }
 
-        // Presets
-        QJsonObject getSettings() const;
-        void setSettings(const QJsonObject &settings);
-        // Primary Settings
-        QJsonObject getPrimarySettings() const;
-        void setPrimarySettings(const QJsonObject &settings);
-        // Process Settings
-        QJsonObject getProcessSettings() const;
-        void setProcessSettings(const QJsonObject &settings);
-        // Mechanics Settings
-        QJsonObject getMechanicsSettings() const;
-        void setMechanicsSettings(const QJsonObject &settings);
+        // Settings
+        QVariantMap getAllSettings() const;
+        void setAllSettings(const QVariantMap &settings);
 
     public slots:
 
@@ -294,16 +286,16 @@ class Focus : public QWidget, public Ui::Focus
         void meridianFlipStarted();
 
         /**
-             * @brief Check CCD and make sure information is updated accordingly. This simply calls syncCCDInfo for the current CCD.
+             * @brief Check CCD and make sure information is updated accordingly. This simply calls syncCameraInfo for the current CCD.
              * @param CCDNum By default, we check the already selected CCD in the dropdown menu. If CCDNum is specified, the check is made against this specific CCD in the dropdown menu.
              *  CCDNum is the index of the CCD in the dropdown menu.
              */
-        void checkCCD(int CCDNum = -1);
+        void checkCamera();
 
         /**
-             * @brief syncCCDInfo Read current CCD information and update settings accordingly.
+             * @brief syncCameraInfo Read current CCD information and update settings accordingly.
              */
-        void syncCCDInfo();
+        void syncCameraInfo();
 
         /**
          * @brief Update camera controls like Gain, ISO, Offset...etc
@@ -315,20 +307,20 @@ class Focus : public QWidget, public Ui::Focus
              * @param FocuserNum By default, we check the already selected focuser in the dropdown menu. If FocuserNum is specified, the check is made against this specific focuser in the dropdown menu.
              *  FocuserNum is the index of the focuser in the dropdown menu.
              */
-        void checkFocuser(int FocuserNum = -1);
+        void checkFocuser();
 
         /**
              * @brief Check Filter and make sure information is updated accordingly.
              * @param filterNum By default, we check the already selected filter in the dropdown menu. If filterNum is specified, the check is made against this specific filter in the dropdown menu.
              *  filterNum is the index of the filter in the dropdown menu.
              */
-        void checkFilter(int filterNum = -1);
+        void checkFilter();
 
         /**
              * @brief Check temperature source and make sure information is updated accordingly.
-             * @param index Index of source in combo box. If -1, then use the currently selected source.
+             * @param name Name of temperature source, if empty then use current source.
              */
-        void checkTemperatureSource(int index = -1);
+        void checkTemperatureSource(const QString &name = QString());
 
         /**
              * @brief clearDataPoints Remove all data points from HFR plots
@@ -374,12 +366,6 @@ class Focus : public QWidget, public Ui::Focus
              */
         //void setAutoFocusResult(bool status);
 
-        /**
-             * @brief filterChangeWarning Warn the user it is not a good idea to apply image filter in the filter process as they can skew the HFR calculations.
-             * @param index Index of image filter selected by the user.
-             */
-        void filterChangeWarning(int index);
-
         // Logging methods - one for INFO messages to the kstars log, and one for a CSV auto-focus log
         void appendLogText(const QString &);
         void appendFocusLogText(const QString &);
@@ -388,10 +374,10 @@ class Focus : public QWidget, public Ui::Focus
         void adjustFocusOffset(int value, bool useAbsoluteOffset);
 
         // Update Mount module status
-        void setMountStatus(ISD::Telescope::Status newState);
+        void setMountStatus(ISD::Mount::Status newState);
 
         // Update Altitude From Mount
-        void setMountCoords(const SkyPoint &position, ISD::Telescope::PierSide pierSide, const dms &ha);
+        void setMountCoords(const SkyPoint &position, ISD::Mount::PierSide pierSide, const dms &ha);
 
         /**
          * @brief toggleVideo Turn on and off video streaming if supported by the camera.
@@ -416,7 +402,14 @@ class Focus : public QWidget, public Ui::Focus
          */
         QStringList getStellarSolverProfiles();
 
-
+        QString opticalTrain() const
+        {
+            return opticalTrainCombo->currentText();
+        }
+        void setOpticalTrain(const QString &value)
+        {
+            opticalTrainCombo->setCurrentText(value);
+        }
 
     protected:
         void addPlotPosition(int pos, double hfr, bool plot = true);
@@ -436,7 +429,7 @@ class Focus : public QWidget, public Ui::Focus
 
         void processCaptureTimeout();
 
-        void processCaptureError(ISD::CCD::ErrorType type);
+        void processCaptureError(ISD::Camera::ErrorType type);
 
         void setCaptureComplete();
 
@@ -444,9 +437,7 @@ class Focus : public QWidget, public Ui::Focus
 
         void toggleFocusingWidgetFullScreen();
 
-        void setVideoStreamEnabled(bool enabled);
-
-        void syncSettings();
+        void setVideoStreamEnabled(bool enabled);        
 
         void calculateHFR();
         void setCurrentHFR(double value);
@@ -460,17 +451,22 @@ class Focus : public QWidget, public Ui::Focus
         void absolutePositionChanged(int value);
         void focusPositionAdjusted();
 
+        void trainChanged();
+
         void suspendGuiding();
         void resumeGuiding();
         void newImage(const QSharedPointer<FITSView> &view);
         void newStarPixmap(QPixmap &);
-        void settingsUpdated(const QJsonObject &settings);
+        void settingsUpdated(const QVariantMap &settings);
 
         // Signals for Analyze.
         void autofocusStarting(double temperature, const QString &filter);
         void autofocusComplete(const QString &filter, const QString &points);
         void autofocusAborted(const QString &filter, const QString &points);
 
+        // Filter Manager
+        void filterManagerUpdated(ISD::FilterWheel *device);
+
         // HFR V curve plot events
         /**
          * @brief initialize the HFR V plot
@@ -556,13 +552,32 @@ class Focus : public QWidget, public Ui::Focus
         ////////////////////////////////////////////////////////////////////
 
         /**
-         * @brief initSettings Connect settings to slots to update the value when changed
+         * @brief Connect GUI elements to sync settings once updated.
          */
-        void initSettingsConnections();
+        void connectSettings();
+        /**
+         * @brief Stop updating settings when GUI elements are updated.
+         */
+        void disconnectSettings();
         /**
          * @brief loadSettings Load setting from Options and set them accordingly.
          */
-        void loadSettings();
+        void loadGlobalSettings();
+
+        /**
+         * @brief syncSettings When checkboxes, comboboxes, or spin boxes are updated, save their values in the
+         * global and per-train settings.
+         */
+        void syncSettings();
+
+        /**
+         * @brief syncControl Sync setting to widget. The value depends on the widget type.
+         * @param settings Map of all settings
+         * @param key name of widget to sync
+         * @param widget pointer of widget to set
+         * @return True if sync successful, false otherwise
+         */
+        bool syncControl(const QVariantMap &settings, const QString &key, QWidget * widget);
 
         ////////////////////////////////////////////////////////////////////
         /// HFR Plot
@@ -589,7 +604,7 @@ class Focus : public QWidget, public Ui::Focus
          */
         bool isPositionBased()
         {
-            return (canAbsMove || canRelMove || (focusAlgorithm == FOCUS_LINEAR) || (focusAlgorithm == FOCUS_LINEAR1PASS));
+            return (canAbsMove || canRelMove || (m_FocusAlgorithm == FOCUS_LINEAR) || (m_FocusAlgorithm == FOCUS_LINEAR1PASS));
         }
         void resetButtons();
         void stop(FocusState completionState = FOCUS_ABORTED);
@@ -600,14 +615,14 @@ class Focus : public QWidget, public Ui::Focus
          * @brief prepareCapture Set common settings for capture for focus module
          * @param targetChip target Chip
          */
-        void prepareCapture(ISD::CCDChip *targetChip);
+        void prepareCapture(ISD::CameraChip *targetChip);
         ////////////////////////////////////////////////////////////////////
         /// HFR
         ////////////////////////////////////////////////////////////////////
         void setHFRComplete();
 
         // Sets the algorithm and enables/disables various UI inputs.
-        void setFocusAlgorithm(FocusAlgorithm algorithm);
+        void setFocusAlgorithm(Algorithm algorithm);
 
         void setCurveFit(CurveFitting::CurveFit curvefit);
 
@@ -656,13 +671,11 @@ class Focus : public QWidget, public Ui::Focus
          */
         void settle(const FocusState completionState, const bool autoFocusUsed);
 
-        //        void initializeFocuserTemperature();
         void setLastFocusTemperature();
-        bool findTemperatureElement(ISD::GDInterface *device);
-        //        void updateTemperature(TemperatureSource source, double newTemperature);
-        //        void emitTemperatureEvents(TemperatureSource source, double newTemperature);
+        bool findTemperatureElement(const QSharedPointer<ISD::GenericDevice> &device);        
 
-        bool syncControl(const QJsonObject &settings, const QString &key, QWidget * widget);
+        void setupOpticalTrainManager();
+        void refreshOpticalTrain();
 
         /**
          * @brief handleFocusMotionTimeout When focuser is command to go to a target position, we expect to receive a notification
@@ -671,11 +684,11 @@ class Focus : public QWidget, public Ui::Focus
         void handleFocusMotionTimeout();
 
         /// Focuser device needed for focus operation
-        ISD::Focuser *currentFocuser { nullptr };
+        ISD::Focuser *m_Focuser { nullptr };
         /// CCD device needed for focus operation
-        ISD::CCD *currentCCD { nullptr };
+        ISD::Camera *m_Camera { nullptr };
         /// Optional device filter
-        ISD::GDInterface *currentFilter { nullptr };
+        ISD::FilterWheel *m_FilterWheel { nullptr };
         /// Optional temperature source element
         INumber *currentTemperatureSourceElement {nullptr};
 
@@ -686,27 +699,21 @@ class Focus : public QWidget, public Ui::Focus
         bool filterPositionPending { false };
         bool fallbackFilterPending { false };
 
-        /// List of Focusers
-        QList<ISD::Focuser *> Focusers;
-        /// List of CCDs
-        QList<ISD::CCD *> CCDs;
-        /// They're generic GDInterface because they could be either ISD::CCD or ISD::Filter
-        QList<ISD::GDInterface *> Filters;
-        /// They're generic GDInterface because they could be either ISD::CCD or ISD::Filter or ISD::Weather
-        QList<ISD::GDInterface *> TemperatureSources;
+        /// They're generic GDInterface because they could be either ISD::Camera or ISD::FilterWheel or ISD::Weather
+        QList<QSharedPointer<ISD::GenericDevice>> m_TemperatureSources;
 
         /// As the name implies
-        FocusDirection m_LastFocusDirection { FOCUS_NONE };
+        Direction m_LastFocusDirection { FOCUS_NONE };
         /// Keep track of the last requested steps
         uint32_t m_LastFocusSteps {0};
         /// What type of focusing are we doing right now?
-        FocusType focusType { FOCUS_MANUAL };
+        Type m_FocusType { FOCUS_MANUAL };
         /// Focus HFR & Centeroid algorithms
-        StarAlgorithm focusDetection { ALGORITHM_GRADIENT };
+        StarAlgorithm m_FocusDetection { ALGORITHM_SEP };
         /// Focus Process Algorithm
-        FocusAlgorithm focusAlgorithm { FOCUS_ITERATIVE };
+        Algorithm m_FocusAlgorithm { FOCUS_LINEAR1PASS };
         /// Curve fit, default to Quadratic
-        CurveFitting::CurveFit curveFit { CurveFitting::FOCUS_QUADRATIC };
+        CurveFitting::CurveFit m_CurveFit { CurveFitting::FOCUS_QUADRATIC };
 
         /*********************
          * HFR Club variables
@@ -787,9 +794,7 @@ class Focus : public QWidget, public Ui::Focus
         /// If HFR=-1 which means no stars detected, we need to decide how many times should the re-capture process take place before we give up or reverse direction.
         int noStarCount { 0 };
         /// Track which upload mode the CCD is set to. If set to UPLOAD_LOCAL, then we need to switch it to UPLOAD_CLIENT in order to do focusing, and then switch it back to UPLOAD_LOCAL
-        ISD::CCD::UploadMode rememberUploadMode { ISD::CCD::UPLOAD_CLIENT };
-        /// Previous binning setting
-        int activeBin { 0 };
+        ISD::Camera::UploadMode rememberUploadMode { ISD::Camera::UPLOAD_CLIENT };
         /// HFR values for captured frames before averages
         QVector<double> HFRFrames;
         // Camera Fast Exposure
@@ -826,11 +831,8 @@ class Focus : public QWidget, public Ui::Focus
         /// State
         Ekos::FocusState state { Ekos::FOCUS_IDLE };
 
-        /// FITS Scale
-        FITSScale defaultScale;
-
         /// CCD Chip frame settings
-        QMap<ISD::CCDChip *, QVariantMap> frameSettings;
+        QMap<ISD::CameraChip *, QVariantMap> frameSettings;
 
         /// Selected star coordinates
         QVector3D starCenter;
@@ -873,9 +875,6 @@ class Focus : public QWidget, public Ui::Focus
         // Guide Suspend
         bool m_GuidingSuspended { false };
 
-        // Filter Manager
-        QSharedPointer<FilterManager> filterManager;
-
         // Data
         QSharedPointer<FITSData> m_ImageData;
 
@@ -895,7 +894,12 @@ class Focus : public QWidget, public Ui::Focus
 
         static constexpr uint8_t MAXIMUM_FLUCTUATIONS {10};
 
+        QVariantMap m_Settings;
+        QVariantMap m_GlobalSettings;
+
         // Dark Processor
         QPointer<DarkProcessor> m_DarkProcessor;
+
+        QSharedPointer<FilterManager> m_FilterManager;
 };
 }
