@@ -15,25 +15,18 @@
 #include "ui_manager.h"
 
 #include "ekos.h"
-#include "manager/focusmanager.h"
-#include "manager/guidemanager.h"
 #include "fitsviewer/summaryfitsview.h"
 #include "align/align.h"
-#include "auxiliary/dome.h"
-#include "auxiliary/weather.h"
-#include "auxiliary/dustcap.h"
 #include "capture/capture.h"
 #include "focus/focus.h"
 #include "guide/guide.h"
 #include "indi/indistd.h"
 #include "mount/mount.h"
+#include "observatory/observatory.h"
 #include "scheduler/scheduler.h"
 #include "analyze/analyze.h"
-#include "observatory/observatory.h"
-#include "auxiliary/filtermanager.h"
 #include "auxiliary/portselector.h"
 #include "ksnotification.h"
-// Can't use forward declaration with QPointer. QTBUG-29588
 #include "auxiliary/opslogs.h"
 
 #include <QDialog>
@@ -134,14 +127,6 @@ class Manager : public QDialog, public Ui::Manager
         {
             return focusProcess.get();
         }
-        Dome  *domeModule()
-        {
-            return domeProcess.get();
-        }
-        DustCap *capModule()
-        {
-            return dustCapProcess.get();
-        }
         Capture *captureModule()
         {
             return captureProcess.get();
@@ -150,12 +135,8 @@ class Manager : public QDialog, public Ui::Manager
         {
             return m_SummaryView.get();
         }
-        FilterManager *getFilterManager()
-        {
-            return filterManager.data();
-        }
         QString getCurrentJobName();
-        void announceEvent(const QString &message, KSNotification::EventType event);
+        void announceEvent(const QString &message, KSNotification::EventSource source, KSNotification::EventType event);
 
         /**
          * @brief activateModule Switch tab to specific module name (i.e. CCD) and raise Ekos screen to focus.
@@ -353,6 +334,7 @@ class Manager : public QDialog, public Ui::Manager
 
         void newLog(const QString &text);
         void newModule(const QString &name);
+        void newDevice(const QString &name, int interface);
 
     protected:
         void closeEvent(QCloseEvent *event) override;
@@ -379,9 +361,10 @@ class Manager : public QDialog, public Ui::Manager
         void processINDI();
         void cleanDevices(bool stopDrivers = true);
 
-        void processNewDevice(ISD::GDInterface *);
+        void processNewDevice(const QSharedPointer<ISD::GenericDevice> &device);
         void processNewProperty(INDI::Property);
         void processDeleteProperty(const QString &name);
+        void setDeviceReady();
 
         void processNewNumber(INumberVectorProperty *nvp);
         void processNewText(ITextVectorProperty *tvp);
@@ -410,7 +393,7 @@ class Manager : public QDialog, public Ui::Manager
         void setClientFailed(const QString &host, int port, const QString &message);
         void setClientTerminated(const QString &host, int port, const QString &message);
 
-        void removeDevice(ISD::GDInterface *);
+        void removeDevice(const QSharedPointer<ISD::GenericDevice> &device);
 
         void deviceConnected();
         void deviceDisconnected();
@@ -422,15 +405,24 @@ class Manager : public QDialog, public Ui::Manager
         void updateDebugInterfaces();
         void watchDebugProperty(ISwitchVectorProperty *svp);
 
-        void setTelescope(ISD::GDInterface *);
-        void setCCD(ISD::GDInterface *);
-        void setFilter(ISD::GDInterface *);
-        void setFocuser(ISD::GDInterface *);
-        void setDome(ISD::GDInterface *);
-        void setWeather(ISD::GDInterface *);
-        void setDustCap(ISD::GDInterface *);
-        void setLightBox(ISD::GDInterface *);
-        void setST4(ISD::ST4 *);
+        void addMount(ISD::Mount *device);
+        void addCamera(ISD::Camera *device);
+        void addFilterWheel(ISD::FilterWheel *device);
+        void addFocuser(ISD::Focuser *device);
+        void addRotator(ISD::Rotator *device);
+        void addDome(ISD::Dome *device);
+        void addWeather(ISD::Weather *device);
+        void addDustCap(ISD::DustCap *device);
+        void addLightBox(ISD::LightBox *device);
+        void addGuider(ISD::Guider *device);
+        void addGPS(ISD::GPS *device);
+
+        /**
+         * @brief syncGenericDevice Check if this device needs to be added to any Ekos module.
+         * @param device pointer to generic device.
+         */
+        void syncGenericDevice(const QSharedPointer<ISD::GenericDevice> &device);
+        void createModules(const QSharedPointer<ISD::GenericDevice> &device);
 
         // Profiles
         void addProfile();
@@ -439,9 +431,9 @@ class Manager : public QDialog, public Ui::Manager
         void wizardProfile();
 
         // Mount Summary
-        void updateMountCoords(const SkyPoint position, ISD::Telescope::PierSide pierSide, const dms &ha);
-        void updateMountStatus(ISD::Telescope::Status status);
-        void setTarget(SkyObject *o);
+        void updateMountCoords(const SkyPoint position, ISD::Mount::PierSide pierSide, const dms &ha);
+        void updateMountStatus(ISD::Mount::Status status);
+        void setTarget(QString name);
 
         // Capture Summary
         void updateCaptureStatus(CaptureState status);
@@ -468,10 +460,7 @@ class Manager : public QDialog, public Ui::Manager
         void initGuide();
         void initAlign();
         void initMount();
-        void initDome();
-        void initWeather();
-        void initObservatory(Weather *weather, Dome *dome);
-        void initDustCap();
+        void initObservatory();
 
         void loadDrivers();
         void loadProfiles();
@@ -483,23 +472,20 @@ class Manager : public QDialog, public Ui::Manager
          */
         void syncActiveDevices();
 
+        /**
+         * @brief isINDIReady Check whether all INDI devices are connected and ready and emit signals accordingly.
+         * @return True if all INDI devices are connected and ready.
+         */
+        bool isINDIReady();
+
         // Connect Signals/Slots of Ekos modules
         void connectModules();
 
         // Check if INDI server is already running
         bool isRunning(const QString &process);
 
-        // Find List of devices of specific family type
-        QList<ISD::GDInterface *> findDevices(DeviceFamily type);
-        // Find list of devices by device interface
-        QList<ISD::GDInterface *> findDevicesByInterface(uint32_t interface);
-        // Get all detected devices
-        const QList<ISD::GDInterface *> &getAllDevices() const;
-
-        ProfileInfo *getCurrentProfile();
-        void getCurrentProfileTelescopeInfo(double &primaryFocalLength, double &primaryAperture, double &guideFocalLength,
-                                            double &guideAperture);
-        void updateProfileLocation(ProfileInfo *pi);
+        bool getCurrentProfile(QSharedPointer<ProfileInfo> &profile) const;
+        void updateProfileLocation(const QSharedPointer<ProfileInfo> &profile);
         void setProfileMapping(const QJsonObject &payload)
         {
             m_ProfileMapping = payload;
@@ -511,9 +497,6 @@ class Manager : public QDialog, public Ui::Manager
         // so we only need to start a single instance to handle them all.
         bool checkUniqueBinaryDriver(DriverInfo * primaryDriver, DriverInfo * secondaryDriver);
 
-        bool useGuideHead { false };
-        bool useST4 { false };
-
         // Containers
 
         // All Drivers
@@ -522,15 +505,6 @@ class Manager : public QDialog, public Ui::Manager
         // All managed drivers
         QList<DriverInfo *> managedDrivers;
 
-        // All generic devices (i.e. those define by INDI server)
-        QList<ISD::GDInterface *> genericDevices;
-
-        // All proxy devices (generated devices by Ekos Manager for specific interfaces)
-        QList<ISD::GDInterface *> proxyDevices;
-
-        // All Managed devices (ie. those explicitly defined in the profile)
-        QMultiMap<DeviceFamily, ISD::GDInterface *> managedDevices;
-
         // Smart pointers for the various Ekos Modules
         std::unique_ptr<Capture> captureProcess;
         std::unique_ptr<Focus> focusProcess;
@@ -540,16 +514,13 @@ class Manager : public QDialog, public Ui::Manager
         std::unique_ptr<Analyze> analyzeProcess;
         std::unique_ptr<Scheduler> schedulerProcess;
         std::unique_ptr<Observatory> observatoryProcess;
-        std::unique_ptr<Dome> domeProcess;
-        std::unique_ptr<Weather> weatherProcess;
-        std::unique_ptr<DustCap> dustCapProcess;
         std::unique_ptr<EkosLive::Client> ekosLiveClient;
 
         bool m_LocalMode { true };
         bool m_isStarted { false };
         bool m_RemoteManagerStart { false };
 
-        int nDevices { 0 };
+        int m_DriverDevicesCount { 0 };
 
         QStringList m_LogText;
         KPageWidgetItem *ekosOptionsWidget { nullptr };
@@ -561,14 +532,13 @@ class Manager : public QDialog, public Ui::Manager
         CommunicationStatus m_settleStatus { Ekos::Idle };
 
         std::unique_ptr<QStandardItemModel> profileModel;
-        QList<std::shared_ptr<ProfileInfo>> profiles;
+        QList<QSharedPointer<ProfileInfo>> profiles;
         QJsonObject m_ProfileMapping;
 
-        // Filter Manager
-        QSharedPointer<FilterManager> filterManager;
-
         // Mount Summary
         QPointer<QProcess> indiHubAgent;
+        KLed *mountMotionState { nullptr };
+
 
         // Capture Summary
         QTimer m_CountdownTimer;
@@ -576,8 +546,9 @@ class Manager : public QDialog, public Ui::Manager
         // Preview Frame
         QSharedPointer<SummaryFITSView> m_SummaryView;
 
-        ProfileInfo *currentProfile { nullptr };
+        QSharedPointer<ProfileInfo> m_CurrentProfile;
         bool profileWizardLaunched { false };
+        QString m_PrimaryCamera, m_GuideCamera;
 
         // Port Selector
         std::unique_ptr<Selector::Dialog> m_PortSelector;
