@@ -13,7 +13,8 @@
 
 #include "indi/indistd.h"
 #include "indi/indifocuser.h"
-#include "indi/inditelescope.h"
+#include "indi/indimount.h"
+#include "ekos/manager/meridianflipstate.h"
 #include "ekos/align/polaralignmentassistant.h"
 
 class QQuickView;
@@ -28,11 +29,14 @@ namespace Ekos
  *@version 1.5
  */
 
+class OpticalTrainManager;
+
 class Mount : public QWidget, public Ui::Mount
 {
         Q_OBJECT
         Q_CLASSINFO("D-Bus Interface", "org.kde.kstars.Ekos.Mount")
-        Q_PROPERTY(ISD::Telescope::Status status READ status NOTIFY newStatus)
+        Q_PROPERTY(QString opticalTrain READ opticalTrain WRITE setOpticalTrain)
+        Q_PROPERTY(ISD::Mount::Status status READ status NOTIFY newStatus)
         Q_PROPERTY(ISD::ParkStatus parkStatus READ parkStatus NOTIFY newParkStatus)
         Q_PROPERTY(QStringList logText READ logText NOTIFY newLog)
         Q_PROPERTY(QList<double> altitudeLimits READ altitudeLimits WRITE setAltitudeLimits)
@@ -42,12 +46,11 @@ class Mount : public QWidget, public Ui::Mount
         Q_PROPERTY(bool autoParkEnabled READ autoParkEnabled WRITE setAutoParkEnabled)
         Q_PROPERTY(QList<double> equatorialCoords READ equatorialCoords)
         Q_PROPERTY(QList<double> horizontalCoords READ horizontalCoords)
-        Q_PROPERTY(QList<double> telescopeInfo READ telescopeInfo WRITE setTelescopeInfo)
         Q_PROPERTY(double hourAngle READ hourAngle)
         Q_PROPERTY(int slewRate READ slewRate WRITE setSlewRate)
         Q_PROPERTY(int slewStatus READ slewStatus)
         Q_PROPERTY(bool canPark READ canPark)
-        Q_PROPERTY(ISD::Telescope::PierSide pierSide READ pierSide NOTIFY pierSideChanged)
+        Q_PROPERTY(ISD::Mount::PierSide pierSide READ pierSide NOTIFY pierSideChanged)
 
     public:
         Mount();
@@ -55,35 +58,32 @@ class Mount : public QWidget, public Ui::Mount
 
         //typedef enum { PARKING_IDLE, PARKING_OK, UNPARKING_OK, PARKING_BUSY, UNPARKING_BUSY, PARKING_ERROR } ParkingStatus;
 
-        // This enum defines the meridian flip state machine, this is implemented in
-        typedef enum
-        {
-            FLIP_NONE,      // this is the default state, comparing the hour angle with the next flip position
-            // it moves to FLIP_PLANNED when a flip is needed
-            FLIP_PLANNED,   // a meridian flip is ready to be started due to the target position and the
-            // configured offsets and signals to the Capture class that a flip is required
-            FLIP_WAITING,   // step after FLUP_PLANNED waiting until Capture completes a running exposure
-            FLIP_ACCEPTED,  // Capture is idle or has completed the exposure and will wait until the flip
-            // is completed.
-            FLIP_RUNNING,   // this signals that a flip slew is in progress, when the slew is completed
-            // the state is set to FLIP_COMPLETED
-            FLIP_COMPLETED, // this checks that the flip was completed successfully or not and after tidying up
-            // moves to FLIP_NONE to wait for the next flip requirement.
-            // Capture sees this and resumes.
-            FLIP_ERROR,     // errors in the flip process should end up here
-            FLIP_INACTIVE   // do not execute a meridian flip since it will disturb other activities like
-            // a running polar alignment
-        } MeridianFlipStatus;
+        /**
+             * @brief addMount Add a new Mount device
+             * @param device pointer to mount device
+             * @return True if added successfully, false if duplicate or failed to add.
+             */
+        bool setMount(ISD::Mount *device);
 
         /**
-             * @brief setTelescope Sets the mount module telescope interface
-             * @param newTelescope pointer to telescope interface object
+             * @brief addGPS Add a new GPS device
+             * @param device pointer to gps device
+             * @return True if added successfully, false if duplicate or failed to add.
              */
-        void setTelescope(ISD::GDInterface *newTelescope);
+        bool addGPS(ISD::GPS *device);
 
-        void setGPS(ISD::GDInterface *newGPS);
+        void removeDevice(const QSharedPointer<ISD::GenericDevice> &device);
 
-        void removeDevice(ISD::GDInterface *device);
+        void setupOpticalTrainManager();
+        void refreshOpticalTrain();
+        QString opticalTrain() const
+        {
+            return opticalTrainCombo->currentText();
+        }
+        void setOpticalTrain(const QString &value)
+        {
+            opticalTrainCombo->setCurrentText(value);
+        }
 
         // Log functions
         void appendLogText(const QString &);
@@ -97,33 +97,38 @@ class Mount : public QWidget, public Ui::Mount
             return m_LogText.join("\n");
         }
 
-        ISD::Telescope::Status status() const
+        ISD::Mount::Status status() const
         {
             return m_Status;
         }
-        QString statusString(bool translated) const
+        ISD::Mount *activeMount() const
         {
-            if (currentTelescope)
-                return currentTelescope->statusString(m_Status, translated);
+            return m_Mount;
+        }
+        QString statusString(bool translated = true) const
+        {
+            if (m_Mount)
+                return m_Mount->statusString(m_Status, translated);
             else
                 return "NA";
         }
-        ISD::Telescope::PierSide pierSide() const
+        ISD::Mount::PierSide pierSide() const
         {
-            if (currentTelescope)
-                return currentTelescope->pierSide();
+            if (m_Mount)
+                return m_Mount->pierSide();
             else
-                return ISD::Telescope::PIER_UNKNOWN;
+                return ISD::Mount::PIER_UNKNOWN;
         }
         ISD::ParkStatus parkStatus() const
         {
             return m_ParkStatus;
         }
 
-        MeridianFlipStatus meridianFlipStatus() const
-        {
-            return m_MFStatus;
-        }
+        /**
+         * @brief getMeridianFlipState
+         * @return
+         */
+        QSharedPointer<MeridianFlipState> getMeridianFlipState() const { return mf_state; }
 
         /** @defgroup MountDBusInterface Ekos Mount DBus Interface
              * Mount interface provides advanced scripting capabilities to control INDI mounts.
@@ -265,7 +270,6 @@ class Mount : public QWidget, public Ui::Mount
              */
         Q_SCRIPTABLE double hourAngle();
 
-        double initialPositionHA;
         /** DBUS interface function.
              * Get the hour angle of that time the mount has slewed to the current position.
              * This is used to manage the meridian flip for mounts which do not report pier side.
@@ -273,12 +277,12 @@ class Mount : public QWidget, public Ui::Mount
              */
         Q_SCRIPTABLE double initialHA()
         {
-            return initialPositionHA;
+            return mf_state->initialPositionHA();
         }
 
-        Q_SCRIPTABLE void setInitialHA(double ha)
+        Q_SCRIPTABLE void setInitialHA(double RA)
         {
-            initialPositionHA = ha;
+            mf_state->setInitialPositionHA(RA);
         }
 
         /** DBUS interface function.
@@ -300,22 +304,6 @@ class Mount : public QWidget, public Ui::Mount
 
         Q_INVOKABLE Q_SCRIPTABLE bool setSlewRate(int index);
 
-        /** DBUS interface function.
-             * Get telescope and guide scope info. An array of doubles is returned in order.
-             * Primary Telescope Focal Length (mm), Primary Telescope Aperture (mm), Guide Telescope Focal Length (mm), Guide Telescope Aperture (mm)
-             */
-        Q_INVOKABLE Q_SCRIPTABLE QList<double> telescopeInfo();
-
-        /** DBUS interface function.
-             * Set telescope and guide scope info and save them in INDI mount driver. All measurements is in millimeters.
-             * @param info An ordered 4-item list as following:
-             * primaryFocalLength Primary Telescope Focal Length. Set to 0 to skip setting this value.
-             * primaryAperture Primary Telescope Aperture. Set to 0 to skip setting this value.
-             * guideFocalLength Guide Telescope Focal Length. Set to 0 to skip setting this value.
-             * guideAperture Guide Telescope Aperture. Set to 0 to skip setting this value.
-             */
-        Q_SCRIPTABLE Q_NOREPLY void setTelescopeInfo(const QList<double> &info);
-
         /** DBUS interface function.
              * Reset mount model if supported by the mount.
              * @return true if the command is executed successfully, false otherwise.
@@ -375,22 +363,19 @@ class Mount : public QWidget, public Ui::Mount
          * @brief Execute a meridian flip if necessary.
          * @return true if a meridian flip was necessary
          */
-        Q_INVOKABLE bool executeMeridianFlip();
+        Q_INVOKABLE bool startMeridianFlip();
 
         Q_INVOKABLE void setUpDownReversed(bool enabled);
         Q_INVOKABLE void setLeftRightReversed(bool enabled);
 
         QString meridianFlipStatusDescription()
         {
-            return meridianFlipStatusText->text();
+            return meridianFlipStatusWidget->getStatus();
         }
 
-        ///
-        /// \brief meridianFlipStatusString
-        /// \param status
-        /// \return return the string for the status
-        ///
-        static QString meridianFlipStatusString(MeridianFlipStatus status);
+        // Settings
+        QVariantMap getAllSettings() const;
+        void setAllSettings(const QVariantMap &settings);
 
     public slots:
 
@@ -410,13 +395,6 @@ class Mount : public QWidget, public Ui::Mount
              */
         void updateSwitch(ISwitchVectorProperty *svp);
 
-
-        /**
-             * @brief updateText Update text properties under watch in the mount module
-             * @param tvp pointer to text property
-             */
-        void updateText(ITextVectorProperty *tvp);
-
         /**
              * @brief updateLog Update mount module log to include any messages arriving for the telescope driver
              * @param messageID ID of the new message
@@ -424,20 +402,20 @@ class Mount : public QWidget, public Ui::Mount
         void updateLog(int messageID);
 
         /**
-             * @brief updateTelescopeCoords is triggered by the ISD::Telescope::newCoord() event and updates the displayed
+             * @brief updateTelescopeCoords is triggered by the ISD::Mount::newCoord() event and updates the displayed
              * coordinates of the mount and to ensure mount is within altitude limits if the altitude limits are enabled.
              * The frequency of this update depends on the REFRESH parameter of the INDI mount device.
              * @param position latest coordinates the mount reports it is pointing to
              * @param pierSide pierSide
              * @param ha hour angle of the latest coordinates
              */
-        void updateTelescopeCoords(const SkyPoint &position, ISD::Telescope::PierSide pierSide, const dms &ha);
+        void updateTelescopeCoords(const SkyPoint &position, ISD::Mount::PierSide pierSide, const dms &ha);
 
         /**
              * @brief move Issues motion command to the mount to move in a particular direction based the request NS and WE values
-             * @param command Either ISD::Telescope::MOTION_START (0) or ISD::Telescope::MOTION_STOP (1)
-             * @param NS is either -1 for no direction, or ISD::Telescope::MOTION_NORTH (0), or ISD::Telescope::MOTION_SOUTH (1)
-             * @param WE is either -1 for no direction, or ISD::Telescope::MOTION_WEST (0), or ISD::Telescope::MOTION_EAST (1)
+             * @param command Either ISD::Mount::MOTION_START (0) or ISD::Mount::MOTION_STOP (1)
+             * @param NS is either -1 for no direction, or ISD::Mount::MOTION_NORTH (0), or ISD::Mount::MOTION_SOUTH (1)
+             * @param WE is either -1 for no direction, or ISD::Mount::MOTION_WEST (0), or ISD::Mount::MOTION_EAST (1)
              */
         void motionCommand(int command, int NS, int WE);
 
@@ -450,21 +428,16 @@ class Mount : public QWidget, public Ui::Mount
          */
         void doPulse(GuideDirection ra_dir, int ra_msecs, GuideDirection dec_dir, int dec_msecs);
 
-        /**
-             * @brief save Save telescope focal length and aperture in the INDI telescope driver configuration.
-             */
-        void save();
-
         /**
              * @brief saveLimits Saves altitude limit to the user options and updates the INDI telescope driver limits
              */
         void saveLimits();
 
         /**
-             * @brief enableAltitudeLimits Enable or disable altitude limits
+             * @brief Enable or disable altitude limits
              * @param enable True to enable, false to disable.
              */
-        void enableAltitudeLimits(bool enable);
+        void setAltitudeLimits(bool enable);
 
         /**
              * @brief enableAltLimits calls enableAltitudeLimits(true). This function is mostly used to enable altitude limit after a meridian flip is complete.
@@ -492,18 +465,14 @@ class Mount : public QWidget, public Ui::Mount
              */
         void disableHaLimits();
 
-        bool setScopeConfig(int index);
-
         void toggleMountToolBox();
 
-        void meridianFlipStatusChanged(MeridianFlipStatus status);
-
         /**
          * @brief set meridian flip activation and hours
          * @param activate true iff the meridian flip should be executed
-         * @param hours angle past the meridian when the flip should be delayed
+         * @param degrees angle past the meridian when the flip should be delayed
          */
-        void setMeridianFlipValues(bool activate, double hours);
+        void setMeridianFlipValues(bool activate, double degrees);
 
         /**
          * @brief React upon status changes of the polar alignment - mainly to
@@ -539,7 +508,7 @@ class Mount : public QWidget, public Ui::Mount
          */
         void stopTimers();
 
-    private slots:
+private slots:
         void startParkTimer();
         void stopParkTimer();
         void startAutoPark();
@@ -556,7 +525,7 @@ class Mount : public QWidget, public Ui::Mount
          * the mount is on the western side of the pier pointing east of the meridian).
          * @param ha current hour angle
          */
-        void newCoords(const SkyPoint &position, ISD::Telescope::PierSide pierSide, const dms &ha);
+        void newCoords(const SkyPoint &position, ISD::Mount::PierSide pierSide, const dms &ha);
         /**
          * @brief The mount has finished the slew to a new target.
          * @param currentCoords exact position where the mount is positioned
@@ -571,36 +540,65 @@ class Mount : public QWidget, public Ui::Mount
         /**
          * @brief Change in the mount status.
          */
-        void newStatus(ISD::Telescope::Status status);
+        void newStatus(ISD::Mount::Status status);
         void newParkStatus(ISD::ParkStatus status);
-        void pierSideChanged(ISD::Telescope::PierSide side);
+        void pierSideChanged(ISD::Mount::PierSide side);
         void slewRateChanged(int index);
         void ready();
-        void newMeridianFlipStatus(MeridianFlipStatus status);
         void newMeridianFlipText(const QString &text);
 
+        void settingsUpdated(const QVariantMap &settings);
+
+        void trainChanged();
+
     private:
+        ////////////////////////////////////////////////////////////////////
+        /// Settings
+        ////////////////////////////////////////////////////////////////////
+
+        /**
+         * @brief Connect GUI elements to sync settings once updated.
+         */
+        void connectSettings();
+        /**
+         * @brief Stop updating settings when GUI elements are updated.
+         */
+        void disconnectSettings();
+        /**
+         * @brief loadSettings Load setting from Options and set them accordingly.
+         */
+        void loadGlobalSettings();
+
+        /**
+         * @brief syncSettings When checkboxes, comboboxes, or spin boxes are updated, save their values in the
+         * global and per-train settings.
+         */
+        void syncSettings();
+
+        /**
+         * @brief syncControl Sync setting to widget. The value depends on the widget type.
+         * @param settings Map of all settings
+         * @param key name of widget to sync
+         * @param widget pointer of widget to set
+         * @return True if sync successful, false otherwise
+         */
+        bool syncControl(const QVariantMap &settings, const QString &key, QWidget * widget);
+
         void syncGPS();
-        void setScopeStatus(ISD::Telescope::Status status);
-        MeridianFlipStatus m_MFStatus = FLIP_NONE;
-        void setMeridianFlipStatus(MeridianFlipStatus status);
-        void meridianFlipStatusChangedInternal(MeridianFlipStatus status);
+        void setScopeStatus(ISD::Mount::Status status);
+        /* Meridian flip state handling */
+        QSharedPointer<MeridianFlipState> mf_state;
         QString pierSideStateString();
-
-        // A meridian flip requires a slew of 180 degrees in the hour angle axis so will take at least
-        // the time for that, currently set to 20 seconds
-        // not reliable for pointing state change detection but reported if the pier side is unknown
-        QDateTime minMeridianFlipEndTime;
-        int minMeridianFlipDurationSecs = 20;
-
-        double flipDelayHrs = 0.0;      // delays the next flip attempt if it fails
+        void setupParkUI();
 
         QPointer<QDBusInterface> captureInterface { nullptr };
 
-        ISD::Telescope *currentTelescope = nullptr;
-        ISD::GDInterface *currentGPS = nullptr;
+        ISD::Mount *m_Mount {nullptr};
+        ISD::GPS *m_GPS {nullptr};
+        QList<ISD::GPS*> m_GPSes;
+
         QStringList m_LogText;
-        SkyPoint *currentTargetPosition = nullptr;
+        SkyPoint *currentTargetPosition {nullptr};
         SkyPoint telescopeCoord;
         QString lastNotificationMessage;
 
@@ -617,9 +615,13 @@ class Mount : public QWidget, public Ui::Mount
         // GPS
         bool GPSInitialized = {false};
 
-        ISD::Telescope::Status m_Status = ISD::Telescope::MOUNT_IDLE;
+        ISD::Mount::Status m_Status = ISD::Mount::MOUNT_IDLE;
         ISD::ParkStatus m_ParkStatus = ISD::PARK_UNKNOWN;
 
+        // Settings
+        QVariantMap m_Settings;
+        QVariantMap m_GlobalSettings;
+
         QQuickView *m_BaseView = nullptr;
         QQuickItem *m_BaseObj  = nullptr;
         QQmlContext *m_Ctxt    = nullptr;
