@@ -534,6 +534,13 @@ class Scheduler : public QWidget, public Ui::Scheduler
          */
         void registerNewModule(const QString &name);
 
+        /**
+           * @brief registerNewDevice register interfaces associated with devices
+           * @param name Device name
+           * @param interface Device driver interface
+           */
+        void registerNewDevice(const QString &name, int interface);
+
         /**
          * @brief syncProperties Sync startup properties from the various device to enable/disable features in the scheduler
          * like the ability to park/unpark..etc
@@ -552,7 +559,7 @@ class Scheduler : public QWidget, public Ui::Scheduler
         void setGuideStatus(Ekos::GuideState status);
         void setCaptureStatus(Ekos::CaptureState status);
         void setFocusStatus(Ekos::FocusState status);
-        void setMountStatus(ISD::Telescope::Status status);
+        void setMountStatus(ISD::Mount::Status status);
         void setWeatherStatus(ISD::Weather::Status status);
 
         /**
@@ -978,6 +985,7 @@ class Scheduler : public QWidget, public Ui::Scheduler
 
         Ekos::Scheduler *ui { nullptr };
         //DBus interfaces
+        QPointer<QDBusInterface> indiInterface { nullptr };
         QPointer<QDBusInterface> focusInterface { nullptr };
         QPointer<QDBusInterface> ekosInterface { nullptr };
         QPointer<QDBusInterface> captureInterface { nullptr };
@@ -1051,34 +1059,47 @@ class Scheduler : public QWidget, public Ui::Scheduler
             guidePathString = interface;
         }
 
-        QString domeInterfaceString { "org.kde.kstars.Ekos.Dome" };
+        QString INDIInterfaceString { "org.kde.kstars.INDI" };
+        void setINDIInterfaceString(const QString &interface)
+        {
+            INDIInterfaceString = interface;
+        }
+
+        QString INDIPathString {"/KStars/INDI"};
+        void setINDIPathString(const QString &interface)
+        {
+            INDIPathString = interface;
+        }
+
+        QString domeInterfaceString { "org.kde.kstars.INDI.Dome" };
         void setDomeInterfaceString(const QString &interface)
         {
             domeInterfaceString = interface;
         }
-        QString domePathString { "/KStars/Ekos/Dome" };
+
+        QString domePathString;
         void setDomePathString(const QString &interface)
         {
             domePathString = interface;
         }
 
-        QString weatherInterfaceString { "org.kde.kstars.Ekos.Weather" };
+        QString weatherInterfaceString { "org.kde.kstars.INDI.Weather" };
         void setWeatherInterfaceString(const QString &interface)
         {
             weatherInterfaceString = interface;
         }
-        QString weatherPathString { "/KStars/Ekos/Weather" };
+        QString weatherPathString;
         void setWeatherPathString(const QString &interface)
         {
             weatherPathString = interface;
         }
 
-        QString dustCapInterfaceString { "org.kde.kstars.Ekos.DustCap" };
+        QString dustCapInterfaceString { "org.kde.kstars.INDI.DustCap" };
         void setDustCapInterfaceString(const QString &interface)
         {
             dustCapInterfaceString = interface;
         }
-        QString dustCapPathString { "/KStars/Ekos/DustCap" };
+        QString dustCapPathString;
         void setDustCapPathString(const QString &interface)
         {
             dustCapPathString = interface;
