@@ -328,7 +328,29 @@ void ClientManager::establishConnection()
     m_PendingConnection = true;
     m_ConnectionRetries = 2;
 
-    connectServer();
+    if (connectServer() == false)
+    {
+        if (m_PendingConnection)
+        {
+            // Should we retry again?
+            if (m_ConnectionRetries-- > 0)
+            {
+                // Connect again in 1 second.
+                QTimer::singleShot(1000, this, [this]()
+                {
+                    qCDebug(KSTARS_INDI) << "Retrying connection again";
+                    connectServer();
+                });
+            }
+            // Nope cannot connect to server.
+            else
+            {
+                m_PendingConnection = false;
+                m_ConnectionRetries = MAX_RETRIES;
+                emit failed(i18n("Failed to connect to INDI server %1:%2", getHost(), getPort()));
+            }
+        }
+    }
 }
 
 DriverInfo *ClientManager::findDriverInfoByName(const QString &name)
