@@ -68,19 +68,27 @@ void DriverManagerUI::makePortEditable(QTreeWidgetItem *selectedItem, int column
 }
 
 DriverManager *DriverManager::_DriverManager = nullptr;
+INDIDBus *DriverManager::_INDIDBus = nullptr;
 
 DriverManager *DriverManager::Instance()
 {
     if (_DriverManager == nullptr)
     {
-        _DriverManager     = new DriverManager(KStars::Instance());
-        INDIDBus *indiDBUS = new INDIDBus(KStars::Instance());
-        Q_UNUSED(indiDBUS)
+        _DriverManager = new DriverManager(KStars::Instance());
+        _INDIDBus = new INDIDBus(KStars::Instance());
     }
 
     return _DriverManager;
 }
 
+void DriverManager::release()
+{
+    delete (_DriverManager);
+    delete (_INDIDBus);
+    _DriverManager = nullptr;
+    _INDIDBus = nullptr;
+}
+
 DriverManager::DriverManager(QWidget *parent) : QDialog(parent)
 {
 #ifdef Q_OS_OSX
@@ -359,6 +367,7 @@ void DriverManager::startDevices(QList<DriverInfo *> &dList)
             return;
         }
 
+        servers.append(serverManager);
         serverManager->setPendingDrivers(qdv);
         serverManager->setMode(connectionMode);
 
@@ -422,7 +431,7 @@ void DriverManager::startClientManager(const QList<DriverInfo *> &qdv, const QSt
 {
     auto clientManager = new ClientManager();
 
-    for (DriverInfo *dv : qdv)
+    for (auto &dv : qdv)
         clientManager->appendManagedDriver(dv);
 
     connect(clientManager, &ClientManager::started, this, &DriverManager::setClientStarted, Qt::UniqueConnection);
@@ -629,11 +638,14 @@ void DriverManager::setClientFailed(const QString &message)
     GUIManager::Instance()->removeClient(client);
     INDIListener::Instance()->removeClient(client);
 
-    KSNotification::event(QLatin1String("IndiServerMessage"), message);
+    KSNotification::event(QLatin1String("IndiServerMessage"), message, KSNotification::INDI);
 
     clients.removeOne(client);
     client->deleteLater();
 
+    if (connectionMode == SERVER_CLIENT)
+        activateStopService();
+
     updateMenuActions();
     updateLocalTab();
 }
@@ -655,7 +667,7 @@ void DriverManager::setClientTerminated(const QString &message)
     GUIManager::Instance()->removeClient(client);
     INDIListener::Instance()->removeClient(client);
 
-    KSNotification::event(QLatin1String("IndiServerMessage"), message, KSNotification::EVENT_ALERT);
+    KSNotification::event(QLatin1String("IndiServerMessage"), message, KSNotification::INDI, KSNotification::Alert);
 
     clients.removeOne(client);
     client->deleteLater();
@@ -785,8 +797,7 @@ void DriverManager::setClientStarted()
     updateMenuActions();
 
     KSNotification::event(QLatin1String("ConnectionSuccessful"),
-                          i18n("Connected to INDI server"),
-                          KSNotification::EVENT_INFO);
+                          i18n("Connected to INDI server"));
 
     emit clientStarted(clientManager->getHost(), clientManager->getPort());
 }
@@ -966,14 +977,6 @@ bool DriverManager::readXMLDrivers()
     QDir indiDir;
     QString driverName;
 
-    // This is the XML file shipped with KStars that contains all supported INDI drivers.
-    /*QString indiDriversXML = KSPaths::locate(QStandardPaths::AppLocalDataLocation, "indidrivers.xml");
-    if (indiDriversXML.isEmpty() == false)
-        processXMLDriver(indiDriversXML);
-    */
-
-    processXMLDriver(QLatin1String(":/indidrivers.xml"));
-
     QString driversDir = Options::indiDriversDir();
 #ifdef Q_OS_OSX
     if (Options::indiDriversAreInternal())
@@ -997,26 +1000,15 @@ bool DriverManager::readXMLDrivers()
 
     for (auto &fileInfo : list)
     {
-        // libindi 0.7.1: Skip skeleton files
         if (fileInfo.fileName().endsWith(QLatin1String("_sk.xml")))
             continue;
 
-        //        if (fileInfo.fileName() == "drivers.xml")
-        //        {
-        //            // Let first attempt to load the local version of drivers.xml
-        //            driverName = QDir(KSPaths::writableLocation(QStandardPaths::AppLocalDataLocation)).filePath("drivers.xml");
-
-        //            // If found, we continue, otherwise, we load the system file
-        //            if (driverName.isEmpty() == false && QFile(driverName).exists())
-        //            {
-        //                processXMLDriver(driverName);
-        //                continue;
-        //            }
-        //        }
-
         processXMLDriver(fileInfo.absoluteFilePath());
     }
 
+    // JM 2022.08.24: Process local source last as INDI sources should have higher priority than KStars own database.
+    processXMLDriver(QLatin1String(":/indidrivers.xml"));
+
     return true;
 }
 
