@@ -47,6 +47,7 @@
 #include "skymap.h"
 #include "hipscomponent.h"
 #include "terraincomponent.h"
+#include "mosaiccomponent.h"
 #endif
 
 #include <QApplication>
@@ -155,6 +156,11 @@ SkyMapComposite::SkyMapComposite(SkyComposite *parent)
 
     addComponent(m_Terrain = new TerrainComponent(this));
 
+    // Mosaic Component
+#ifdef HAVE_INDI
+    addComponent(m_Mosaic = new MosaicComponent(this));
+#endif
+
     addComponent(m_ArtificialHorizon = new ArtificialHorizonComponent(this), 110);
 
     addComponent(m_SolarSystem = new SolarSystemComposite(this), 2);
@@ -353,6 +359,10 @@ void SkyMapComposite::draw(SkyPainter *skyp)
         QPen(QColor(data->colorScheme()->colorNamed("StarHopRouteColor")), 1.);
     m_StarHopRouteList->draw(skyp);
 
+#ifdef HAVE_INDI
+    m_Mosaic->draw(skyp);
+#endif
+
     m_ArtificialHorizon->draw(skyp);
 
     m_Horizon->draw(skyp);
