@@ -709,7 +709,7 @@ void SkyMap::slotEndRulerMode()
                                     ((f->sizeX() >= f->sizeY() && f->sizeY() != 0) ? f->sizeY() : f->sizeX()));
             }
             fov = nameToFovMap[QInputDialog::getItem(this, i18n("Star Hopper: Choose a field-of-view"),
-                                                           i18n("FOV to use for star hopping:"), nameToFovMap.uniqueKeys(), 0,
+                                                           i18n("FOV to use for star hopping:"), nameToFovMap.keys(), 0,
                                                            false, &ok)];
         }
         else
@@ -1080,7 +1080,7 @@ void SkyMap::slewFocus()
             while (r > step)
             {
                 //DEBUG
-                //qDebug() << step << ": " << r << ": " << r0;
+                //qDebug() << Q_FUNC_INFO << step << ": " << r << ": " << r0;
                 double fX = dX / r;
                 double fY = dY / r;
 
