@@ -22,6 +22,7 @@
 #include <basedevice.h>
 #include "indi/indilistener.h"
 #include "indi/indistd.h"
+#include "indi/indimount.h"
 #include "indi/driverinfo.h"
 #endif
 
@@ -261,13 +262,13 @@ void FlagManager::slotCenterTelescope()
 
     if (INDIListener::Instance()->size() == 0)
     {
-        KSNotification::sorry(i18n("KStars did not find any active telescopes."));
+        KSNotification::sorry(i18n("No connected mounts found."));
         return;
     }
 
-    for (auto oneDevice : INDIListener::Instance()->getDevices())
+    for (auto oneDevice : INDIListener::devices())
     {
-        if (oneDevice->getType() != KSTARS_TELESCOPE)
+        if (!(oneDevice->getDriverInterface() & INDI::BaseDevice::TELESCOPE_INTERFACE))
             continue;
 
         if (oneDevice->isConnected() == false)
@@ -277,16 +278,16 @@ void FlagManager::slotCenterTelescope()
             return;
         }
 
-        ISD::GDSetCommand SlewCMD(INDI_SWITCH, "ON_COORD_SET", "TRACK", ISS_ON, this);
+        auto mount = oneDevice->getMount();
+        if (!mount)
+            continue;
 
-        oneDevice->setProperty(&SlewCMD);
-        oneDevice->runCommand(INDI_SEND_COORDS,
-                              m_Ks->data()->skyComposite()->flags()->pointList().at(ui->flagList->currentIndex().row()).get());
+        mount->Slew(m_Ks->data()->skyComposite()->flags()->pointList().at(ui->flagList->currentIndex().row()).get());
 
         return;
     }
 
-    KSNotification::sorry(i18n("KStars did not find any active telescopes."));
+    KSNotification::sorry(i18n("No connected mounts found."));
 
 #endif
 }
